@@ -1,7 +1,7 @@
 #ifndef MY_MD5_INCLUDED
 #define MY_MD5_INCLUDED
 
-/* Copyright (c) 2000, 2012, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,8 +24,7 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */
 
 #include "m_string.h"
-
-#define MD5_HASH_SIZE 16 /* Hash size in bytes */
+#include "my_md5_size.h"
 
 /*
   Wrapper function for MD5 implementation.
@@ -52,10 +51,16 @@ static inline void array_to_hex(char *to, const unsigned char *str, uint len)
 }
 
 #ifdef WITH_WSREP
+
+/* For certification we need to identify each row uniquely.
+Generally this is done using PK but if table is created w/o PK
+then a md5-hash (16 bytes) string is generated using the complete record.
+Following functions act as helper function in generation of this md5-hash. */
 void *wsrep_md5_init();
 void wsrep_md5_update(void *ctx, char* buf, int len);
-  void wsrep_compute_md5_hash(char *digest, void *ctx);
-#endif
+void wsrep_compute_md5_hash(char *digest, void *ctx);
+#endif /* WITH_WSREP */
+
 #ifdef __cplusplus
 }
 #endif
