@@ -1,15 +1,21 @@
 # -*- cperl -*-
-# Copyright (c) 2007, 2016, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2017, Oracle and/or its affiliates. All rights reserved.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Library General Public
-# License as published by the Free Software Foundation; version 2
-# of the License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
+#
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms,
+# as designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Library General Public License for more details.
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
@@ -36,8 +42,9 @@ sub add_opt_values {
   my ($self, $config)= @_;
 
   # add auto-options
-  $config->insert('OPT', 'port'   => sub { fix_port($self, $config) });
-  $config->insert('mysqld', "loose-skip-plugin-$_" => undef) for (@::optional_plugins);
+  $config->insert('OPT', 'port'   => fix_port($self, $config) );
+  $config->insert('OPT', 'vardir' => sub { $self->{ARGS}->{vardir} });
+  $config->insert('mysqld', "loose-skip-$_" => undef) for (@::optional_plugins);
 }
 
 my @pre_rules=
@@ -273,6 +280,8 @@ my @mysqld_rules=
  # Galera uses base_port + 1 for IST, so we do not use it for things such as SST
  { '#ist_port' => \&fix_port },
  { '#sst_port' => \&fix_port },
+ # Reserve a port for group_replication
+ { '#group_replication_port' => \&fix_port },
  { 'socket' => \&fix_socket },
  { 'loose-mysqlx-port' => \&fix_x_port },
  { 'loose-mysqlx-socket' => \&fix_x_socket },
@@ -399,6 +408,7 @@ my @mysqltest_rules=
 my @mysqlbinlog_rules=
 (
  { 'character-sets-dir' => \&fix_charset_dir },
+ { 'local-load' => sub { return shift->{ARGS}->{tmpdir}; } },
 );
 
 
@@ -693,8 +703,20 @@ sub new_config {
     croak "you must pass '$required'" unless defined $args->{$required};
   }
 
+  # Fill in hosts/port hash
+  my $hosts= {};
+  my $baseport= $args->{baseport};
+  $args->{hosts}= [ 'localhost' ] unless exists($args->{hosts});
+  foreach my $host ( @{$args->{hosts}} ) {
+     $hosts->{$host}= $baseport;
+  }
+
   # Open the config template
   my $config= My::Config->new($args->{'template_path'});
+  my $extra_template_path= $args->{'extra_template_path'};
+  if ($extra_template_path){
+    $config->append(My::Config->new($extra_template_path));
+  }
   my $self= bless {
 		   CONFIG       => $config,
 		   ARGS         => $args,
