@@ -1,15 +1,22 @@
 # -*- cperl -*-
-# Copyright (c) 2004, 2016, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2004, 2018, Oracle and/or its affiliates. All rights reserved.
 # 
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 of the License.
-# 
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
+#
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms,
+# as designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-# 
+# GNU General Public License, version 2.0, for more details.
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -32,13 +39,16 @@ our @EXPORT= qw(report_option mtr_print_line mtr_print_thick_line
 use mtr_match;
 use File::Spec;
 use My::Platform;
-use POSIX qw[ _exit ];
+use POSIX qw(_exit floor);
 use IO::Handle qw[ flush ];
 require "mtr_io.pl";
 use mtr_results;
 
 my $tot_real_time= 0;
 
+my $done_percentage= 0;
+my $tests_completed= 0;
+
 our $timestamp= 0;
 our $timediff= 0;
 our $name;
@@ -65,18 +75,28 @@ sub _name {
   return $name ? $name." " : undef;
 }
 
-sub _mtr_report_test_name ($) {
+sub _mtr_report_test_name ($$)
+{
   my $tinfo= shift;
+  my $done_percentage= shift;
+
   my $tname= $tinfo->{name};
 
   return unless defined $verbose;
 
   # Add combination name if any
-  $tname.= " '$tinfo->{combination}'"
-    if defined $tinfo->{combination};
+  $tname.= " '$tinfo->{combination}'" if defined $tinfo->{combination};
 
   print _name(). _timestamp();
-  printf "%-40s ", $tname;
+  if($::opt_test_progress)
+  {
+    printf "[%3s%] %-40s ", $done_percentage, $tname ;
+  }
+  else
+  {
+    printf "%-40s ", $tname;
+  }
+
   my $worker = $tinfo->{worker};
   print "w$worker " if defined $worker;
 
@@ -120,7 +140,6 @@ sub mtr_report_test_passed ($) {
 
 sub mtr_report_test ($) {
   my ($tinfo)= @_;
-  my $test_name = _mtr_report_test_name($tinfo);
 
   my $comment=  $tinfo->{'comment'};
   my $logfile=  $tinfo->{'logfile'};
@@ -128,6 +147,18 @@ sub mtr_report_test ($) {
   my $result=   $tinfo->{'result'};
   my $retry=    $tinfo->{'retries'} ? "retry-" : "";
 
+  if ($::opt_test_progress)
+  {
+    if ($tinfo->{'name'} && !$retry)
+    {
+      $tests_completed= $tests_completed + 1;
+      $done_percentage=
+        floor(($tests_completed / $::num_tests_for_report) * 100);
+    }
+  }
+
+  my $test_name = _mtr_report_test_name($tinfo, $done_percentage);
+
   if ($result eq 'MTR_RES_FAILED'){
 
     my $timest = format_time();
@@ -234,98 +265,245 @@ sub mtr_report_test ($) {
 
 
 sub mtr_generate_xml_report($) {
-  my ($tests) = @_;
-  my $tsuite;
-  my $tname;
+  my ($tests)= @_;
   my $suite_group;
 
-  my $all_t = 0;
-  my $all_f = 0;
-  my $all_d = 0;
-  my $all_e = 0;
-  my $all_time = 0;
+  # ------------------------------------
+  # Calculate suitewise/total statistics
+  # ------------------------------------
 
-  my @suite_t;
-  my @suite_f;
-  my @suite_d;
-  my @suite_e;
-  my @suite_time;
+  my %suite_stats;
+  my %total_stats;
+  my @stat_list= ('tests', 'failures', 'disabled', 'skipped',
+                  'errors', 'unstable', 'time');
 
+  # Initialize overall statistics
+  map { $total_stats{$_}= 0 } @stat_list;
 
-  # calculate totals
-  foreach my $tinfo (@$tests) 
+  # Sort in order to group tests suitewise
+  my @sorted_tests= sort { $a->{'name'} cmp $b->{'name'} } @$tests;
+
+  foreach my $tinfo (@sorted_tests)
   {
-    my @parts = split /\./, $tinfo->{'name'};
-    $tsuite = $parts[0];
-    $tname = $parts[1];
+    my ($tsuite, $tname) = split /\./, $tinfo->{'name'};
 
+    # Initialize suitewise statistics for new suites
     if ($tsuite ne $suite_group)
     {
-      push(@suite_t, 0);
-      push(@suite_f, 0);
-      push(@suite_d, 0);
-      push(@suite_e, 0);
-      push(@suite_time, 0);
-      $suite_group = $tsuite;
+      map{ $suite_stats{$tsuite}{$_}= 0 } @stat_list;
+      $suite_group= $tsuite;
     }
 
-    $all_t++;
-    $suite_t[-1]++;
+    # Increment test counters
+    $suite_stats{$tsuite}{'tests'}++;
+    $total_stats{'tests'}++;
 
-    if ( $tinfo->{'result'} eq 'MTR_RES_FAILED' ) {
-        $suite_f[-1]++;
-        $all_f++;
+    # Increment result counters based on outcome of the test
+    if ($tinfo->{'result'} eq 'MTR_RES_UNSTABLE')
+    {
+      $suite_stats{$tsuite}{'unstable'}++;
+      $total_stats{'unstable'}++;
     }
-    elsif ( $tinfo->{'result'} eq 'MTR_RES_SKIPPED' ) {
-        $suite_d[-1]++;
-        $all_d++;
+    elsif ($tinfo->{'result'} eq 'MTR_RES_FAILED' or $tinfo->{failures})
+    {
+      $suite_stats{$tsuite}{'failures'}++;
+      $total_stats{'failures'}++;
+    }
+    elsif ($tinfo->{'result'} eq 'MTR_RES_SKIPPED')
+    {
+      if ($tinfo->{'disable'})
+      {
+        $suite_stats{$tsuite}{'disabled'}++;
+        $total_stats{'disabled'}++;
+      }
+      else
+      {
+        $suite_stats{$tsuite}{'skipped'}++;
+        $total_stats{'skipped'}++;
+      }
     }
 
-    $all_time = $all_time + $tinfo->{'timer'};
-    $suite_time[-1] = $suite_time[-1] + $tinfo->{'timer'};
+    # Update duration
+    $suite_stats{$tsuite}{'time'}+= $tinfo->{'timer'};
+    $total_stats{'time'}+= $tinfo->{'timer'};
   }
 
-  $suite_group = "";
-  my $s = 0;
-  # output data
- 
-  $all_time = $all_time / 1000.0;
+  # --------------------------------------
+  # Generate xml report from obtained data
+  # --------------------------------------
 
-  print $xml_report_file "<testsuites tests=\"$all_t\" failures=\"$all_f\" disabled=\"$all_d\" errors=\"0\" time=\"$all_time\" name=\"AllTests\">\n";
-  foreach my $tinfo (@$tests) 
+  # Convert overall test duration from ms to s
+  $total_stats{'time'}/= 1000.0;
+
+  my $total_unstable_tests= $::opt_report_unstable_tests ?
+                            "unstable-tests=\"$total_stats{'unstable'}\" "
+                            : "";
+
+  # Top level xml tag
+  print $xml_report_file "<testsuites tests=\"$total_stats{'tests'}\" ".
+                         "failures=\"$total_stats{'failures'}\" ".
+                         "$total_unstable_tests".
+                         "disabled=\"$total_stats{'disabled'}\" ".
+                         "skipped=\"$total_stats{'skipped'}\" ".
+                         "errors=\"$total_stats{'errors'}\" ".
+                         "time=\"$total_stats{'time'}\" ".
+                         "name=\"AllTests\">\n";
+
+  $suite_group= "";
+
+  foreach my $tinfo (@sorted_tests)
   {
-    my @parts = split /\./, $tinfo->{'name'};
-    $tsuite = $parts[0];
-    $tname = $parts[1];
+    my ($tsuite, $tname)= split /\./, $tinfo->{'name'};
 
+    # Put reports in a suite-wise manner
     if ($tsuite ne $suite_group)
     {
       if ($suite_group)
       {
-        print $xml_report_file "  </testsuite>\n";
+        print $xml_report_file " " x 2;
+        print $xml_report_file "</testsuite>\n";
       }
-      $suite_group = $tsuite;
-      my $tmp = $suite_time[$s] / 1000.0;
-      print $xml_report_file "  <testsuite name=\"$tsuite\" tests=\"@suite_t[$s]\" failures=\"$suite_f[$s]\" disabled=\"$suite_d[$s]\" errors=\"$suite_e[$s]\" time=\"$tmp\">\n";
-      $s++;
+
+      $suite_group= $tsuite;
+      my $suite_time= $suite_stats{$tsuite}{'time'} / 1000.0;
+      my $suite_unstable_tests= $::opt_report_unstable_tests ?
+                                "unstable-tests=\"".
+                                $suite_stats{$tsuite}{'unstable'}."\" "
+                                : "";
+
+      print $xml_report_file " " x 2;
+      print $xml_report_file "<testsuite name=\"$tsuite\" ".
+                             "tests=\"$suite_stats{$tsuite}{'tests'}\" ".
+                             "failures=\"$suite_stats{$tsuite}{'failures'}\" ".
+                             "$suite_unstable_tests".
+                             "disabled=\"$suite_stats{$tsuite}{'disabled'}\" ".
+                             "skipped=\"$suite_stats{$tsuite}{'skipped'}\" ".
+                             "errors=\"$suite_stats{$tsuite}{'errors'}\" ".
+                             "time=\"$suite_time\">\n";
     }
 
-    my $time = $tinfo->{'timer'} / 1000.0;
-    if ( $tinfo->{'result'} eq 'MTR_RES_FAILED' ) {
-      print $xml_report_file "    <testcase name=\"$tname\" status=\"run\" time=\"$time\" classname=\"$tsuite\" >\n";
-      print $xml_report_file "       <failure message=\"test failed:\" type=\"\"><![CDATA[$tinfo->{'comment'}]]></failure>\n";
-      print $xml_report_file "    </testcase>\n";
+    # Get the test's variant(if any)
+    my $combination= $tinfo->{combination} ?
+                     " combination=\"".$tinfo->{combination}."\"" : "";
+
+    my $test_time= $tinfo->{'timer'} / 1000.0;
+
+    # Print outcome-based tags
+    if ($tinfo->{'result'} eq 'MTR_RES_UNSTABLE')
+    {
+      print $xml_report_file " " x 4;
+      print $xml_report_file "<testcase name=\"$tname\"$combination ".
+                             "status=\"unstable\" ".
+                             "time=\"$test_time\" ".
+                             "suitename=\"$tsuite\" >\n";
+
+      print $xml_report_file " " x 7;
+      print $xml_report_file "<unstable message=\"Test unstable: It failed ".
+                             "initially but passed on one or more retries\" ".
+                             "passed=\"".
+                             ($tinfo->{retries} - $tinfo->{failures} - 1).
+                             "\" failed=\"$tinfo->{failures}\" />\n";
+
+      print $xml_report_file " " x 4;
+      print $xml_report_file "</testcase>\n";
     }
-    elsif ( $tinfo->{'result'} eq 'MTR_RES_SKIPPED' ) {
-      print $xml_report_file "    <testcase name=\"$tname\" status=\"notrun\" time=\"$time\" classname=\"$tsuite\" />\n";
+    elsif ($tinfo->{'result'} eq 'MTR_RES_FAILED' or $tinfo->{'failures'})
+    {
+      print $xml_report_file " " x 4;
+      print $xml_report_file "<testcase name=\"$tname\"$combination ".
+                             "status=\"fail\" ".
+                             "time=\"$test_time\" ".
+                             "suitename=\"$tsuite\" >\n";
+
+      # Get information regarding the failure
+      my $failure_log= $tinfo->{'logfile'};
+      my $failure_comment;
+
+      if ($tinfo->{'warnings'})
+      {
+        $failure_comment= ": Found warnings/errors in server log file!";
+        $failure_log= $failure_log."\n".$tinfo->{'warnings'};
+      }
+
+      $failure_comment= ": ".$1 if ($failure_log =~ /mysqltest: (.*)/);
+
+      # For large comments from mysql-test-run.pl, display a brief message in
+      # the attribute 'message' and prepend details within the <failure> tag
+      my @comment_patterns=
+      ("MTR's internal check of the test case '.+' failed\\.",
+       "Test case timeout after \\d+ seconds",
+       "Could not execute 'check-testcase' [a-z]+ testcase '.+' \\(res: .*\\)",
+       "Could not execute 'check-warnings' for testcase '.+' \\(res: .*\\)",
+       "Server .+ failed during test run",
+       "The server .+ crashed while running 'check testcase [a-z]+ test'",
+       "The server .+ crashed while running 'check warnings'",
+       "mysqltest failed but provided no output");
+
+      my $comment_regex= "(". join('|', @comment_patterns). ")(.*)";
+      if ($tinfo->{'comment'}=~ /$comment_regex/s)
+      {
+        $failure_comment.= ", ".$1;
+        $failure_log= $2."\n".$failure_log;
+      }
+      else
+      {
+        $failure_comment= ", ".$tinfo->{'comment'} if ($tinfo->{'comment'});
+      }
+
+      if (!$failure_log)
+      {
+        $failure_log= "Failure log not found";
+        $failure_comment= ": Unknown result, no comment or log found"
+          if (!$failure_comment);
+      }
+
+      print $xml_report_file " " x 7;
+      print $xml_report_file "<failure message=\"Test failed".
+                             "$failure_comment\">\n";
+
+      # Embed failure log as a literal string so that the XML parser
+      # ignores any markup which might be included
+      my $failure_log_indent= " " x 10;
+      my $failure_log_message= "";
+      map { $failure_log_message= $failure_log_message.$failure_log_indent.$_ }
+          (split /^/, $failure_log);
+
+      print $xml_report_file $failure_log_indent."<![CDATA[\n".
+                             $failure_log_message."\n".
+                             $failure_log_indent."]]>\n";
+
+      print $xml_report_file " " x 7;
+      print $xml_report_file "</failure>\n";
+
+      print $xml_report_file " " x 4;
+      print $xml_report_file "</testcase>\n";
     }
-    elsif ( $tinfo->{'result'} eq 'MTR_RES_PASSED' ) {
-      print $xml_report_file "    <testcase name=\"$tname\" status=\"run\" time=\"$time\" classname=\"$tsuite\" />\n";
+    elsif ($tinfo->{'result'} eq 'MTR_RES_SKIPPED')
+    {
+      # Differentiate disabled and skipped tests as we return
+      # the same outcome code for both
+      my $not_run_status= $tinfo->{'disable'} ? "disabled" : "skipped";
+
+      print $xml_report_file " " x 4;
+      print $xml_report_file "<testcase name=\"$tname\"$combination ".
+                             "status=\"$not_run_status\" ".
+                             "time=\"$test_time\" ".
+                             "suitename=\"$tsuite\" ".
+                             "comment=\"$tinfo->{'comment'}\" />\n";
+    }
+    elsif ($tinfo->{'result'} eq 'MTR_RES_PASSED')
+    {
+      print $xml_report_file " " x 4;
+      print $xml_report_file "<testcase name=\"$tname\"$combination ".
+                             "status=\"pass\" ".
+                             "time=\"$test_time\" ".
+                             "suitename=\"$tsuite\" />\n";
     }
   }
   if ($suite_group)
   {
-    print $xml_report_file "  </testsuite>\n";
+    print $xml_report_file " " x 2;
+    print $xml_report_file "</testsuite>\n";
   }
   print $xml_report_file "</testsuites>\n";
 }
@@ -334,12 +512,6 @@ sub mtr_generate_xml_report($) {
 sub mtr_report_stats ($$;$) {
   my ($prefix, $tests, $dont_error)= @_;
 
-  if ($xml_report_file)
-  {
-    mtr_generate_xml_report($tests);
-    $xml_report_file->flush();
-  }
-
   # ----------------------------------------------------------------------
   # Find out how we where doing
   # ----------------------------------------------------------------------
@@ -348,6 +520,7 @@ sub mtr_report_stats ($$;$) {
   my $tot_skipdetect= 0;
   my $tot_passed= 0;
   my $tot_failed= 0;
+  my $tot_unstable= 0;
   my $tot_tests=  0;
   my $tot_restarts= 0;
   my $found_problems= 0;
@@ -359,6 +532,20 @@ sub mtr_report_stats ($$;$) {
       # Test has failed at least one time
       $tot_tests++;
       $tot_failed++;
+      if ($::opt_report_unstable_tests and defined $tinfo->{retries})
+      {
+        my $num_passed= $tinfo->{retries} - $tinfo->{failures} - 1;
+
+        # Tests which exhibit both passing and failing behaviour
+        # with the same code are unstable tests. The level of in-
+        # stability is not restricted i.e., a failed test which is
+        # successful on at least one retry is marked unstable.
+        if ($num_passed > 0)
+        {
+          $tot_unstable++;
+          $tinfo->{'result'}= 'MTR_RES_UNSTABLE';
+        }
+      }
     }
     elsif ( $tinfo->{'result'} eq 'MTR_RES_SKIPPED' )
     {
@@ -463,22 +650,50 @@ sub mtr_report_stats ($$;$) {
     my $ratio=  $tot_passed * 100 / $tot_tests;
     summary_print(sprintf("Failed $tot_failed/$tot_tests tests, %.2f%% were successful.\n\n", $ratio));
 
-    # Print the list of test that failed in a format
-    # that can be copy pasted to rerun only failing tests
-    summary_print("Failing test(s):");
-
+    # Hashes to keep track of reported failures
     my %seen= ();
+    my %seen_unstable= ();
+
     foreach my $tinfo (@$tests)
     {
       my $tname= $tinfo->{'name'};
-      if ( ($tinfo->{failures} || $tinfo->{rep_failures}) and ! $seen{$tname})
+      if (($tinfo->{failures} || $tinfo->{rep_failures}))
       {
-        summary_print(" $tname");
-	$seen{$tname}= 1;
+        # Check for unstable tests
+        if ($tinfo->{result} eq "MTR_RES_UNSTABLE" and !$seen_unstable{$tname})
+        {
+          # Report all unstable tests in the format :-
+          # <test_name>(<number of failures>/<total attempts>).
+          #
+          # Marking the test as 'seen' in case of unstable tests might cause
+          # hard-failures in other combination runs of the same test to
+          # not be reported. Separate hash used to avoid redundant mentions
+          # of an unstable test
+          $seen_unstable{$tname}= "(".$tinfo->{failures}."/".
+                                  ($tinfo->{retries} - 1).")";
+        }
+        elsif (!$seen{$tname})
+        {
+          $seen{$tname}= 1;
+        }
       }
     }
-    summary_print("\n");
-    print "\n";
+
+    # Print the list of tests that failed in a format
+    # that can be copy pasted to rerun only failing tests
+    if (%seen)
+    {
+      summary_print("Failing test(s): ".
+                    join(" ", keys %seen). "\n\n");
+    }
+
+    # Print unstable tests, if any
+    if (%seen_unstable)
+    {
+      summary_print("Unstable test(s)(failures/attempts): ".
+                    join(" ", map {$_.$seen_unstable{$_}} keys %seen_unstable).
+                    "\n\n");
+    }
 
     # Print info about reporting the error
     print
@@ -494,12 +709,27 @@ sub mtr_report_stats ($$;$) {
   }
   close($summary_report_file) if defined($summary_report_file);
 
+  if ($xml_report_file)
+  {
+    mtr_generate_xml_report($tests);
+    $xml_report_file->flush();
+  }
+
   print "$tot_skipped tests were skipped, ".
     "$tot_skipdetect by the test itself.\n\n" if $tot_skipped;
 
   if ( $tot_failed != 0 || $found_problems)
   {
-    mtr_error("there were failing test cases") unless $dont_error;
+    if ($tot_failed == $tot_unstable)
+    {
+      # Print a warning if all failures are due to unstable tests
+      mtr_warning("There are failures due to unstable test cases.\n".
+                  "However, the tests are not hard-failing.");
+    }
+    else
+    {
+      mtr_error("there were failing test cases") unless $dont_error;
+    }
   }
 }
 
@@ -544,7 +774,14 @@ sub mtr_print_header ($) {
 ##############################################################################
 
 sub mtr_xml_init($) {
-  my ($fn) = @_;
+  my ($fn)= @_;
+  # Write xml-report in the build directory for out-of-source builds or current
+  # path otherwise, unless an absolute path is already specified
+  $fn= File::Spec->rel2abs($fn,
+                           $ENV{MTR_BINDIR} ?
+                           File::Spec->catfile($ENV{MTR_BINDIR}, "mysql-test"):
+                           "");
+
   unless(open $xml_report_file, '>', $fn) {
     mtr_error("could not create xml_report file $fn");
   }
