@@ -77,6 +77,7 @@ typedef enum status_vars
     STATS_STATE_UUID = 0,
     STATS_PROTOCOL_VERSION,
     STATS_LAST_APPLIED,
+    STATS_LAST_COMMITTED,
     STATS_REPLICATED,
     STATS_REPLICATED_BYTES,
     STATS_KEYS_COUNT,
@@ -120,6 +121,8 @@ typedef enum status_vars
     STATS_GCACHE_POOL_SIZE,
     STATS_CAUSAL_READS,
     STATS_CERT_INTERVAL,
+    STATS_OPEN_TRX,
+    STATS_OPEN_CONN,
     STATS_IST_RECEIVE_STATUS,
     STATS_IST_RECEIVE_SEQNO_START,
     STATS_IST_RECEIVE_SEQNO_CURRENT,
@@ -132,6 +135,7 @@ static const struct wsrep_stats_var wsrep_stats[STATS_MAX + 1] =
 {
     { "local_state_uuid",         WSREP_VAR_STRING, { 0 }  },
     { "protocol_version",         WSREP_VAR_INT64,  { 0 }  },
+    { "last_applied",             WSREP_VAR_INT64,  { -1 } },
     { "last_committed",           WSREP_VAR_INT64,  { -1 } },
     { "replicated",               WSREP_VAR_INT64,  { 0 }  },
     { "replicated_bytes",         WSREP_VAR_INT64,  { 0 }  },
@@ -176,6 +180,8 @@ static const struct wsrep_stats_var wsrep_stats[STATS_MAX + 1] =
     { "gcache_pool_size",         WSREP_VAR_INT64,  { 0 }  },
     { "causal_reads",             WSREP_VAR_INT64,  { 0 }  },
     { "cert_interval",            WSREP_VAR_DOUBLE, { 0 }  },
+    { "open_transactions",        WSREP_VAR_INT64,  { 0 }  },
+    { "open_connections",         WSREP_VAR_INT64,  { 0 }  },
     { "ist_receive_status",       WSREP_VAR_STRING, { 0 }  },
     { "ist_receive_seqno_start",  WSREP_VAR_INT64,  { 0 }  },
     { "ist_receive_seqno_current",WSREP_VAR_INT64,  { 0 }  },
@@ -208,6 +214,7 @@ galera::ReplicatorSMM::stats_get()
 
     sv[STATS_PROTOCOL_VERSION   ].value._int64  = protocol_version_;
     sv[STATS_LAST_APPLIED       ].value._int64  = apply_monitor_.last_left();
+    sv[STATS_LAST_COMMITTED     ].value._int64  = commit_monitor_.last_left();
     sv[STATS_REPLICATED         ].value._int64  = replicated_();
     sv[STATS_REPLICATED_BYTES   ].value._int64  = replicated_bytes_();
     sv[STATS_KEYS_COUNT         ].value._int64  = keys_count_();
@@ -284,6 +291,10 @@ galera::ReplicatorSMM::stats_get()
                                                                    sst_state_);
     sv[STATS_CAUSAL_READS].value._int64    = causal_reads_();
 
+    Wsdb::stats wsdb_stats(wsdb_.get_stats());
+    sv[STATS_OPEN_TRX].value._int64 = wsdb_stats.n_trx_;
+    sv[STATS_OPEN_CONN].value._int64 = wsdb_stats.n_conn_;
+
     if (ist_receiver_.running())
     {
         // calculate %-age complete
