@@ -189,7 +189,7 @@ StateRequest_v1::StateRequest_v1 (
                                << ") unrepresentable";
 
     if (ist_req_len > INT32_MAX || ist_req_len < 0)
-        gu_throw_error (EMSGSIZE) << "IST request length (" << sst_req_len
+        gu_throw_error (EMSGSIZE) << "IST request length (" << ist_req_len
                                << ") unrepresentable";
 
     char* ptr(req_);
@@ -603,7 +603,7 @@ ReplicatorSMM::prepare_state_request (const void* const   sst_req,
             {
                 log_info << "State gap can't be serviced using IST."
                             " Switching to SST";
-                log_debug
+                log_info
                     << "Failed to prepare for incremental state transfer: "
                     << e.what() << ". IST will be unavailable.";
             }
@@ -707,11 +707,9 @@ ReplicatorSMM::send_state_request (const StateRequest* const req, const bool uns
             /* Check that we're not running out of space in monitor. */
             if (local_monitor_.would_block(seqno_l))
             {
-                long const seconds = sst_retry_sec_ * tries;
-                log_error << "We ran out of resources, seemingly because "
-                          << "we've been unsuccessfully requesting state "
-                          << "transfer for over " << seconds << " seconds. "
-                          << "Please check that there is "
+                log_error << "Slave queue grew too long while trying to "
+                          << "request state transfer " << tries << " time(s). "
+                          << "Please make sure that there is "
                           << "at least one fully synced member in the group. "
                           << "Application must be restarted.";
                 ret = -EDEADLK;
