@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2016 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2018 Codership Oy <info@codership.com>
  */
 
 /*! @file ring buffer storage class */
@@ -27,6 +27,7 @@ namespace gcache
                     size_t             size,
                     seqno2ptr_t&       seqno2ptr,
                     gu::UUID&          gid,
+                    int                dbg,
                     bool               recover);
 
         ~RingBuffer ();
@@ -124,6 +125,8 @@ namespace gcache
                     ? (false) : (seqno >= freeze_purge_at_seqno_));
         }
 
+        void set_debug(int const dbg) { debug_ = dbg & DEBUG; }
+
     private:
 
         static size_t const PREAMBLE_LEN = 1024;
@@ -134,6 +137,8 @@ namespace gcache
         // 2 - buffer alignemnt to GU_WORD_BYTES
         static int    const VERSION = 2;
 
+        static int    const DEBUG = 2; // debug flag
+
         gu::FileDescriptor fd_;
         gu::MMap           mmap_;
         char*        const preamble_; // ASCII text preamble
@@ -154,6 +159,8 @@ namespace gcache
         size_t             size_used_;
         size_t             size_trail_;
 
+        int                debug_;
+
         bool               open_;
 
         BufferHeader* get_new_buffer (size_type size);
