@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2014 Codership Oy <info@codership.com>
+ * Copyright (C) 2009-2019 Codership Oy <info@codership.com>
  */
 
 #include "gmcast.hpp"
@@ -118,7 +118,7 @@ gcomm::GMCast::GMCast(Protonet& net, const gu::URI& uri,
         param<int>(conf_, uri,
                    Conf::GMCastMaxInitialReconnectAttempts,
                    gu::to_string(max_retry_cnt_))),
-    next_check_   (gu::datetime::Date::now())
+    next_check_   (gu::datetime::Date::monotonic())
 {
     log_info << "GMCast version " << version_;
 
@@ -389,6 +389,90 @@ void gcomm::GMCast::close(bool force)
     prim_view_reached_ = false;
 }
 
+//
+// Private
+//
+
+
+// Find other local endpoint matching to proto
+static const Proto*
+find_other_local_endpoint(const gcomm::gmcast::ProtoMap& proto_map,
+                          const gcomm::gmcast::Proto* proto)
+{
+    for (gcomm::gmcast::ProtoMap::const_iterator i(proto_map.begin());
+         i != proto_map.end(); ++i)
+    {
+        if (i->second != proto &&
+            i->second->handshake_uuid() == proto->handshake_uuid())
+        {
+            return i->second;
+        }
+    }
+    return 0;
+}
+
+// Find other endpoint with same remote UUID
+static const Proto*
+find_other_endpoint_same_remote_uuid(const gcomm::gmcast::ProtoMap& proto_map,
+                                     const gcomm::gmcast::Proto* proto)
+{
+    for (gcomm::gmcast::ProtoMap::const_iterator i(proto_map.begin());
+         i != proto_map.end(); ++i)
+    {
+        if (i->second != proto &&
+            i->second->remote_uuid() == proto->remote_uuid())
+        {
+            return i->second;
+        }
+    }
+    return 0;
+}
+
+bool gcomm::GMCast::is_own(const gmcast::Proto* proto) const
+{
+    assert(proto->remote_uuid() != gcomm::UUID::nil());
+    if (proto->remote_uuid() != uuid())
+    {
+        return false;
+    }
+    return find_other_local_endpoint(*proto_map_, proto);
+}
+
+void gcomm::GMCast::blacklist(const gmcast::Proto* proto)
+{
+    initial_addrs_.erase(proto->remote_addr());
+    pending_addrs_.erase(proto->remote_addr());
+    addr_blacklist_.insert(std::make_pair(
+                               proto->remote_addr(),
+                               AddrEntry(gu::datetime::Date::monotonic(),
+                                         gu::datetime::Date::monotonic(),
+                                         proto->remote_uuid())));
+}
+
+bool gcomm::GMCast::is_not_own_and_duplicate_exists(
+    const Proto* proto) const
+{
+    assert(proto->remote_uuid() != gcomm::UUID::nil());
+    const Proto* other(find_other_local_endpoint(*proto_map_, proto));
+    if (!other)
+    {
+        // Not own
+        // Check if remote UUID matches to self
+        if (proto->remote_uuid() == uuid())
+        {
+            return true;
+        }
+        // Check if other proto entry with same remote
+        // UUID but different remote address exists.
+        other = find_other_endpoint_same_remote_uuid(*proto_map_, proto);
+        if (other && other->remote_addr() != proto->remote_addr())
+        {
+            return true;
+        }
+    }
+    return false;
+}
+
 // Erase proto entry in safe manner
 // 1) Erase from relay_set_
 // 2) Erase from proto_map_
@@ -396,7 +480,8 @@ void gcomm::GMCast::close(bool force)
 void gcomm::GMCast::erase_proto(gmcast::ProtoMap::iterator i)
 {
     Proto* p(ProtoMap::value(i));
-    std::set<Socket*>::iterator si(relay_set_.find(p->socket().get()));
+    RelayEntry e(p, p->socket().get());
+    RelaySet::iterator si(relay_set_.find(e));
     if (si != relay_set_.end())
     {
         relay_set_.erase(si);
@@ -543,13 +628,13 @@ void gcomm::GMCast::gmcast_forget(const UUID& uuid,
             }
             ae.set_max_retries(0);
             ae.set_retry_cnt(1);
-            gu::datetime::Date now(gu::datetime::Date::now());
+            gu::datetime::Date now(gu::datetime::Date::monotonic());
             // Don't reduce next reconnect time if it is set greater than
             // requested
             if ((now + wait_period > ae.next_reconnect()) ||
                 (ae.next_reconnect() == gu::datetime::Date::max()))
             {
-                ae.set_next_reconnect(gu::datetime::Date::now() + wait_period);
+                ae.set_next_reconnect(gu::datetime::Date::monotonic() + wait_period);
             }
             else
             {
@@ -576,35 +661,13 @@ void gcomm::GMCast::handle_connected(Proto* rp)
     }
 }
 
-// Throw if the proto has foreign origin. If another proto entry with the
-// same handshake UUID is found from proto_map, the connection is local
-// and the function returns. If such an entry is not found, the function
-// throws.
-static void throw_if_foreign(
-    const gcomm::gmcast::ProtoMap& proto_map,
-    const gcomm::gmcast::Proto* proto,
-    const char* message)
-{
-    for (ProtoMap::const_iterator i(proto_map.begin());
-         i != proto_map.end(); ++i)
-    {
-        // The condition i->first != proto->socket()->id() is needed
-        // if this function is called after the established connection
-        // has been inserted into proto_map.
-        if (i->second->handshake_uuid() == proto->handshake_uuid() &&
-            i->first != proto->socket()->id())
-        {
-            return;
-        }
-    }
-    gu_throw_fatal << message;
-}
-
 void gcomm::GMCast::handle_established(Proto* est)
 {
     log_info << self_string() << " connection established to "
              << est->remote_uuid() << " "
              << est->remote_addr();
+    // UUID checks are handled during protocol handshake
+    assert(est->remote_uuid() != uuid());
 
     if (is_evicted(est->remote_uuid()))
     {
@@ -614,41 +677,6 @@ void gcomm::GMCast::handle_established(Proto* est)
         return;
     }
 
-    if (est->remote_uuid() == uuid())
-    {
-        if (prim_view_reached_ == false)
-        {
-            throw_if_foreign(
-                *proto_map_, est,
-                "A node with the same UUID already exists in the cluster");
-        }
-
-        std::set<std::string>::iterator
-            ia_i(initial_addrs_.find(est->remote_addr()));
-        if (ia_i != initial_addrs_.end())
-        {
-            initial_addrs_.erase(ia_i);
-        }
-        AddrList::iterator i(pending_addrs_.find(est->remote_addr()));
-        if (i != pending_addrs_.end())
-        {
-            if (addr_blacklist_.find(est->remote_addr()) == addr_blacklist_.end())
-            {
-                log_warn << self_string()
-                         << " address '" << est->remote_addr()
-                         << "' points to own listening address, blacklisting";
-            }
-            pending_addrs_.erase(i);
-            addr_blacklist_.insert(make_pair(est->remote_addr(),
-                                             AddrEntry(gu::datetime::Date::now(),
-                                                       gu::datetime::Date::now(),
-                                                       est->remote_uuid())));
-        }
-        erase_proto(proto_map_->find_checked(est->socket()->id()));
-        update_addresses();
-        return;
-    }
-
     // If address is found from pending_addrs_, move it to remote_addrs list
     // and set retry cnt to -1
     const std::string& remote_addr(est->remote_addr());
@@ -675,8 +703,8 @@ void gcomm::GMCast::handle_established(Proto* est)
         remote_addrs_.erase(i);
         i = remote_addrs_.insert_unique(
             make_pair(est->remote_addr(),
-                      AddrEntry(gu::datetime::Date::now(),
-                                gu::datetime::Date::now(),
+                      AddrEntry(gu::datetime::Date::monotonic(),
+                                gu::datetime::Date::monotonic(),
                                 est->remote_uuid())));
     }
 
@@ -783,7 +811,7 @@ void gcomm::GMCast::handle_failed(Proto* failed)
             AddrEntry& ae(AddrList::value(i));
             ae.set_retry_cnt(ae.retry_cnt() + 1);
 
-            gu::datetime::Date rtime = gu::datetime::Date::now() + gu::datetime::Period("PT1S");
+            gu::datetime::Date rtime = gu::datetime::Date::monotonic() + gu::datetime::Period("PT1S");
             log_debug << self_string()
                       << " setting next reconnect time to "
                       << rtime << " for " << remote_addr;
@@ -823,8 +851,8 @@ void gcomm::GMCast::insert_address (const std::string& addr,
     }
 
     if (alist.insert(make_pair(addr,
-                               AddrEntry(gu::datetime::Date::now(),
-                                         gu::datetime::Date::now(), uuid))).second == false)
+                               AddrEntry(gu::datetime::Date::monotonic(),
+                                         gu::datetime::Date::monotonic(), uuid))).second == false)
     {
         log_warn << "Duplicate entry: " << addr;
     }
@@ -945,7 +973,7 @@ void gcomm::GMCast::update_addresses()
 
                     // Add some randomness for first reconnect to avoid
                     // simultaneous connects
-                    gu::datetime::Date rtime(gu::datetime::Date::now());
+                    gu::datetime::Date rtime(gu::datetime::Date::monotonic());
 
                     rtime = rtime + ::rand() % (100*gu::datetime::MSec);
                     ae.set_next_reconnect(rtime);
@@ -964,25 +992,25 @@ void gcomm::GMCast::update_addresses()
     if (mcast_)
     {
         log_debug << mcast_addr_;
-        local_segment.push_back(mcast_.get());
+        local_segment.push_back(RelayEntry(0, mcast_.get()));
     }
 
     self_index_ = 0;
     for (ProtoMap::const_iterator i(proto_map_->begin()); i != proto_map_->end();
          ++i)
     {
-        const Proto& p(*ProtoMap::value(i));
+        Proto* p(ProtoMap::value(i));
 
-        log_debug << "Proto: " << p;
+        log_debug << "Proto: " << *p;
 
-        if (p.remote_segment() == segment_)
+        if (p->remote_segment() == segment_)
         {
-            if (p.state() == Proto::S_OK &&
-                (p.mcast_addr() == "" ||
-                 p.mcast_addr() != mcast_addr_))
+            if (p->state() == Proto::S_OK &&
+                (p->mcast_addr() == "" ||
+                 p->mcast_addr() != mcast_addr_))
             {
-                local_segment.push_back(p.socket().get());
-                if (p.remote_uuid() < uuid())
+                local_segment.push_back(RelayEntry(p, p->socket().get()));
+                if (p->remote_uuid() < uuid())
                 {
                     ++self_index_;
                 }
@@ -990,10 +1018,10 @@ void gcomm::GMCast::update_addresses()
         }
         else
         {
-            if (p.state() == Proto::S_OK)
+            if (p->state() == Proto::S_OK)
             {
-                Segment& remote_segment(segment_map_[p.remote_segment()]);
-                remote_segment.push_back(p.socket().get());
+                Segment& remote_segment(segment_map_[p->remote_segment()]);
+                remote_segment.push_back(RelayEntry(p, p->socket().get()));
             }
         }
     }
@@ -1012,7 +1040,7 @@ void gcomm::GMCast::reconnect()
 
     /* Loop over known remote addresses and connect if proto entry
      * does not exist */
-    gu::datetime::Date now = gu::datetime::Date::now();
+    gu::datetime::Date now = gu::datetime::Date::monotonic();
     AddrList::iterator i, i_next;
 
     for (i = pending_addrs_.begin(); i != pending_addrs_.end(); i = i_next)
@@ -1131,32 +1159,48 @@ void gcomm::GMCast::check_liveness()
     std::set<UUID> live_uuids;
 
     // iterate over proto map and mark all timed out entries as failed
-    gu::datetime::Date now(gu::datetime::Date::now());
+    gu::datetime::Date now(gu::datetime::Date::monotonic());
     for (ProtoMap::iterator i(proto_map_->begin()); i != proto_map_->end(); )
     {
+        // Store next iterator into temporary, handle_failed() may remove
+        // the entry proto_map_.
         ProtoMap::iterator i_next(i);
         ++i_next;
         Proto* p(ProtoMap::value(i));
         if (p->state() > Proto::S_INIT &&
             p->state() < Proto::S_FAILED &&
-            p->tstamp() + peer_timeout_ < now)
+            p->recv_tstamp() + peer_timeout_ < now)
         {
             // Only log if addr has not been blacklisted
             if (addr_blacklist_.find(p->remote_addr()) == addr_blacklist_.end())
             {
+                gcomm::SocketStats stats(p->socket()->stats());
                 log_info << self_string()
                          << " connection to peer "
                          << p->remote_uuid() << " with addr "
                          << p->remote_addr()
                          << " timed out, no messages seen in " << peer_timeout_
-                         << " (gmcast.peer_timeout)";
+                         << " (gmcast.peer_timeout), socket stats: "
+                         << stats;
             }
             p->set_state(Proto::S_FAILED);
             handle_failed(p);
         }
         else if (p->state() == Proto::S_OK)
         {
-            if (p->tstamp() + peer_timeout_*2/3 < now)
+            gcomm::SocketStats stats(p->socket()->stats());
+            if (stats.send_queue_length >= 1024)
+            {
+                log_debug << self_string()
+                          << " socket send queue to "
+                          << " peer "
+                          << p->remote_uuid() << " with addr "
+                          << p->remote_addr()
+                          << ", socket stats: "
+                          << stats;
+            }
+            if ((p->recv_tstamp() + peer_timeout_*2/3 < now) ||
+                (p->send_tstamp() + peer_timeout_*1/3 < now))
             {
                 p->send_keepalive();
             }
@@ -1234,7 +1278,7 @@ void gcomm::GMCast::check_liveness()
             log_debug << "relay set maxel :" << *p << " count: "
                       << CmpUuidCounts(nonlive_uuids, segment_).count(p);
 
-            relay_set_.insert(p->socket().get());
+            relay_set_.insert(RelayEntry(p, p->socket().get()));
             const LinkMap& lm(p->link_map());
             for (LinkMap::const_iterator lm_i(lm.begin()); lm_i != lm.end();
                  ++lm_i)
@@ -1255,7 +1299,7 @@ void gcomm::GMCast::check_liveness()
 
 gu::datetime::Date gcomm::GMCast::handle_timers()
 {
-    const gu::datetime::Date now(gu::datetime::Date::now());
+    const gu::datetime::Date now(gu::datetime::Date::monotonic());
 
     if (now >= next_check_)
     {
@@ -1268,14 +1312,18 @@ gu::datetime::Date gcomm::GMCast::handle_timers()
 }
 
 
-void send(gcomm::Socket* s, gcomm::Datagram& dg)
+void gcomm::GMCast::send(const RelayEntry& re, int segment, gcomm::Datagram& dg)
 {
     int err;
-    if ((err = s->send(dg)) != 0)
+    if ((err = re.socket->send(segment, dg)) != 0)
     {
-        log_debug << "failed to send to " << s->remote_addr()
+        log_debug << "failed to send to " << re.socket->remote_addr()
                   << ": (" << err << ") " << strerror(err);
     }
+    else
+    {
+        re.proto->set_send_tstamp(gu::datetime::Date::monotonic());
+    }
 }
 
 void gcomm::GMCast::relay(const Message& msg,
@@ -1295,15 +1343,16 @@ void gcomm::GMCast::relay(const Message& msg,
     if (msg.flags() & Message::F_RELAY)
     {
         gu_trace(push_header(relay_msg, relay_dg));
-        for (SegmentMap::iterator i(segment_map_.begin());
-             i != segment_map_.end(); ++i)
+        for (SegmentMap::iterator segment_i(segment_map_.begin());
+             segment_i != segment_map_.end(); ++segment_i)
         {
-            Segment& segment(i->second);
-            for (Segment::iterator j(segment.begin()); j != segment.end(); ++j)
+            Segment& segment(segment_i->second);
+            for (Segment::iterator target_i(segment.begin());
+                 target_i != segment.end(); ++target_i)
             {
-                if ((*j)->id() != exclude_id)
+                if ((*target_i).socket->id() != exclude_id)
                 {
-                    send(*j, relay_dg);
+                    send(*target_i, msg.segment_id(), relay_dg);
                 }
             }
         }
@@ -1316,10 +1365,13 @@ void gcomm::GMCast::relay(const Message& msg,
             // nodes in local segment that are not directly reachable
             relay_msg.set_flags(relay_msg.flags() | Message::F_RELAY);
             gu_trace(push_header(relay_msg, relay_dg));
-            for (std::set<Socket*>::iterator ri(relay_set_.begin());
-                 ri != relay_set_.end(); ++ri)
+            for (RelaySet::iterator relay_i(relay_set_.begin());
+                 relay_i != relay_set_.end(); ++relay_i)
             {
-                send(*ri, relay_dg);
+                if ((*relay_i).socket->id() != exclude_id)
+                {
+                    send(*relay_i, msg.segment_id(), relay_dg);
+                }
             }
             gu_trace(pop_header(relay_msg, relay_dg));
             relay_msg.set_flags(relay_msg.flags() & ~Message::F_RELAY);
@@ -1336,7 +1388,7 @@ void gcomm::GMCast::relay(const Message& msg,
         Segment& segment(segment_map_[segment_]);
         for (Segment::iterator i(segment.begin()); i != segment.end(); ++i)
         {
-            send(*i, relay_dg);
+            send(*i, msg.segment_id(), relay_dg);
         }
     }
     else
@@ -1371,7 +1423,7 @@ void gcomm::GMCast::handle_up(const void*        id,
             }
             else
             {
-                gu_trace(msg.unserialize(&dg.payload()[0],
+                gu_trace(msg.unserialize(dg.payload().data(),
                                          dg.len(),
                                          dg.offset()));
             }
@@ -1413,7 +1465,7 @@ void gcomm::GMCast::handle_up(const void*        id,
 
             try
             {
-                msg.unserialize(&dg.payload()[0], dg.len(),
+                msg.unserialize(dg.payload().data(), dg.len(),
                                 dg.offset());
             }
             catch (gu::Exception& e)
@@ -1439,7 +1491,7 @@ void gcomm::GMCast::handle_up(const void*        id,
                           Datagram(dg, dg.offset() + msg.serial_size()),
                           id);
                 }
-                p->set_tstamp(gu::datetime::Date::now());
+                p->set_recv_tstamp(gu::datetime::Date::monotonic());
                 send_up(Datagram(dg, dg.offset() + msg.serial_size()),
                         ProtoUpMeta(msg.source_uuid()));
                 return;
@@ -1448,18 +1500,19 @@ void gcomm::GMCast::handle_up(const void*        id,
             {
                 try
                 {
-                    p->set_tstamp(gu::datetime::Date::now());
+                    p->set_recv_tstamp(gu::datetime::Date::monotonic());
                     gu_trace(p->handle_message(msg));
                 }
-                catch (gu::Exception& e)
+                catch (const gu::Exception& e)
                 {
-                    log_warn << "handling gmcast protocol message failed: "
-                             << e.what();
                     handle_failed(p);
                     if (e.get_errno() == ENOTRECOVERABLE)
                     {
                         throw;
                     }
+                    log_warn
+                        << "handling gmcast protocol message failed: "
+                        << e.what();
                     return;
                 }
 
@@ -1468,7 +1521,7 @@ void gcomm::GMCast::handle_up(const void*        id,
                     handle_failed(p);
                     return;
                 }
-                else if (p->changed() == true)
+                else if (p->check_changed_and_reset() == true)
                 {
                     update_addresses();
                     check_liveness();
@@ -1507,27 +1560,72 @@ void gcomm::GMCast::handle_up(const void*        id,
     }
 }
 
+static gcomm::gmcast::Proto* find_by_remote_uuid(
+    const gcomm::gmcast::ProtoMap& proto_map,
+    const gcomm::UUID& uuid)
+{
+    for (gcomm::gmcast::ProtoMap::const_iterator i(proto_map.begin());
+         i != proto_map.end(); ++i)
+    {
+        if (i->second->remote_uuid() == uuid)
+        {
+            return i->second;
+        }
+    }
+    return 0;
+}
 
 int gcomm::GMCast::handle_down(Datagram& dg, const ProtoDownMeta& dm)
 {
     Message msg(version_, Message::GMCAST_T_USER_BASE, uuid(), 1, segment_);
 
+    // If target is set and proto entry for target is found,
+    // send a direct message. Otherwise fall back for broadcast
+    // to ensure message delivery via relay
+    if (dm.target() != UUID::nil())
+    {
+        Proto* target_proto(find_by_remote_uuid(*proto_map_, dm.target()));
+        if (target_proto && target_proto->state() == Proto::S_OK)
+        {
+            gu_trace(push_header(msg, dg));
+            int err;
+            if ((err = target_proto->socket()->send(msg.segment_id(), dg)) != 0)
+            {
+                log_debug << "failed to send to "
+                          << target_proto->socket()->remote_addr()
+                          << ": (" << err << ") " << strerror(err);
+            }
+            else
+            {
+                target_proto->set_send_tstamp(gu::datetime::Date::monotonic());
+            }
+            gu_trace(pop_header(msg, dg));
+            if (err == 0)
+            {
+                return 0;
+            }
+            // In case of error fall back to broadcast
+        }
+        else
+        {
+            log_debug << "Target " << dm.target() << " proto not found";
+        }
+    }
+
     // handle relay set first, skip these peers below
     if (relay_set_.empty() == false)
     {
         msg.set_flags(msg.flags() | Message::F_RELAY);
         gu_trace(push_header(msg, dg));
-        for (std::set<Socket*>::iterator ri(relay_set_.begin());
+        for (RelaySet::iterator ri(relay_set_.begin());
              ri != relay_set_.end(); ++ri)
         {
-            send(*ri, dg);
+            send(*ri, msg.segment_id(), dg);
         }
         gu_trace(pop_header(msg, dg));
         msg.set_flags(msg.flags() & ~Message::F_RELAY);
     }
 
-
-
     for (SegmentMap::iterator si(segment_map_.begin());
          si != segment_map_.end(); ++si)
     {
@@ -1543,7 +1641,7 @@ int gcomm::GMCast::handle_down(Datagram& dg, const ProtoDownMeta& dm)
                 relay_set_.find(segment[target_idx]) == relay_set_.end())
             {
                 gu_trace(push_header(msg, dg));
-                send(segment[target_idx], dg);
+                send(segment[target_idx], msg.segment_id(), dg);
                 gu_trace(pop_header(msg, dg));
             }
         }
@@ -1558,7 +1656,7 @@ int gcomm::GMCast::handle_down(Datagram& dg, const ProtoDownMeta& dm)
                 if (relay_set_.empty() == true ||
                     relay_set_.find(*i) == relay_set_.end())
                 {
-                    send(*i, dg);
+                    send(*i, msg.segment_id(), dg);
                 }
             }
             gu_trace(pop_header(msg, dg));
@@ -1735,7 +1833,7 @@ void gcomm::GMCast::add_or_del_addr(const std::string& val)
             AddrEntry& ae(AddrList::value(ai));
             ae.set_max_retries(0);
             ae.set_retry_cnt(1);
-            ae.set_next_reconnect(gu::datetime::Date::now() + time_wait_);
+            ae.set_next_reconnect(gu::datetime::Date::monotonic() + time_wait_);
             update_addresses();
         }
         else
