@@ -906,6 +906,7 @@ _join (gcs_conn_t* conn, gcs_seqno_t seqno)
     case -ENOTCONN:
         gu_warn ("Sending JOIN failed: %d (%s). "
                  "Will retry in new primary component.", err, strerror(-err));
+        // fall through
     case 0:
         return 0;
     default:
@@ -1656,9 +1657,9 @@ long gcs_interrupt (gcs_conn_t* conn, long handle)
     return gcs_sm_interrupt (conn->sm, handle);
 }
 
-gcs_seqno_t gcs_caused(gcs_conn_t* conn)
+long gcs_caused(gcs_conn_t* conn, gcs_seqno_t& seqno)
 {
-    return gcs_core_caused(conn->core);
+    return gcs_core_caused(conn->core, seqno);
 }
 
 /* Puts action in the send queue and returns after it is replicated */
