@@ -1,12 +1,19 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -630,24 +637,12 @@ bool set_and_validate_user_attributes(THD *thd,
      Str->uses_authentication_string_clause)
   {
     st_mysql_auth *auth= (st_mysql_auth *) plugin_decl(plugin)->info;
-    /*
-      Validate hash string in following cases:
-        1. IDENTIFIED BY PASSWORD.
-        2. IDENTIFIED WITH .. AS 'auth_str' for ALTER USER statement
-           and its a replication slave thread
-    */
-    if (Str->uses_identified_by_password_clause ||
-        (Str->uses_authentication_string_clause &&
-        thd->lex->sql_command == SQLCOM_ALTER_USER &&
-        thd->slave_thread))
+    if (auth->validate_authentication_string((char*)Str->auth.str,
+                                             Str->auth.length))
     {
-      if (auth->validate_authentication_string((char*)Str->auth.str,
-                                               Str->auth.length))
-      {
-        my_error(ER_PASSWORD_FORMAT, MYF(0));
-        plugin_unlock(0, plugin);
-        return(1);
-      }
+      my_error(ER_PASSWORD_FORMAT, MYF(0));
+      plugin_unlock(0, plugin);
+      return(1);
     }
   }
   plugin_unlock(0, plugin);
@@ -693,6 +688,7 @@ bool change_password(THD *thd, const char *host, const char *user,
   size_t new_password_len= strlen(new_password);
   size_t escaped_hash_str_len= 0;
   bool result= true, rollback_whole_statement= false;
+  sql_mode_t old_sql_mode= thd->variables.sql_mode;
   int ret;
 #ifdef WITH_WSREP
   const LEX_CSTRING query_save = thd->query();
@@ -808,7 +804,11 @@ bool change_password(THD *thd, const char *host, const char *user,
     In case its a slave thread or a binlog applier thread, the password
     is already hashed. Do not generate another hash!
    */
+#ifdef WITH_WSREP
+  if ((thd->slave_thread && !thd->wsrep_applier) || thd->is_binlog_applier())
+#else
   if (thd->slave_thread || thd->is_binlog_applier())
+#endif
   {
     /* Password is in hash form */
     combo->uses_authentication_string_clause= true;
@@ -824,7 +824,10 @@ bool change_password(THD *thd, const char *host, const char *user,
     goto end;
   }
 
+  thd->variables.sql_mode&= ~MODE_PAD_CHAR_TO_FULL_LENGTH;
   ret= replace_user_table(thd, table, combo, 0, false, false, what_to_set);
+  thd->variables.sql_mode= old_sql_mode;
+
   if (ret)
   {
     mysql_mutex_unlock(&acl_cache->lock);
@@ -1556,22 +1559,32 @@ bool mysql_create_user(THD *thd, List <LEX_USER> &list, bool if_not_exists)
 
   if (some_users_created || (if_not_exists && !thd->is_error()))
   {
-    String *rlb= &thd->rewritten_query;
-    rlb->mem_free();
-    mysql_rewrite_create_alter_user(thd, rlb, &extra_users);
+    /*
+      Rewrite CREATE USER statements to use password hashes instead
+      of <secret> style obfuscation so it can be used in binlog. We
+      are rewriting to a private string rather than the public one on
+      the THD (thd->m_rewritten_query). This will save us from having
+      to acquire the lock to update the string on the THD. As
+      slow-logging (if enabled) will happen later and use the string
+      on the THD, the slow log will not contain the local rewrite
+      we're doing here, but the original one.
+    */
+    String rlb;
+
+    mysql_rewrite_create_alter_user(thd, &rlb, &extra_users);
 
     int ret= commit_owned_gtid_by_partial_command(thd);
 
     if (ret == 1)
     {
-      if (!thd->rewritten_query.length())
+      if (!rlb.length())
         result|= write_bin_log(thd, false, thd->query().str, thd->query().length,
                                transactional_tables);
-      else
-        result|= write_bin_log(thd, false,
-                               thd->rewritten_query.c_ptr_safe(),
-                               thd->rewritten_query.length(),
+      else {
+        result|= write_bin_log(thd, false, rlb.c_ptr_safe(), rlb.length(),
                                transactional_tables);
+        thd->swap_rewritten_query(rlb); // must come last!
+      }
     }
     else if (ret == -1)
       result|= -1;
@@ -1904,6 +1917,7 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
   bool is_privileged_user= false;
   bool rollback_whole_statement= false;
   std::set<LEX_USER *> extra_users;
+  std::set<LEX_USER *> reset_users;
   Acl_table_intact table_intact;
 
   DBUG_ENTER("mysql_alter_user");
@@ -2066,6 +2080,8 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
                   false);
       continue;
     }
+    if (what_to_alter & RESOURCE_ATTR)
+      reset_users.insert(tmp_user_from);
     some_user_altered= true;
     update_sctx_cache(thd->security_context(), acl_user,
                       user_from->alter_status.update_password_expired_column);
@@ -2084,20 +2100,29 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
 
   if (some_user_altered || (if_exists && !thd->is_error()))
   {
-    /* do query rewrite for ALTER USER */
-    String *rlb= &thd->rewritten_query;
-    rlb->mem_free();
-    mysql_rewrite_create_alter_user(thd, rlb, &extra_users);
+    /*
+      Rewrite ALTER USER statements to use password hashes instead
+      of <secret> style obfuscation so it can be used in binlog. We
+      are rewriting to a private string rather than the public one on
+      the THD (thd->m_rewritten_query). This will save us from having
+      to acquire the lock to update the string on the THD. As
+      slow-logging (if enabled) will happen later and use the string
+      on the THD, the slow log will not contain the local rewrite
+      we're doing here, but the original one.
+    */
+    String rlb;
+
+    mysql_rewrite_create_alter_user(thd, &rlb, &extra_users);
 
     int ret= commit_owned_gtid_by_partial_command(thd);
     if (ret == 1)
-      result|= (write_bin_log(thd, false,
-                              thd->rewritten_query.c_ptr_safe(),
-                              thd->rewritten_query.length(),
+      result|= (write_bin_log(thd, false, rlb.c_ptr_safe(), rlb.length(),
                               table->file->has_transactions()) != 0);
 
     else if (ret == -1)
       result|= -1;
+
+    thd->swap_rewritten_query(rlb); // must come last!
   }
 
   lock.unlock();
@@ -2108,7 +2133,17 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
                                    rollback_whole_statement);
 
   if (some_user_altered && !result)
+  {
+    std::set<LEX_USER *>::iterator one_user;
+    LEX_USER *ext_user;
+    for (one_user= reset_users.begin(); one_user != reset_users.end(); one_user++)
+    {
+      LEX_USER *user= *one_user;
+      if ((ext_user= get_current_user(thd, user)))
+        reset_mqh(ext_user, false);
+    }
     acl_notify_htons(thd, thd->query().str, thd->query().length);
+  }
 
   /* Restore the state of binlog format */
   DBUG_ASSERT(!thd->is_current_stmt_binlog_format_row());
@@ -2119,5 +2154,3 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
 
 
 #endif
-
-
