@@ -2,13 +2,20 @@
    Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -342,8 +349,7 @@ extern "C" void *handle_connection(void *arg)
 #endif /* HAVE_PSI_THREAD_INTERFACE */
 
 #ifdef WITH_WSREP
-    bool is_applier= (WSREP(thd) && thd->wsrep_applier);
-    my_thread_id tid= thd->thread_id();
+    const bool wsrep_applier_thread= (WSREP(thd) && thd->wsrep_applier);
 #endif /* WITH_WSREP */
 
     delete thd;
@@ -354,18 +360,11 @@ extern "C" void *handle_connection(void *arg)
 
     channel_info= NULL;
 #ifdef WITH_WSREP
-    if (is_applier)
-    {
-      WSREP_DEBUG("avoiding thread re-use for applier, thd: %u", tid);
-      channel_info = NULL;
-    }
-    else
-    {
+    if (wsrep_applier_thread)
+      break;
 #endif /* WITH_WSREP */
+
     channel_info= Per_thread_connection_handler::block_until_new_connection();
-#ifdef WITH_WSREP
-    }
-#endif /* WITH_WSREP */
     if (channel_info == NULL)
       break;
     pthread_reused= true;
