@@ -1,13 +1,20 @@
-/* Copyright (c) 2005, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2005, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -19,6 +26,14 @@
 #include "my_global.h"
 #include "auth/sql_security_ctx.h"  // Security_context
 
+/* max size of log messages (error log, plugins' logging, general log) */
+#ifdef WITH_WSREP
+/* make this bigger to accomodate wsrep_provider_options */
+static const uint MAX_LOG_BUFFER_SIZE= 2048;
+#else
+static const uint MAX_LOG_BUFFER_SIZE= 1024;
+#endif /* WITH_WSREP */
+
 struct TABLE_LIST;
 
 ////////////////////////////////////////////////////////////
