@@ -2,22 +2,31 @@
 #define SQL_JOIN_CACHE_INCLUDED
 
 #include "sql_executor.h"
+#include "mem_root_array.h"
 
-/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
+
 /** @file Join buffer classes */
 
 /* 
@@ -30,6 +39,7 @@
 #define CACHE_VARSTR1   3        /* short string value (length takes 1 byte) */ 
 #define CACHE_VARSTR2   4        /* long string value (length takes 2 bytes) */
 
+
 /*
   The CACHE_FIELD structure used to describe fields of records that
   are written into a join cache buffer from record buffers and backward.
@@ -97,6 +107,11 @@ private:
   uint size_of_rec_len;
   /* Size of the offset of a field within a record in the cache */   
   uint size_of_fld_ofs;
+  /**
+    In init() there are several uses of TABLE::tmp_set, so one tmp_set isn't
+    enough; this one is specific of generated column handling.
+  */
+  Mem_root_array<MY_BITMAP*, true> save_read_set_for_gcol;
 
 protected:
        
@@ -344,6 +359,8 @@ protected:
   /* Shall calculate how much space is remaining in the join buffer */ 
   virtual ulong rem_space() 
   { 
+    DBUG_ASSERT(end_pos >= buff);
+    DBUG_ASSERT(buff_size >= ulong(end_pos - buff));
     return static_cast<ulong>(
       std::max<long>(buff_size-(end_pos-buff)-aux_buff_size, 0L)
     );
@@ -462,8 +479,10 @@ public:
     linked.
   */
   JOIN_CACHE(JOIN *j, QEP_TAB *qep_tab_arg, JOIN_CACHE *prev)
-    : QEP_operation(qep_tab_arg), join(j), buff(NULL), prev_cache(prev),
-    next_cache(NULL)
+    : QEP_operation(qep_tab_arg),
+      save_read_set_for_gcol(qep_tab_arg->table()->in_use->mem_root),
+      join(j), buff(NULL), prev_cache(prev),
+      next_cache(NULL)
     {
       if (prev_cache)
         prev_cache->next_cache= this;
@@ -814,6 +833,9 @@ protected:
   */ 
   ulong rem_space() 
   { 
+    DBUG_ASSERT(last_key_entry >= end_pos);
+    DBUG_ASSERT(buff_size >= aux_buff_size);
+    DBUG_ASSERT(ulong(last_key_entry - end_pos) >= aux_buff_size);
     return static_cast<ulong>(
       std::max<long>(last_key_entry - end_pos-aux_buff_size, 0L)
     );
