@@ -1,13 +1,20 @@
 /* Copyright (c) 2012, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -15,7 +22,9 @@
 
 #include "table_cache.h"
 #include "sql_test.h" // lock_descriptions[]
-
+#ifdef WITH_WSREP
+#include "wsrep_thd.h" // wsrep_thd_is_BF
+#endif /* WITH_WSREP */
 
 /**
   Container for all table cache instances in the system.
@@ -362,7 +371,18 @@ void Table_cache_manager::free_table(THD *thd,
 
 #ifndef DBUG_OFF
       if (remove_type == TDC_RT_REMOVE_ALL)
+#ifndef WITH_WSREP
         DBUG_ASSERT(cache_el[i]->used_tables.is_empty());
+#else
+      {
+        if (!cache_el[i]->used_tables.is_empty())
+          {
+            WSREP_DEBUG("ASSERT skipped, thd: %u mode %d conf %d query %s",
+                        thd->thread_id(), thd->wsrep_exec_mode,
+                        thd->wsrep_conflict_state, thd->query().str);
+          }
+      }
+#endif
       else if (remove_type == TDC_RT_REMOVE_NOT_OWN ||
                remove_type == TDC_RT_REMOVE_NOT_OWN_KEEP_SHARE)
       {
@@ -370,15 +390,27 @@ void Table_cache_manager::free_table(THD *thd,
         while ((table= it2++))
         {
 #ifdef WITH_WSREP
-          /* if thd was BF aborted, exclusive locks were canceled,
-          thus others can use table */
-
+          mysql_mutex_lock(&thd->LOCK_wsrep_thd);
           if (table->in_use != thd &&
-               table->in_use->wsrep_conflict_state != MUST_ABORT)
+              !(table->in_use->wsrep_conflict_state == MUST_ABORT  ||
+                table->in_use->wsrep_conflict_state == ABORTING    ||
+                table->in_use->wsrep_conflict_state == ABORTED     ||
+                table->in_use->wsrep_conflict_state == MUST_REPLAY ||
+                table->in_use->wsrep_conflict_state == CERT_FAILURE))
+          {
+            mysql_mutex_unlock(&thd->LOCK_wsrep_thd);
+            WSREP_DEBUG("ASSERT  BF me %d BF other %d conf %d",
+                        wsrep_thd_is_BF(thd, FALSE),
+                        wsrep_thd_is_BF(table->in_use, FALSE),
+                        table->in_use->wsrep_conflict_state);
 #else
           if (table->in_use != thd)
 #endif /* WITH_WSREP */
             DBUG_ASSERT(0);
+#ifdef WITH_WSREP
+          }
+          mysql_mutex_unlock(&thd->LOCK_wsrep_thd);
+#endif /* WITH_WSREP */
         }
       }
 #endif
