@@ -39,8 +39,12 @@ enum wsrep_trx_status wsrep_run_wsrep_commit(THD *thd, handlerton *hton,
 void wsrep_cleanup_transaction(THD *thd)
 {
   if (!WSREP(thd)) return;
+
   // Enabling it can cause password to appear in log.
   // WSREP_DEBUG("wsrep_cleanup_transaction %s", thd->query().str);
+  DBUG_ASSERT(thd->wsrep_conflict_state != MUST_REPLAY &&
+              thd->wsrep_conflict_state != REPLAYING);
+
   if (wsrep_emulate_bin_log) wsrep_thd_binlog_trx_reset(thd);
   thd->wsrep_ws_handle.trx_id= WSREP_UNDEFINED_TRX_ID;
   thd->wsrep_trx_meta.gtid= WSREP_GTID_UNDEFINED;
@@ -50,6 +54,7 @@ void wsrep_cleanup_transaction(THD *thd)
   thd->wsrep_affected_rows= 0;
   thd->wsrep_void_applier_trx= true;
   thd->wsrep_skip_wsrep_GTID= false;
+  thd->wsrep_skip_SE_checkpoint= false;
   return;
 }
 
@@ -210,9 +215,31 @@ void wsrep_post_commit(THD* thd, bool all)
      /* non-InnoDB statements may have populated events in stmt cache 
 	=> cleanup 
      */
-     WSREP_DEBUG("Cleaning up wsrep-transaction for local query: %s",
-                 WSREP_QUERY(thd));
-     wsrep_cleanup_transaction(thd);
+     if (thd->wsrep_conflict_state != MUST_REPLAY)
+     {
+       WSREP_DEBUG("cleanup transaction for LOCAL_STATE: %s",
+                   WSREP_QUERY(thd));
+     }
+
+     /*
+       Run post-rollback hook to clean up in the case if
+       some keys were populated for the transaction in provider
+       but during commit time there was no write set to replicate.
+       This may happen when client sets the SAVEPOINT and immediately
+       rolls back to savepoint after first operation. 
+      */
+     if (all && thd->wsrep_conflict_state != MUST_REPLAY &&
+         thd->wsrep_conflict_state != REPLAYING &&
+         wsrep->post_rollback(wsrep, &thd->wsrep_ws_handle))
+     {
+         WSREP_WARN("post_rollback fail: %llu %d",
+                    (long long)thd->thread_id(), thd->get_stmt_da()->status());
+     }
+     if (thd->wsrep_conflict_state != MUST_REPLAY &&
+         thd->wsrep_conflict_state != REPLAYING)
+     {
+       wsrep_cleanup_transaction(thd);
+     }
      break;
    }
  case REPL_RECV:
@@ -222,8 +249,13 @@ void wsrep_post_commit(THD* thd, bool all)
      assigned we need to update commit monitor by enter and leaving it.
      Empty transaction use case: CTAS, pre-executed action on pxc-node
      acting as slave to an independent master. */
+
+     /* Also note that replay transaction are executed with exec_code=REPL_RECV
+     So ensure we check for the wsrep_conflict_state. */
+
      if (thd->wsrep_void_applier_trx &&
          thd->wsrep_ws_handle.opaque &&
+         thd->wsrep_conflict_state != REPLAYING &&
          thd->wsrep_applier)
        wsrep->applier_pre_commit(wsrep, thd->wsrep_ws_handle.opaque);
 
