@@ -2379,13 +2379,13 @@ void wsrep_to_isolation_end(THD *thd)
       wsrep_get_query_state(req->wsrep_query_state),                           \
       wsrep_get_conflict_state(req->wsrep_conflict_state),                     \
       req->get_command(), req->lex->sql_command,                               \
-      (req->rewritten_query.length() ? req->rewritten_query.c_ptr_safe() : req->query().str), \
+      (req->rewritten_query().length() ? wsrep_thd_rewritten_query(req).c_ptr_safe() : req->query().str), \
       gra->thread_id(), (long long)wsrep_thd_trx_seqno(gra),                   \
       wsrep_get_exec_mode(gra->wsrep_exec_mode),                               \
       wsrep_get_query_state(gra->wsrep_query_state),                           \
       wsrep_get_conflict_state(gra->wsrep_conflict_state),                     \
       gra->get_command(), gra->lex->sql_command,                               \
-      (gra->rewritten_query.length() ? gra->rewritten_query.c_ptr_safe() : gra->query().str));
+      (gra->rewritten_query().length() ? wsrep_thd_rewritten_query(gra).c_ptr_safe() : gra->query().str))
 
 bool
 wsrep_grant_mdl_exception(const MDL_context *requestor_ctx,
@@ -2398,6 +2398,26 @@ wsrep_grant_mdl_exception(const MDL_context *requestor_ctx,
   THD *granted_thd  = ticket->get_ctx()->wsrep_get_thd();
   bool ret          = FALSE;
 
+  /* If the said thd/transaction is already aborted and is in process of
+  rolling back avoid re-aborting it. During first abort (initated through
+  can_grant) bf-query-id is updated in trx->wsrep_killed_by_query field.
+  This field is reset to 0 on successful rollback. If before transaction
+  completes rollback flow (post resetting value to 0) and second attempt to kill
+  the same transaction is done it will reset the said field
+  (trx->wsrep_killed_by_query) one more time but this time there is no post
+  rollback action to clear it up.
+
+  Other approach of solving it could be clearing this field as part of
+  wsrep_cleanup_transaction but this action of double killing is anyway
+  redundant so better skip it. */
+  if (granted_thd->wsrep_conflict_state == MUST_ABORT)
+  {
+    /* Granted thd is already scheduled for abort.
+    No point in repeating the action .*/
+    WSREP_DEBUG("state of granted_thd: %d\n", granted_thd->killed);
+    return (ret);
+  }
+
   const char* schema= key->db_name();
   int schema_len= key->db_name_length();
 
@@ -2563,3 +2583,17 @@ const char* wsrep_get_wsrep_status(wsrep_status status)
 
   return "NULL";
 }
+
+/*
+ Returns non-const copy of thd->rewritten_query().
+
+ PXC code uses rewritten_query.c_ptr_safe() in several places.
+ As c_ptr_save() is non const method, it requires non const object.
+ Here we return copy of thd->rewritten_query() instead of const-casting
+ thd->rewritten_query() in place.
+ This is to avoid interference of PXC specific part with generic server part.
+ */
+String wsrep_thd_rewritten_query(THD *thd)
+{
+  return thd->rewritten_query();
+}
