@@ -235,7 +235,7 @@ extern void wsrep_close_client_connections(my_bool wait_to_end, bool server_shtu
 extern int  wsrep_wait_committing_connections_close(int wait_time);
 extern void wsrep_close_applier(THD *thd);
 extern void wsrep_wait_appliers_close(THD *thd);
-extern void wsrep_close_applier_threads(int count);
+extern void wsrep_close_applier_threads();
 extern void wsrep_kill_mysql(THD *thd);
 
 /* new defines */
@@ -250,6 +250,8 @@ extern void wsrep_prepend_PATH (const char* path);
 /* Provide a wrapper of the WSREP_ON macro for plugins to use */
 extern "C" bool wsrep_is_wsrep_on(void);
 
+/* return non-const copy of thd->rewritten_query() */
+extern String wsrep_thd_rewritten_query(THD *thd);
 
 /* Other global variables */
 extern wsrep_seqno_t wsrep_locked_seqno;
@@ -314,8 +316,8 @@ extern wsrep_seqno_t wsrep_locked_seqno;
   }
 
 #define WSREP_QUERY(thd)                                \
-  ((!opt_general_log_raw) && thd->rewritten_query.length()      \
-   ? thd->rewritten_query.c_ptr_safe() : thd->query().str)
+  ((!opt_general_log_raw) && thd->rewritten_query().length()      \
+   ? wsrep_thd_rewritten_query(thd).c_ptr_safe() : thd->query().str)
 
 extern my_bool wsrep_ready_get();
 extern void wsrep_ready_wait();
