@@ -17,6 +17,7 @@
 #include <sql_class.h>
 #include <sql_parse.h>
 #include <sql_reload.h>
+#include <mysql_version.h>
 #include "debug_sync.h"
 #include "wsrep_priv.h"
 #include "wsrep_thd.h"
@@ -44,6 +45,7 @@ extern const char wsrep_defaults_group_suffix[];
 #define WSREP_SST_OPT_CONF_SUFFIX "--defaults-group-suffix"
 #define WSREP_SST_OPT_PARENT   "--parent"
 #define WSREP_SST_OPT_BINLOG   "--binlog"
+#define WSREP_SST_OPT_VERSION  "--mysqld-version"
 
 // mysqldump-specific options
 #define WSREP_SST_OPT_USER     "--user"
@@ -352,7 +354,10 @@ void wsrep_sst_continue ()
   if (sst_needed)
   {
     WSREP_INFO("Signalling provider to continue on SST completion.");
-    wsrep_sst_received (wsrep, local_uuid, local_seqno, NULL, 0);
+    // local_uuid and local_seqno are global variables and are volatile
+    wsrep_uuid_t  const sst_uuid  = local_uuid;
+    wsrep_seqno_t const sst_seqno = local_seqno;
+    wsrep_sst_received (wsrep, sst_uuid, sst_seqno, NULL, 0);
   }
 }
 
@@ -682,11 +687,13 @@ static ssize_t sst_prepare_other (const char*  method,
                  WSREP_SST_OPT_DATA" '%s' "
                  WSREP_SST_OPT_CONF" '%s' "
                  WSREP_SST_OPT_CONF_SUFFIX" '%s' "
-                 WSREP_SST_OPT_PARENT" '%d'"
+                 WSREP_SST_OPT_PARENT" '%d' "
+                 WSREP_SST_OPT_VERSION" '%s' "
                  " %s '%s' ",
                  method, addr_in, mysql_real_data_home,
                  wsrep_defaults_file, wsrep_defaults_group_suffix,
-                 (int)getpid(), binlog_opt, binlog_opt_val);
+                 (int)getpid(), MYSQL_SERVER_VERSION MYSQL_SERVER_SUFFIX_DEF,
+                 binlog_opt, binlog_opt_val);
   my_free(binlog_opt_val);
 
   if (ret < 0 || ret >= cmd_len)
@@ -980,11 +987,14 @@ static int sst_donate_mysqldump (const char*         addr,
                      WSREP_SST_OPT_LPORT" '%u' "
                      WSREP_SST_OPT_SOCKET" '%s' "
                      WSREP_SST_OPT_CONF" '%s' "
-                     WSREP_SST_OPT_GTID" '%s:%lld'"
+                     WSREP_SST_OPT_GTID" '%s:%lld' "
+                     WSREP_SST_OPT_VERSION" '%s' "
                      "%s",
                      addr, mysqld_port, mysqld_unix_port,
                      wsrep_defaults_file, uuid_str,
-                     (long long)seqno, bypass ? " " WSREP_SST_OPT_BYPASS : "");
+                     (long long)seqno,
+                     MYSQL_SERVER_VERSION MYSQL_SERVER_SUFFIX_DEF,
+                     bypass ? " " WSREP_SST_OPT_BYPASS : "");
 
   if (ret < 0 || ret >= cmd_len)
   {
@@ -1016,7 +1026,7 @@ static int run_sql_command(THD *thd, const char *query)
     return -1;
   }
 
-  mysql_parse(thd, &ps);
+  mysql_parse(thd, &ps, false);
   if (thd->is_error())
   {
     int const err= thd->get_stmt_da()->mysql_errno();
@@ -1273,11 +1283,13 @@ static int sst_donate_other (const char*   method,
                  WSREP_SST_OPT_DATA" '%s' "
                  WSREP_SST_OPT_CONF" '%s' "
                  WSREP_SST_OPT_CONF_SUFFIX" '%s' "
+                 WSREP_SST_OPT_VERSION" '%s' "
                  " %s '%s' "
-                 WSREP_SST_OPT_GTID" '%s:%lld'"
+                 WSREP_SST_OPT_GTID" '%s:%lld' "
                  "%s",
                  method, addr, mysqld_unix_port, mysql_real_data_home,
                  wsrep_defaults_file, wsrep_defaults_group_suffix,
+                 MYSQL_SERVER_VERSION MYSQL_SERVER_SUFFIX_DEF,
                  binlog_opt, binlog_opt_val,
                  uuid, (long long) seqno,
                  bypass ? " " WSREP_SST_OPT_BYPASS : "");
