@@ -27,6 +27,7 @@
 
 #include <unistd.h>   // pipe()
 #include <errno.h>    // errno
+#include <signal.h>   // sigemptyset(), sigaddset()
 #include <string.h>   // strerror()
 #include <sys/wait.h> // waitpid()
 #include <sys/types.h>
@@ -96,6 +97,17 @@ wsrep_prepend_PATH (const char* path)
                  "State snapshot transfer may not be working.");
 }
 
+/**
+ * This is a wrapper around the WSREP_ON macro.  This allows this check
+ * to be called from plugins.
+ */
+extern "C" bool
+wsrep_is_wsrep_on(void)
+{
+  return WSREP_ON;
+}
+
+
 namespace wsp
 {
 
@@ -934,3 +946,18 @@ size_t wsrep_guess_ip (char* buf, size_t buf_len)
 
   return 0;
 }
+
+/* returns the length of the host part of the address string */
+size_t wsrep_host_len(const char* const addr, size_t const addr_len)
+{
+  // check for IPv6 notation first
+  const char* const bracket= ('[' == addr[0] ? strchr(addr, ']') : NULL);
+
+  if (bracket) { // IPv6
+    return (bracket - addr + 1);
+  }
+  else { // host part ends at ':' or end of string
+    const char* const colon= strchr(addr, ':');
+    return (colon ? colon - addr : addr_len);
+  }
+}
