@@ -1,14 +1,22 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2015, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1996, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; version 2 of the License.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+This program is also distributed with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have included with MySQL.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
@@ -33,6 +41,39 @@ Created 2/23/1996 Heikki Tuuri
 #include "rem0cmp.h"
 #include "trx0trx.h"
 
+/** Updates fragmentation statistics for a single page transition.
+@param[in]	page			the current page being processed
+@param[in]	page_no			page number to move to (next_page_no
+					if forward_direction is true,
+					prev_page_no otherwise.
+@param[in]	forward_direction	move direction: true means moving
+					forward, false - backward. */
+static
+void
+btr_update_scan_stats(const page_t* page, ulint page_no, bool forward_direction)
+{
+	fragmentation_stats_t	stats;
+	memset(&stats, 0, sizeof(stats));
+	ulint	extracted_page_no = page_get_page_no(page);
+	ulint	delta = forward_direction ?
+		page_no - extracted_page_no :
+		extracted_page_no - page_no;
+
+	if (delta == 1) {
+		++stats.scan_pages_contiguous;
+	} else {
+		++stats.scan_pages_disjointed;
+	}
+	stats.scan_pages_total_seek_distance +=
+		extracted_page_no > page_no ?
+		extracted_page_no - page_no :
+		page_no - extracted_page_no;
+
+	stats.scan_data_size += page_get_data_size(page);
+	stats.scan_deleted_recs_size +=
+		page_header_get_field(page, PAGE_GARBAGE);
+	thd_add_fragmentation_stats(current_thd, &stats);
+}
 /**************************************************************//**
 Allocates memory for a persistent cursor object and initializes the cursor.
 @return own: persistent cursor */
@@ -317,7 +358,7 @@ btr_pcur_restore_position_func(
 
 				ut_ad(!cmp_rec_rec(cursor->old_rec,
 						   rec, offsets1, offsets2,
-						   index));
+						   index,page_is_spatial_non_leaf(rec, index)));
 				mem_heap_free(heap);
 #endif /* UNIV_DEBUG */
 				return(TRUE);
@@ -446,11 +487,16 @@ btr_pcur_move_to_next_page(
 
 	buf_block_t*	block = btr_pcur_get_block(cursor);
 
+	btr_update_scan_stats(page, next_page_no, true /* forward */);
+
 	next_block = btr_block_get(
 		page_id_t(block->page.id.space(), next_page_no),
 		block->page.size, mode,
 		btr_pcur_get_btr_cur(cursor)->index, mtr);
 
+        if (!next_block && !btr_pcur_get_btr_cur(cursor)->index->table->is_readable())
+           return; /* decryption failure */
+
 	next_page = buf_block_get_frame(next_block);
 
 	SRV_CORRUPT_TABLE_CHECK(next_page,
@@ -464,9 +510,23 @@ btr_pcur_move_to_next_page(
 	});
 
 #ifdef UNIV_BTR_DEBUG
-	ut_a(page_is_comp(next_page) == page_is_comp(page));
-	ut_a(btr_page_get_prev(next_page, mtr)
-	     == btr_pcur_get_block(cursor)->page.id.page_no());
+	if (!cursor->import_ctx) {
+		ut_a(page_is_comp(next_page) == page_is_comp(page));
+		ut_a(btr_page_get_prev(next_page, mtr)
+			== btr_pcur_get_block(cursor)->page.id.page_no());
+	}
+	else {
+		if (page_is_comp(next_page) != page_is_comp(page)
+			|| btr_page_get_prev(next_page, mtr) !=
+			btr_pcur_get_block(cursor)->page.id.page_no()) {
+			/* next page does not contain valid previous page
+			number, next page is corrupted, can't move cursor
+			to the next page */
+			cursor->import_ctx->is_error = true;
+		}
+		DBUG_EXECUTE_IF("ib_import_page_corrupt",
+				cursor->import_ctx->is_error = true;);
+	}
 #endif /* UNIV_BTR_DEBUG */
 
 	btr_leaf_page_release(btr_pcur_get_block(cursor), mode, mtr);
@@ -532,6 +592,10 @@ btr_pcur_move_backward_from_page(
 	no left block that is pinned that needs to be released. */
 	if (!dict_table_is_intrinsic(
 		btr_cur_get_index(btr_pcur_get_btr_cur(cursor))->table)) {
+		if (prev_page_no != FIL_NULL) {
+			btr_update_scan_stats(page, prev_page_no,
+				false /* backward */);
+		}
 
 		if (prev_page_no == FIL_NULL) {
 		} else if (btr_pcur_is_before_first_on_page(cursor)) {
