@@ -1,14 +1,22 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2016, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2018, Oracle and/or its affiliates. All Rights Reserved.
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; version 2 of the License.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+This program is also distributed with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have included with MySQL.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
@@ -743,6 +751,9 @@ buf_flush_dirty_pages(
 		err = buf_flush_or_remove_pages(
 			buf_pool, id, observer, flush, trx);
 
+	        ut_ad(err == DB_INTERRUPTED || err == DB_FAIL
+	              || buf_pool_get_dirty_pages_count(buf_pool, id, observer) == 0);
+
 		mutex_exit(&buf_pool->LRU_list_mutex);
 
 		ut_ad(buf_flush_validate(buf_pool));
@@ -765,8 +776,6 @@ buf_flush_dirty_pages(
 
 	} while (err == DB_FAIL);
 
-	ut_ad(err == DB_INTERRUPTED
-	      || buf_pool_get_dirty_pages_count(buf_pool, id, observer) == 0);
 }
 
 /******************************************************************//**
@@ -1362,16 +1371,16 @@ the error log if more than two seconds have been spent already.
 */
 static
 void
-buf_LRU_handle_lack_of_free_blocks(ulint n_iterations, ulint started_ms,
+buf_LRU_handle_lack_of_free_blocks(ulint n_iterations, ib_time_monotonic_ms_t started_ms,
 				   ulint flush_failures,
 				   bool *mon_value_was,
 				   bool *started_monitor)
 {
-	static ulint last_printout_ms = 0;
+	static ib_time_monotonic_ms_t last_printout_ms = 0;
 
 	/* Legacy algorithm started warning after at least 2 seconds, we
 	emulate	this. */
-	const ulint current_ms = ut_time_ms();
+	const ib_time_monotonic_ms_t current_ms = ut_time_monotonic_ms();
 
 	if ((current_ms > started_ms + 2000)
 	    && (current_ms > last_printout_ms + 2000)
@@ -1449,7 +1458,7 @@ buf_LRU_get_free_block(
 	ulint		flush_failures	= 0;
 	bool		mon_value_was	= false;
 	bool		started_monitor	= false;
-	ulint		started_ms	= 0;
+	ib_time_monotonic_ms_t started_ms = 0;
 
 	ut_ad(!mutex_own(&buf_pool->LRU_list_mutex));
 
@@ -1493,7 +1502,7 @@ loop:
 	}
 
 	if (!started_ms)
-		started_ms = ut_time_ms();
+		started_ms = ut_time_monotonic_ms();
 
 	MONITOR_INC( MONITOR_LRU_GET_FREE_LOOPS );
 
@@ -2152,7 +2161,7 @@ not_freed:
 	}
 
 	if (b) {
-		memcpy(static_cast<void*>(b), bpage, sizeof *b);
+		new (b) buf_page_t(*bpage);
 	}
 
 	if (!buf_LRU_block_remove_hashed(bpage, zip)) {
