@@ -2,13 +2,21 @@
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; version 2 of the License.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+This program is also distributed with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have included with MySQL.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
@@ -106,6 +114,13 @@ flag is cleared and the x-lock released by an i/o-handler thread.
 @param[in] mode		BUF_READ_IBUF_PAGES_ONLY, ...,
 @param[in] page_id	page id
 @param[in] unzip	true=request uncompressed page
+@param[in] should_buffer
+			whether to buffer an aio request.
+			AIO read ahead uses this. If you plan to
+			use this parameter, make sure you remember to
+			call os_aio_dispatch_read_array_submit()
+			when you're ready to commit all your requests.
+
 @return 1 if a read request was queued, 0 if the page already resided
 in buf_pool, or if the page is in the doublewrite buffer blocks in
 which case it is never read into the pool, or if the tablespace does
@@ -120,12 +135,11 @@ buf_read_page_low(
 	const page_id_t&	page_id,
 	const page_size_t&	page_size,
 	bool			unzip,
-	trx_t*			trx)
+	trx_t*			trx,
+	bool			should_buffer)
 {
 	buf_page_t*	bpage;
 
-	ut_ad(!trx || trx->take_stats);
-
 	*err = DB_SUCCESS;
 
 	if (page_id.space() == TRX_SYS_SPACE
@@ -154,52 +168,6 @@ buf_read_page_low(
 	bpage = buf_page_init_for_read(err, mode, page_id, page_size, unzip);
 
 	if (bpage == NULL) {
-		/* bugfix: http://bugs.mysql.com/bug.php?id=43948 */
-		if (recv_recovery_is_on() && *err == DB_TABLESPACE_DELETED) {
-			/* hashed log recs must be treated here */
-			recv_addr_t*    recv_addr;
-
-			mutex_enter(&(recv_sys->mutex));
-
-			if (!recv_sys->apply_log_recs) {
-				mutex_exit(&(recv_sys->mutex));
-				goto not_to_recover;
-			}
-
-			/* recv_get_fil_addr_struct() */
-			recv_addr = (recv_addr_t*)
-				HASH_GET_FIRST(recv_sys->addr_hash,
-					       hash_calc_hash(
-						       ut_fold_ulint_pair(
-							       page_id.space(),
-							       page_id.page_no()),
-						recv_sys->addr_hash));
-			while (recv_addr) {
-				if ((recv_addr->space == page_id.space())
-				    && (recv_addr->page_no
-					== page_id.page_no())) {
-					break;
-				}
-				recv_addr = (recv_addr_t*)HASH_GET_NEXT(addr_hash, recv_addr);
-			}
-
-			if ((recv_addr == NULL)
-			    || (recv_addr->state == RECV_BEING_PROCESSED)
-			    || (recv_addr->state == RECV_PROCESSED)) {
-				mutex_exit(&(recv_sys->mutex));
-				goto not_to_recover;
-			}
-
-			ib::info() << " (cannot find space: "
-				   << page_id.space() << ")";
-			recv_addr->state = RECV_PROCESSED;
-
-			ut_a(recv_sys->n_addrs);
-			recv_sys->n_addrs--;
-
-			mutex_exit(&(recv_sys->mutex));
-		}
-not_to_recover:
 
 		return(0);
 	}
@@ -228,16 +196,30 @@ not_to_recover:
 		dst = ((buf_block_t*) bpage)->frame;
 	}
 
+	/* This debug code is only for 5.7. In trunk, with newDD,
+	the space->name is no longer same as table name. */
+	DBUG_EXECUTE_IF("innodb_invalid_read_after_truncate",
+		fil_space_t*	space = fil_space_get(page_id.space());
+
+		if (space != NULL && strcmp(space->name, "test/t1") == 0
+		    && page_id.page_no() == space->size - 1) {
+			type = IORequest::READ;
+			sync = true;
+		}
+	);
+
 	IORequest	request(type | IORequest::READ);
 
 	*err = _fil_io(
 		request, sync, page_id, page_size, 0, page_size.physical(),
-		dst, bpage, trx);
+		dst, bpage, trx, should_buffer);
 
 	if (sync) {
 		thd_wait_end(NULL);
 	}
 
+	bpage->is_corrupt = bpage->encrypted = false;
+
 	if (*err != DB_SUCCESS) {
 		if (*err == DB_TABLESPACE_TRUNCATED) {
 			/* Remove the page which is outside the
@@ -255,9 +237,12 @@ not_to_recover:
 			   || *err == DB_TABLESPACE_DELETED) {
 			buf_read_page_handle_error(bpage);
 			return(0);
-		}
+		} else if (*err == DB_IO_DECRYPT_FAIL)
+                {
+                        bpage->encrypted= true;
+                }
 
-		SRV_CORRUPT_TABLE_CHECK(*err == DB_SUCCESS,
+		SRV_CORRUPT_TABLE_CHECK(*err == DB_SUCCESS || bpage->encrypted,
 					bpage->is_corrupt = true;);
 
 	}
@@ -265,7 +250,10 @@ not_to_recover:
 	if (sync) {
 		/* The i/o is already completed when we arrive from
 		fil_read */
-		if (!buf_page_io_complete(bpage)) {
+
+		*err = buf_page_io_complete(bpage);
+
+		if (*err != DB_SUCCESS) {
 			return(0);
 		}
 	}
@@ -306,8 +294,6 @@ buf_read_ahead_random(
 	const ulint	buf_read_ahead_random_area
 				= BUF_READ_AHEAD_AREA(buf_pool);
 
-	ut_ad(!trx || trx->take_stats);
-
 	if (!srv_random_read_ahead) {
 		/* Disabled by user */
 		return(0);
@@ -350,8 +336,6 @@ buf_read_ahead_random(
 				size += os_file_get_size(node->handle)
 					/ page_size.physical();
 			}
-
-			ut_ad(size == space->size);
 		}
 #endif /* UNIV_DEBUG */
 
@@ -374,6 +358,17 @@ buf_read_ahead_random(
 	that is, reside near the start of the LRU list. */
 
 	for (i = low; i < high; i++) {
+		/* This debug code is only for 5.7. In trunk, with newDD,
+		the space->name is no longer same as table name. */
+		DBUG_EXECUTE_IF("innodb_invalid_read_after_truncate",
+			fil_space_t*	space = fil_space_get(page_id.space());
+
+			if (space != NULL
+			    && strcmp(space->name, "test/t1") == 0) {
+				high = space->size;
+				goto read_ahead;
+			}
+		);
 
 		rw_lock_t* hash_lock;
 
@@ -427,14 +422,22 @@ read_ahead:
 				&err, false,
 				IORequest::DO_NOT_WAKE,
 				ibuf_mode,
-				cur_page_id, page_size, false, trx);
+				cur_page_id, page_size, false, trx, false);
 
-			if (err == DB_TABLESPACE_DELETED) {
-				ib::warn() << "Random readahead trying to"
+			switch (err) {
+			case DB_SUCCESS:
+			case DB_TABLESPACE_TRUNCATED:
+			case DB_ERROR:
+			case DB_IO_DECRYPT_FAIL:
+				break;
+			case DB_TABLESPACE_DELETED:
+				ib::info() << "Random readahead trying to"
 					" access page " << cur_page_id
 					<< " in nonexisting or"
 					" being-dropped tablespace";
 				break;
+			default:
+				ut_error;
 			}
 		}
 	}
@@ -467,17 +470,19 @@ an exclusive lock on the buffer frame. The flag is cleared and the x-lock
 released by the i/o-handler thread.
 @param[in]	page_id		page id
 @param[in]	page_size	page size
-@return TRUE if page has been read in, FALSE in case of failure */
-ibool
+@retval DB_SUCCESS if the page was read and is not corrupted,
+@retval DB_PAGE_CORRUPTED if page based on checksum check is corrupted,
+@retval DB_DECRYPTION_FAILED if page post encryption checksum matches but
+after decryption normal page checksum does not match.
+@retval DB_TABLESPACE_DELETED if tablespace .ibd file is missing */
+dberr_t
 buf_read_page(
 	const page_id_t&	page_id,
 	const page_size_t&	page_size,
 	trx_t*			trx)
 {
 	ulint		count;
-	dberr_t		err;
-
-	ut_ad(!trx || trx->take_stats);
+	dberr_t		err = DB_SUCCESS;
 
 	/* We do synchronous IO because our AIO completion code
 	is sub-optimal. See buf_page_io_complete(), we have to
@@ -487,7 +492,7 @@ buf_read_page(
 
 	count = buf_read_page_low(
 		&err, true,
-		0, BUF_READ_ANY_PAGE, page_id, page_size, false, trx);
+		0, BUF_READ_ANY_PAGE, page_id, page_size, false, trx, false);
 
 	srv_stats.buf_pool_reads.add(count);
 
@@ -499,7 +504,7 @@ buf_read_page(
 	/* Increment number of I/O operations used for LRU policy. */
 	buf_LRU_stat_inc_io();
 
-	return(count > 0);
+	return(err);
 }
 
 /** High-level function which reads a page asynchronously from a file to the
@@ -523,7 +528,7 @@ buf_read_page_background(
 		&err, sync,
 		IORequest::DO_NOT_WAKE | IORequest::IGNORE_MISSING,
 		BUF_READ_ANY_PAGE,
-		page_id, page_size, false, NULL);
+		page_id, page_size, false, NULL, false);
 
 	srv_stats.buf_pool_reads.add(count);
 
@@ -587,8 +592,6 @@ buf_read_ahead_linear(
 		= BUF_READ_AHEAD_AREA(buf_pool);
 	ulint		threshold;
 
-	ut_ad(!trx || trx->take_stats);
-
 	/* check if readahead is disabled */
 	if (!srv_read_ahead_threshold) {
 		return(0);
@@ -802,17 +805,33 @@ buf_read_ahead_linear(
 			count += buf_read_page_low(
 				&err, false,
 				IORequest::DO_NOT_WAKE,
-				ibuf_mode, cur_page_id, page_size, false, trx);
+				ibuf_mode, cur_page_id, page_size, false,
+				trx, true);
 
-			if (err == DB_TABLESPACE_DELETED) {
-				ib::warn() << "linear readahead trying to"
+			switch (err) {
+			case DB_TABLESPACE_DELETED:
+                        	ib::warn() << "linear readahead trying to"
 					" access page "
 					<< page_id_t(page_id.space(), i)
 					<< " in nonexisting or being-dropped"
 					" tablespace";
+				break;
+			case DB_SUCCESS:
+			case DB_TABLESPACE_TRUNCATED:
+			case DB_ERROR:
+				break;
+			case DB_PAGE_CORRUPTED:
+			case DB_DECRYPTION_FAILED:
+				ib::error() << "linear readahead failed to"
+					" read or decrypt "
+					<< page_id_t(page_id.space(), i);
+				break;
+			default:
+				ut_error;
 			}
 		}
 	}
+	os_aio_dispatch_read_array_submit();
 
 	/* In simulated aio we wake the aio handler threads only after
 	queuing all aio requests, in native aio the following call does
@@ -889,13 +908,26 @@ buf_read_ibuf_merge_pages(
 				  sync && (i + 1 == n_stored),
 				  0,
 				  BUF_READ_ANY_PAGE, page_id, page_size,
-				  true, NULL);
+				  true, NULL, false);
 
-		if (err == DB_TABLESPACE_DELETED) {
+		switch(err) {
+		case DB_SUCCESS:
+		case DB_TABLESPACE_TRUNCATED:
+		case DB_ERROR:
+			break;
+		case DB_TABLESPACE_DELETED:
 			/* We have deleted or are deleting the single-table
 			tablespace: remove the entries for that page */
 			ibuf_merge_or_delete_for_page(NULL, page_id,
 						      &page_size, FALSE);
+			// fallthrough
+		case DB_PAGE_CORRUPTED:
+		case DB_DECRYPTION_FAILED:
+			ib::error() << "Failed to read or decrypt " << page_id
+				<< " for change buffer merge";
+			break;
+		default:
+			ut_error;
 		}
 	}
 
@@ -928,40 +960,7 @@ buf_read_recv_pages(
 	fil_space_t*		space	= fil_space_get(space_id);
 
 	if (space == NULL) {
-		/* the log records should be treated here same reason
-		for http://bugs.mysql.com/bug.php?id=43948 */
-
-		if (recv_recovery_is_on()) {
-			mutex_enter(&(recv_sys->mutex));
-
-			if (!recv_sys->apply_log_recs) {
-				mutex_exit(&(recv_sys->mutex));
-				goto not_to_recover;
-			}
-
-			for (i = 0; i < n_stored; i++) {
-
-				recv_addr_t* recv_addr
-					= recv_get_fil_addr_struct(space_id,
-								   page_nos[i]);
-
-				if ((recv_addr == NULL)
-				    || (recv_addr->state == RECV_BEING_PROCESSED)
-				    || (recv_addr->state == RECV_PROCESSED)) {
-					continue;
-				}
-
-				recv_addr->state = RECV_PROCESSED;
-
-				ut_a(recv_sys->n_addrs);
-				recv_sys->n_addrs--;
-			}
-
-			mutex_exit(&(recv_sys->mutex));
-
-			ib::info() << " (cannot find space: " << space << ")";
-		}
-not_to_recover:
+		/* The tablespace is missing: do nothing */
 		return;
 	}
 
@@ -999,13 +998,21 @@ not_to_recover:
 				&err, true,
 				0,
 				BUF_READ_ANY_PAGE,
-				cur_page_id, page_size, true, NULL);
+				cur_page_id, page_size, true, NULL, false);
 		} else {
 			buf_read_page_low(
 				&err, false,
 				IORequest::DO_NOT_WAKE,
 				BUF_READ_ANY_PAGE,
-				cur_page_id, page_size, true, NULL);
+				cur_page_id, page_size, true, NULL, false);
+		}
+
+		if (err == DB_DECRYPTION_FAILED) {
+			ib::error() << "Recovery failed to decrypt page "
+				<< cur_page_id;
+		} else if (err == DB_PAGE_CORRUPTED) {
+			ib::error() << "Recovery failed due to corrupted page "
+				<< cur_page_id;
 		}
 	}
 
