@@ -2,13 +2,21 @@
 
 Copyright (c) 1996, 2018, Oracle and/or its affiliates. All Rights Reserved.
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; version 2 of the License.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+This program is also distributed with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have included with MySQL.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
@@ -506,8 +514,7 @@ dict_build_tablespace_for_table(
 
 		/* Determine the tablespace flags. */
 		bool	is_temp = dict_table_is_temporary(table);
-		bool	is_encrypted = (srv_tmp_tablespace_encrypt && is_temp)
-					|| dict_table_is_encrypted(table);
+		bool	is_encrypted = dict_table_is_encrypted(table);
 		bool	has_data_dir = DICT_TF_HAS_DATA_DIR(table->flags);
 		ulint	fsp_flags = dict_tf_to_fsp_flags(table->flags,
 							 is_temp,
@@ -1202,7 +1209,8 @@ dict_drop_index_tree_in_mem(
 	tablespace and the .ibd file is missing do nothing,
 	else free the all the pages */
 	if (root_page_no != FIL_NULL && found) {
-		btr_free(page_id_t(space, root_page_no), page_size);
+		btr_free(page_id_t(space, root_page_no), page_size,
+			 dict_table_is_intrinsic(index->table));
 	}
 }
 
@@ -1345,7 +1353,8 @@ dict_truncate_index_tree_in_mem(
 	tablespace objects. */
 
 	if (truncate) {
-		btr_free(page_id_t(space, root_page_no), page_size);
+		btr_free(page_id_t(space, root_page_no), page_size,
+			 dict_table_is_intrinsic(index->table));
 	}
 
 	mtr_start(&mtr);
@@ -2887,7 +2896,7 @@ InnoDB system table.
 @return	error code or DB_SUCCESS */
 dberr_t
 dict_create_add_zip_dict_reference(
-	ulint		table_id,	/*!< in: table id */
+	table_id_t	table_id,	/*!< in: table id */
 	ulint		column_pos,	/*!< in: column position */
 	ulint		dict_id,	/*!< in: dict id */
 	trx_t*		trx)		/*!< in/out: transaction */
@@ -2913,10 +2922,10 @@ dict_create_add_zip_dict_reference(
 @return	error code or DB_SUCCESS */
 dberr_t
 dict_create_get_zip_dict_id_by_reference(
-	ulint	table_id,	/*!< in: table id */
-	ulint	column_pos,	/*!< in: column position */
-	ulint*	dict_id,	/*!< out: dict id */
-	trx_t*	trx)		/*!< in/out: transaction */
+	table_id_t	table_id,	/*!< in: table id */
+	ulint		column_pos,	/*!< in: column position */
+	ulint*		dict_id,	/*!< out: dict id */
+	trx_t*		trx)		/*!< in/out: transaction */
 {
 	ut_ad(dict_id);
 
@@ -3247,8 +3256,8 @@ the data dictionary tables in the database.
 @return	error code or DB_SUCCESS */
 dberr_t
 dict_create_remove_zip_dict_references_for_table(
-	ulint	table_id,	/*!< in: table id */
-	trx_t*	trx)		/*!< in/out: transaction */
+	table_id_t	table_id,	/*!< in: table id */
+	trx_t*		trx)		/*!< in/out: transaction */
 {
 	pars_info_t* info = pars_info_create();
 
