@@ -1,14 +1,22 @@
 /*****************************************************************************
 
-Copyright (c) 2007, 2016, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2007, 2018, Oracle and/or its affiliates. All Rights Reserved.
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; version 2 of the License.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+This program is also distributed with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have included with MySQL.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
@@ -28,6 +36,7 @@ Created 2007/3/16 Sunny Bains.
 #include "fts0ast.h"
 #include "fts0pars.h"
 #include "fts0fts.h"
+#include "row0sel.h"
 
 /* The FTS ast visit pass. */
 enum fts_ast_visit_pass_t {
@@ -548,8 +557,7 @@ fts_ast_node_check_union(
 	fts_ast_node_t*	node)
 {
 	if (node->type == FTS_AST_LIST
-	    || node->type == FTS_AST_SUBEXP_LIST
-	    || node->type == FTS_AST_PARSER_PHRASE_LIST) {
+	    || node->type == FTS_AST_SUBEXP_LIST) {
 
 		for (node = node->list.head; node; node = node->next) {
 			if (!fts_ast_node_check_union(node)) {
@@ -562,7 +570,8 @@ fts_ast_node_check_union(
 		       || node->oper == FTS_EXIST)) {
 
 		return(false);
-	} else if (node->type == FTS_AST_TEXT) {
+	} else if (node->type == FTS_AST_TEXT
+		   || node->type == FTS_AST_PARSER_PHRASE_LIST) {
 		/* Distance or phrase search query. */
 		return(false);
 	}
@@ -593,6 +602,7 @@ fts_ast_visit(
 	bool			revisit = false;
 	bool			will_be_ignored = false;
 	fts_ast_visit_pass_t	visit_pass = FTS_PASS_FIRST;
+	trx_t*	trx = node->trx;
 
 	start_node = node->list.head;
 
@@ -691,6 +701,10 @@ fts_ast_visit(
 		}
 	}
 
+	if (trx_is_interrupted(trx)) {
+		return (DB_INTERRUPTED);
+	}
+
 	if (revisit) {
 		/* Exist pass processes the skipped FTS_EXIST operation. */
                 for (node = start_node;
