@@ -1,14 +1,22 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2017, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1994, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; version 2 of the License.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+This program is also distributed with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have included with MySQL.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
@@ -53,6 +61,18 @@ Created 1/20/1994 Heikki Tuuri
 /** Time stamp */
 typedef time_t	ib_time_t;
 
+/** Time stamp read from the monotonic clock (returned by ut_time_monotonic()).
+ */
+typedef int64_t ib_time_monotonic_t;
+
+/** Number of milliseconds read from the monotonic clock (returned by
+ * ut_time_monotonic_ms()). */
+typedef int64_t ib_time_monotonic_ms_t;
+
+/** Number of microseconds read from the monotonic clock (returned by
+ * ut_time_monotonic_us()). */
+typedef int64_t ib_time_monotonic_us_t;
+
 #ifndef UNIV_HOTBACKUP
 # if defined(HAVE_PAUSE_INSTRUCTION)
    /* According to the gcc info page, asm volatile means that the
@@ -91,11 +111,16 @@ if cond becomes true.
 @param max_wait_us in: maximum delay to wait, in microseconds */
 #define UT_WAIT_FOR(cond, max_wait_us)				\
 do {								\
-	uintmax_t	start_us;					\
-	start_us = ut_time_us(NULL);				\
-	while (!(cond)						\
-	       && ut_time_us(NULL) - start_us < (max_wait_us)) {\
-								\
+	uint64_t	start_us;				\
+	start_us = ut_time_monotonic_us();			\
+	while (!(cond)) {					\
+		ib_time_monotonic_us_t diff;                    \
+		diff = ut_time_monotonic_us() - start_us;       \
+		uint64_t limit = max_wait_us;			\
+		if(limit <= 0 || (diff > 0 &&			\
+			         ((uint64_t)diff > limit))) {	\
+			break;					\
+		}						\
 		os_thread_sleep(2000 /* 2 ms */);		\
 	}							\
 } while (0)
@@ -213,33 +238,18 @@ ut_time(void);
 /*=========*/
 #ifndef UNIV_HOTBACKUP
 /**********************************************************//**
-Returns system time.
-Upon successful completion, the value 0 is returned; otherwise the
-value -1 is returned and the global variable errno is set to indicate the
-error.
-@return 0 on success, -1 otherwise */
-int
-ut_usectime(
-/*========*/
-	ulint*	sec,	/*!< out: seconds since the Epoch */
-	ulint*	ms);	/*!< out: microseconds since the Epoch+*sec */
+Returns the number of microseconds since epoch. Uses the monotonic clock.
+ @return us since epoch or 0 if failed to retrieve */
+ib_time_monotonic_us_t ut_time_monotonic_us(void);
+
+/** Returns the number of milliseconds since epoch. Uses the monotonic clock.
+ @return ms since epoch */
+ib_time_monotonic_ms_t ut_time_monotonic_ms(void);
+
+/** Returns the number of seconds since epoch. Uses the monotonic clock.
+ @return us since epoch or 0 if failed to retrieve */
+ib_time_monotonic_t ut_time_monotonic(void);
 
-/**********************************************************//**
-Returns the number of microseconds since epoch. Similar to
-time(3), the return value is also stored in *tloc, provided
-that tloc is non-NULL.
-@return us since epoch */
-uintmax_t
-ut_time_us(
-/*=======*/
-	uintmax_t*	tloc);	/*!< out: us since epoch, if non-NULL */
-/**********************************************************//**
-Returns the number of milliseconds since some epoch.  The
-value may wrap around.  It should only be used for heuristic
-purposes.
-@return ms since epoch */
-ulint
-ut_time_ms(void);
 /*============*/
 #ifdef _WIN32
 /**********************************************************//**
@@ -252,29 +262,6 @@ ut_win_init_time();
 
 #endif /* !UNIV_HOTBACKUP */
 
-/**********************************************************//**
-Returns the number of milliseconds since some epoch.  The
-value may wrap around.  It should only be used for heuristic
-purposes.
-@return ms since epoch */
-ulint
-ut_time_ms(void);
-/*============*/
-
-/** Returns the number of milliseconds since some epoch using monotonic clock.
-The value may wrap around.  This should not be used as an accurate Time Of Day.
-It is only intended to be used as a means of calculating transient elapsed or
-projected time that will not be influenced by changes to the systems real time
-clock.  Returns a small structure that contains the result so as to poison the
-code and reveal any changes that might later be introduced by upstream.
-*/
-struct ut_monotonic_time {
-	ulint ms;
-};
-
-ut_monotonic_time
-ut_monotonic_time_ms(void);
-
 /**********************************************************//**
 Returns the difference of two times in seconds.
 @return time2 - time1 expressed in seconds */
