@@ -1,14 +1,22 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2016, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1996, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; version 2 of the License.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+This program is also distributed with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have included with MySQL.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
@@ -34,6 +42,17 @@ Created 25/5/2010 Sunny Bains
 #include "row0mysql.h"
 #include "srv0start.h"
 #include "lock0priv.h"
+#include "lock0iter.h"
+
+#include <sstream>
+
+struct blocking_trx_info {
+	uint64_t trx_id;
+	uint32_t thread_id;
+	int64_t query_id;
+};
+
+static const size_t MAX_BLOCKING_TRX_IN_REPORT = 10;
 
 /*********************************************************************//**
 Print the contents of the lock_sys_t::waiting_threads array. */
@@ -56,7 +75,7 @@ lock_wait_table_print(void)
 			(ulong) slot->in_use,
 			(ulong) slot->suspended,
 			slot->wait_timeout,
-			(ulong) difftime(ut_time(), slot->suspend_time));
+			(ulong) (ut_time_monotonic() - slot->suspend_time));
 	}
 }
 
@@ -153,7 +172,7 @@ lock_wait_table_reserve_slot(
 
 			os_event_reset(slot->event);
 			slot->suspended = TRUE;
-			slot->suspend_time = ut_time();
+			slot->suspend_time = ut_time_monotonic();
 			slot->wait_timeout = wait_timeout;
 
 			if (slot == lock_sys->last_slot) {
@@ -199,6 +218,47 @@ wsrep_is_BF_lock_timeout(
  }
 #endif /* WITH_WSREP */
 
+/** Print lock wait timeout info to stderr. It's supposed this function
+is executed in trx's THD thread as it calls some non-thread-safe
+functions to get some info from THD.
+@param[in]	trx	requested trx
+@param[in]	blocking	blocking info array
+@param[in]	blocking_count	blocking info array size */
+static
+void
+print_lock_wait_timeout(
+	const trx_t &trx,
+	blocking_trx_info *blocking,
+	size_t blocking_count)
+{
+	std::ostringstream outs;
+	size_t unused;
+
+	outs << "Lock wait timeout info:\n";
+	outs << "Requested thread id: " <<
+		thd_get_thread_id(trx.mysql_thd) <<
+		"\n";
+	outs << "Requested trx id: " << trx_get_id_for_print(&trx) << "\n";
+	outs << "Requested query: " <<
+		innobase_get_stmt_unsafe(trx.mysql_thd, &unused) << "\n";
+
+	outs << "Total blocking transactions count: " <<
+		blocking_count <<
+		"\n";
+
+	for (size_t i = 0; i < blocking_count; ++i) {
+		outs << "Blocking transaction number: " << (i + 1) << "\n";
+		outs << "Blocking thread id: " <<
+			blocking[i].thread_id <<
+			"\n";
+		outs << "Blocking query id: " <<
+			blocking[i].query_id <<
+			"\n";
+		outs << "Blocking trx id: " << blocking[i].trx_id << "\n";
+	}
+	ib::info() << outs.str();
+}
+
 /***************************************************************//**
 Puts a user OS thread to wait for a lock to be released. If an error
 occurs during the wait trx->error_state associated with thr is
@@ -212,14 +272,14 @@ lock_wait_suspend_thread(
 				user OS thread */
 {
 	srv_slot_t*	slot;
-	double		wait_time;
+	int64_t		wait_time;
 	trx_t*		trx;
 	ibool		was_declared_inside_innodb;
-	int64_t		start_time = 0;
-	int64_t		finish_time;
-	ulint		sec;
-	ulint		ms;
+	ib_time_monotonic_ms_t 		start_time = 0;
+	ib_time_monotonic_ms_t 		finish_time;
 	ulong		lock_wait_timeout;
+	blocking_trx_info blocking[MAX_BLOCKING_TRX_IN_REPORT];
+	size_t blocking_count = 0;
 
 	trx = thr_get_trx(thr);
 
@@ -265,11 +325,7 @@ lock_wait_suspend_thread(
 		srv_stats.n_lock_wait_count.inc();
 		srv_stats.n_lock_wait_current_count.inc();
 
-		if (ut_usectime(&sec, &ms) == -1) {
-			start_time = -1;
-		} else {
-			start_time = static_cast<int64_t>(sec) * 1000000 + ms;
-		}
+		start_time = ut_time_monotonic_us();
 	}
 
 	lock_wait_mutex_exit();
@@ -281,6 +337,27 @@ lock_wait_suspend_thread(
 
 	if (const lock_t* wait_lock = trx->lock.wait_lock) {
 		lock_type = lock_get_type_low(wait_lock);
+		if (srv_print_lock_wait_timeout_info) {
+			lock_queue_iterator_t	iter;
+			const lock_t*		curr_lock;
+			lock_queue_iterator_reset(&iter, wait_lock, ULINT_UNDEFINED);
+			for (curr_lock = lock_queue_iterator_get_prev(&iter);
+				curr_lock != NULL;
+				curr_lock = lock_queue_iterator_get_prev(&iter)) {
+				if (lock_has_to_wait(trx->lock.wait_lock, curr_lock)) {
+					blocking[blocking_count].trx_id = lock_get_trx_id(curr_lock);
+					blocking[blocking_count].thread_id =
+						curr_lock->trx->mysql_thd ?
+						thd_get_thread_id(curr_lock->trx->mysql_thd) : 0;
+					blocking[blocking_count].query_id =
+					curr_lock->trx->mysql_thd ?
+						thd_get_query_id(curr_lock->trx->mysql_thd) : 0;
+					/* Only limited number of blocking transaction infos is implemented*/
+					if ((++blocking_count) >= MAX_BLOCKING_TRX_IN_REPORT)
+						break;
+				}
+			}
+		}
 	}
 
 	lock_mutex_exit();
@@ -347,7 +424,7 @@ lock_wait_suspend_thread(
 		row_mysql_freeze_data_dictionary(trx);
 	}
 
-	wait_time = ut_difftime(ut_time(), slot->suspend_time);
+	wait_time = ut_time_monotonic() - slot->suspend_time;
 
 	/* Release the slot for others to use */
 
@@ -355,15 +432,10 @@ lock_wait_suspend_thread(
 
 	if (thr->lock_state == QUE_THR_LOCK_ROW) {
 		ulint	diff_time;
-
-		if (ut_usectime(&sec, &ms) == -1) {
-			finish_time = -1;
-		} else {
-			finish_time = static_cast<int64_t>(sec) * 1000000 + ms;
-		}
+		finish_time = ut_time_monotonic_us();
 
 		diff_time = (finish_time > start_time) ?
-			    (ulint) (finish_time - start_time) : 0;
+			    (uint64_t) (finish_time - start_time) : 0;
 
 		srv_stats.n_lock_wait_current_count.dec();
 		srv_stats.n_lock_wait_time.add(diff_time);
@@ -371,8 +443,7 @@ lock_wait_suspend_thread(
 		/* Only update the variable if we successfully
 		retrieved the start and finish times. See Bug#36819. */
 		if (diff_time > lock_sys->n_lock_max_wait_time
-		    && start_time != -1
-		    && finish_time != -1) {
+		    && start_time != -1) {
 
 			lock_sys->n_lock_max_wait_time = diff_time;
 		}
@@ -402,6 +473,8 @@ lock_wait_suspend_thread(
 #endif /* WITH_WSREP */
 
 		trx->error_state = DB_LOCK_WAIT_TIMEOUT;
+		if (srv_print_lock_wait_timeout_info)
+			print_lock_wait_timeout(*trx, blocking, blocking_count);
 
 #ifdef WITH_WSREP
                 }
@@ -456,8 +529,8 @@ lock_wait_check_and_cancel(
 					thread when the wait started */
 {
 	trx_t*		trx;
-	double		wait_time;
-	ib_time_t	suspend_time = slot->suspend_time;
+	int64_t			wait_time;
+	ib_time_monotonic_t	suspend_time = slot->suspend_time;
 
 	ut_ad(lock_wait_mutex_own());
 
@@ -465,13 +538,13 @@ lock_wait_check_and_cancel(
 
 	ut_ad(slot->suspended);
 
-	wait_time = ut_difftime(ut_time(), suspend_time);
+	wait_time = ut_time_monotonic() - suspend_time;
 
 	trx = thr_get_trx(slot->thr);
 
 	if (trx_is_interrupted(trx)
 	    || (slot->wait_timeout < 100000000
-		&& (wait_time > (double) slot->wait_timeout
+		&& (wait_time > (int64_t) slot->wait_timeout
 		   || wait_time < 0))) {
 
 		/* Timeout exceeded or a wrap-around in system
