@@ -1,14 +1,22 @@
 /*****************************************************************************
 
-Copyright (c) 1997, 2017, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1997, 2019, Oracle and/or its affiliates. All Rights Reserved.
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; version 2 of the License.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
 
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+This program is also distributed with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have included with MySQL.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License, version 2.0, for more details.
 
 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc.,
@@ -23,6 +31,8 @@ Purge obsolete records
 Created 3/14/1997 Heikki Tuuri
 *******************************************************/
 
+#include <debug_sync.h>
+
 #include "row0purge.h"
 
 #ifdef UNIV_NONINL
@@ -170,6 +180,16 @@ row_purge_remove_clust_if_poss_low(
 	} else {
 		dberr_t	err;
 		ut_ad(mode == (BTR_MODIFY_TREE | BTR_LATCH_FOR_DELETE));
+
+		DBUG_EXECUTE_IF("pessimistic_row_purge_clust", {
+			const char act[] =
+				"now SIGNAL pessimistic_row_purge_clust_pause "
+				"WAIT_FOR pessimistic_row_purge_clust_continue";
+			DBUG_ASSERT(opt_debug_sync_timeout > 0);
+			DBUG_ASSERT(!debug_sync_set_action(
+					current_thd, STRING_WITH_LEN(act)));
+		});
+
 		btr_cur_pessimistic_delete(
 			&err, FALSE, btr_pcur_get_btr_cur(&node->pcur), 0,
 			false, &mtr);
