@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -24,7 +24,6 @@
 
 #include <ndb_global.h>
 
-#include <NdbHost.h>
 #include <NdbSleep.h>
 #include <NdbThread.h>
 #include <NdbMain.h>
@@ -46,6 +45,8 @@ static int   minEventSendPoll;
 static int   forceSendPoll;
 static bool  useNdbRecord;
 static bool  useCombUpd;
+int          subscriberCount;
+static bool  robustMode;
 
 static ThreadData *data;
 static Ndb_cluster_connection *g_cluster_connection= 0;
@@ -67,7 +68,7 @@ static void usage(const char *prog)
 
    ndbout_c(
            "Usage: %s [-proc <num>] [-warm <num>] [-time <num>] [ -p <num>]" 
-	   "[-t <num> ] [ -e <num> ] [ -f <num>] [ -ndbrecord ]\n"
+	   "[-t <num> ] [ -e <num> ] [ -f <num>] [ -ndbrecord ] [ -s <num>]\n"
            "  -proc <num>    Specifies that <num> is the number of\n"
            "                 threads. The default is 1.\n"
            "  -time <num>    Specifies that the test will run for <num> sec.\n"
@@ -85,8 +86,10 @@ static void usage(const char *prog)
            "  -ndbrecord     Use NdbRecord Api.\n"
            "                 Default is to use old Api\n"
            "  -combupdread   Use update pre-read operation where possible\n"
-           "                 Default is to use separate read+update ops\n",
-           progname);
+           "                 Default is to use separate read+update ops\n"
+           "  -s <num>       Number of subscribers to operate on, default is %u.\n"
+           "  -r             Whether to be robust to key errors\n",
+           progname, NO_OF_SUBSCRIBERS);
 }
 
 static
@@ -104,6 +107,8 @@ parse_args(int argc, const char **argv)
    forceSendPoll    = 0;
    useNdbRecord     = false;
    useCombUpd       = false;
+   subscriberCount  = NO_OF_SUBSCRIBERS;
+   robustMode       = false;
 
    i = 1;
    while (i < argc){
@@ -183,6 +188,20 @@ parse_args(int argc, const char **argv)
        useCombUpd= true;
        i++;
      }
+     else if (strcmp("-s", argv[i]) == 0) {
+       if (i + 1 >= argc) {
+         return 1;
+       }
+       if (sscanf(argv[i+1], "%u", &subscriberCount) == -1) {
+         ndbout_c("-s flag requires a positive argument.");
+         return 1;
+       }
+       i+=2;
+     }
+     else if (strcmp("-r", argv[i]) == 0) {
+       robustMode= true;
+       i++;
+     }
      else {
        return 1;
      }
@@ -251,16 +270,11 @@ print_stats(const char       *title,
 {
   int    i;
   char buf[10];
-  char name[MAXHOSTNAMELEN];
-  
-  name[0] = 0;
-  NdbHost_GetHostName(name);
   
   ndbout_c("\n------ %s ------",title);
   ndbout_c("Length        : %d %s",
 	 length,
 	 transactionFlag ? "Transactions" : "sec");
-  ndbout_c("Processor     : %s", name);
   ndbout_c("Number of Proc: %d",numProc);
   ndbout_c("Parallellism  : %d", parallellism);
   ndbout_c("UseNdbRecord  : %u", useNdbRecord);
@@ -555,6 +569,7 @@ NDB_COMMAND(DbAsyncGenerator, "DbAsyncGenerator",
       data[tid].runState              = Runnable;
       data[tid].ndbRecordSharedData   = ndbRecordSharedDataPtr;
       data[tid].useCombinedUpdate     = useCombUpd;
+      data[tid].robustMode            = robustMode;
     }
     sprintf(threadName, "AsyncThread[%d]", i);
     pThread = NdbThread_Create(threadRoutine, 
@@ -688,27 +703,22 @@ asyncDbDisconnect(Ndb* pNDB)
   delete pNDB;
 }
 
+static NDB_TICKS initTicks;
+
 double
 userGetTime(void)
 {
-  static bool initialized = false;
-  static NDB_TICKS initSecs = 0;
-  static Uint32 initMicros = 0;
   double timeValue = 0;
 
-  if ( !initialized ) {
-    initialized = true;
-    NdbTick_CurrentMicrosecond(&initSecs, &initMicros); 
+  if ( !NdbTick_IsValid(initTicks)) {
+    initTicks = NdbTick_getCurrentTicks();
     timeValue = 0.0;
   } else {
-    NDB_TICKS secs = 0;
-    Uint32 micros = 0;
+    const NDB_TICKS now = NdbTick_getCurrentTicks();
+    const Uint64 elapsedMicro =
+      NdbTick_Elapsed(initTicks,now).microSec();
 
-    NdbTick_CurrentMicrosecond(&secs, &micros);
-    double s  = (double)secs  - (double)initSecs;
-    double us = (double)micros - (double)initMicros;
-    
-    timeValue = s + (us / 1000000.0);
+    timeValue = ((double)elapsedMicro) / 1000000.0;
   }
   return timeValue;
 }
