@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -28,6 +28,8 @@
 #include "pfs_user.h"
 #include "pfs_host.h"
 #include "pfs_account.h"
+#include "mysqld_thd_manager.h"
+#include "pfs_buffer_container.h"
 
 /**
   @file storage/perfschema/pfs_visitor.cc
@@ -39,182 +41,330 @@
   @{
 */
 
+class All_THD_visitor_adapter : public Do_THD_Impl
+{
+public:
+  All_THD_visitor_adapter(PFS_connection_visitor *visitor)
+    : m_visitor(visitor)
+  {}
+
+  virtual void operator()(THD *thd)
+  {
+    m_visitor->visit_THD(thd);
+  }
+
+private:
+  PFS_connection_visitor *m_visitor;
+};
+
 /** Connection iterator */
 void PFS_connection_iterator::visit_global(bool with_hosts, bool with_users,
                                            bool with_accounts, bool with_threads,
+                                           bool with_THDs,
                                            PFS_connection_visitor *visitor)
 {
   DBUG_ASSERT(visitor != NULL);
+  DBUG_ASSERT(! with_threads || ! with_THDs);
 
   visitor->visit_global();
 
   if (with_hosts)
   {
-    PFS_host *pfs= host_array;
-    PFS_host *pfs_last= pfs + host_max;
-    for ( ; pfs < pfs_last; pfs++)
+    PFS_host_iterator it= global_host_container.iterate();
+    PFS_host *pfs= it.scan_next();
+
+    while (pfs != NULL)
     {
-      if (pfs->m_lock.is_populated())
-        visitor->visit_host(pfs);
+      visitor->visit_host(pfs);
+      pfs= it.scan_next();
     }
   }
 
   if (with_users)
   {
-    PFS_user *pfs= user_array;
-    PFS_user *pfs_last= pfs + user_max;
-    for ( ; pfs < pfs_last; pfs++)
+    PFS_user_iterator it= global_user_container.iterate();
+    PFS_user *pfs= it.scan_next();
+
+    while (pfs != NULL)
     {
-      if (pfs->m_lock.is_populated())
-        visitor->visit_user(pfs);
+      visitor->visit_user(pfs);
+      pfs= it.scan_next();
     }
   }
 
   if (with_accounts)
   {
-    PFS_account *pfs= account_array;
-    PFS_account *pfs_last= pfs + account_max;
-    for ( ; pfs < pfs_last; pfs++)
+    PFS_account_iterator it= global_account_container.iterate();
+    PFS_account *pfs= it.scan_next();
+
+    while (pfs != NULL)
     {
-      if (pfs->m_lock.is_populated())
-        visitor->visit_account(pfs);
+      visitor->visit_account(pfs);
+      pfs= it.scan_next();
     }
   }
 
+
   if (with_threads)
   {
-    PFS_thread *pfs= thread_array;
-    PFS_thread *pfs_last= pfs + thread_max;
-    for ( ; pfs < pfs_last; pfs++)
+    PFS_thread_iterator it= global_thread_container.iterate();
+    PFS_thread *pfs= it.scan_next();
+
+    while (pfs != NULL)
     {
-      if (pfs->m_lock.is_populated())
-        visitor->visit_thread(pfs);
+      visitor->visit_thread(pfs);
+      pfs= it.scan_next();
     }
   }
+
+  if (with_THDs)
+  {
+    All_THD_visitor_adapter adapter(visitor);
+    Global_THD_manager::get_instance()->do_for_all_thd(& adapter);
+  }
 }
 
+class All_host_THD_visitor_adapter : public Do_THD_Impl
+{
+public:
+  All_host_THD_visitor_adapter(PFS_connection_visitor *visitor, PFS_host *host)
+    : m_visitor(visitor), m_host(host)
+  {}
+
+  virtual void operator()(THD *thd)
+  {
+    PSI_thread *psi= thd->get_psi();
+    PFS_thread *pfs= reinterpret_cast<PFS_thread*>(psi);
+    pfs= sanitize_thread(pfs);
+    if (pfs != NULL)
+    {
+      PFS_account *account= sanitize_account(pfs->m_account);
+      if (account != NULL)
+      {
+        if (account->m_host == m_host)
+        {
+          m_visitor->visit_THD(thd);
+        }
+      }
+      else if (pfs->m_host == m_host)
+      {
+        m_visitor->visit_THD(thd);
+      }
+    }
+  }
+
+private:
+  PFS_connection_visitor *m_visitor;
+  PFS_host *m_host;
+};
+
 void PFS_connection_iterator::visit_host(PFS_host *host,
                                          bool with_accounts, bool with_threads,
+                                         bool with_THDs,
                                          PFS_connection_visitor *visitor)
 {
   DBUG_ASSERT(visitor != NULL);
+  DBUG_ASSERT(! with_threads || ! with_THDs);
 
   visitor->visit_host(host);
 
   if (with_accounts)
   {
-    PFS_account *pfs= account_array;
-    PFS_account *pfs_last= pfs + account_max;
-    for ( ; pfs < pfs_last; pfs++)
+    PFS_account_iterator it= global_account_container.iterate();
+    PFS_account *pfs= it.scan_next();
+
+    while (pfs != NULL)
     {
-      if ((pfs->m_host == host) && pfs->m_lock.is_populated())
+      if (pfs->m_host == host)
       {
         visitor->visit_account(pfs);
       }
+      pfs= it.scan_next();
     }
   }
 
   if (with_threads)
   {
-    PFS_thread *pfs= thread_array;
-    PFS_thread *pfs_last= pfs + thread_max;
-    for ( ; pfs < pfs_last; pfs++)
+    PFS_thread_iterator it= global_thread_container.iterate();
+    PFS_thread *pfs= it.scan_next();
+
+    while (pfs != NULL)
     {
-      if (pfs->m_lock.is_populated())
+      PFS_account *safe_account= sanitize_account(pfs->m_account);
+      if (((safe_account != NULL) && (safe_account->m_host == host)) /* 1 */
+          || (pfs->m_host == host))                                  /* 2 */
       {
-        PFS_account *safe_account= sanitize_account(pfs->m_account);
-        if ((safe_account != NULL) && (safe_account->m_host == host))
-        {
-          /*
-            If the thread belongs to a known user@host that belongs to this host,
-            process it.
-          */
-          visitor->visit_thread(pfs);
-        }
-        else if (pfs->m_host == host)
-        {
-          /*
-            If the thread belongs to a 'lost' user@host that belong to this host,
-            process it.
-          */
-          visitor->visit_thread(pfs);
-        }
+        /*
+          If the thread belongs to:
+          - (1) a known user@host that belongs to this host,
+          - (2) a 'lost' user@host that belongs to this host
+          process it.
+        */
+        visitor->visit_thread(pfs);
       }
+      pfs= it.scan_next();
     }
   }
+
+  if (with_THDs)
+  {
+    All_host_THD_visitor_adapter adapter(visitor, host);
+    Global_THD_manager::get_instance()->do_for_all_thd(& adapter);
+  }
 }
 
+class All_user_THD_visitor_adapter : public Do_THD_Impl
+{
+public:
+  All_user_THD_visitor_adapter(PFS_connection_visitor *visitor, PFS_user *user)
+    : m_visitor(visitor), m_user(user)
+  {}
+
+  virtual void operator()(THD *thd)
+  {
+    PSI_thread *psi= thd->get_psi();
+    PFS_thread *pfs= reinterpret_cast<PFS_thread*>(psi);
+    pfs= sanitize_thread(pfs);
+    if (pfs != NULL)
+    {
+      PFS_account *account= sanitize_account(pfs->m_account);
+      if (account != NULL)
+      {
+        if (account->m_user == m_user)
+        {
+          m_visitor->visit_THD(thd);
+        }
+      }
+      else if (pfs->m_user == m_user)
+      {
+        m_visitor->visit_THD(thd);
+      }
+    }
+  }
+
+private:
+  PFS_connection_visitor *m_visitor;
+  PFS_user *m_user;
+};
+
 void PFS_connection_iterator::visit_user(PFS_user *user,
                                          bool with_accounts, bool with_threads,
+                                         bool with_THDs,
                                          PFS_connection_visitor *visitor)
 {
   DBUG_ASSERT(visitor != NULL);
+  DBUG_ASSERT(! with_threads || ! with_THDs);
 
   visitor->visit_user(user);
 
   if (with_accounts)
   {
-    PFS_account *pfs= account_array;
-    PFS_account *pfs_last= pfs + account_max;
-    for ( ; pfs < pfs_last; pfs++)
+    PFS_account_iterator it= global_account_container.iterate();
+    PFS_account *pfs= it.scan_next();
+
+    while (pfs != NULL)
     {
-      if ((pfs->m_user == user) && pfs->m_lock.is_populated())
+      if (pfs->m_user == user)
       {
         visitor->visit_account(pfs);
       }
+      pfs= it.scan_next();
     }
   }
 
   if (with_threads)
   {
-    PFS_thread *pfs= thread_array;
-    PFS_thread *pfs_last= pfs + thread_max;
-    for ( ; pfs < pfs_last; pfs++)
+    PFS_thread_iterator it= global_thread_container.iterate();
+    PFS_thread *pfs= it.scan_next();
+
+    while (pfs != NULL)
     {
-      if (pfs->m_lock.is_populated())
+      PFS_account *safe_account= sanitize_account(pfs->m_account);
+      if (((safe_account != NULL) && (safe_account->m_user == user)) /* 1 */
+          || (pfs->m_user == user))                                  /* 2 */
       {
-        PFS_account *safe_account= sanitize_account(pfs->m_account);
-        if ((safe_account != NULL) && (safe_account->m_user == user))
-        {
-          /*
-            If the thread belongs to a known user@host that belongs to this user,
-            process it.
-          */
-          visitor->visit_thread(pfs);
-        }
-        else if (pfs->m_user == user)
-        {
-          /*
-            If the thread belongs to a 'lost' user@host that belong to this user,
-            process it.
-          */
-          visitor->visit_thread(pfs);
-        }
+        /*
+          If the thread belongs to:
+          - (1) a known user@host that belongs to this user,
+          - (2) a 'lost' user@host that belongs to this user
+          process it.
+        */
+        visitor->visit_thread(pfs);
       }
+      pfs= it.scan_next();
     }
   }
+
+  if (with_THDs)
+  {
+    All_user_THD_visitor_adapter adapter(visitor, user);
+    Global_THD_manager::get_instance()->do_for_all_thd(& adapter);
+  }
 }
 
+class All_account_THD_visitor_adapter : public Do_THD_Impl
+{
+public:
+  All_account_THD_visitor_adapter(PFS_connection_visitor *visitor, PFS_account *account)
+    : m_visitor(visitor), m_account(account)
+  {}
+
+  virtual void operator()(THD *thd)
+  {
+    PSI_thread *psi= thd->get_psi();
+    PFS_thread *pfs= reinterpret_cast<PFS_thread*>(psi);
+    pfs= sanitize_thread(pfs);
+    if (pfs != NULL)
+    {
+      if (pfs->m_account == m_account)
+      {
+        m_visitor->visit_THD(thd);
+      }
+    }
+  }
+
+private:
+  PFS_connection_visitor *m_visitor;
+  PFS_account *m_account;
+};
+
 void PFS_connection_iterator::visit_account(PFS_account *account,
-                                              bool with_threads,
-                                              PFS_connection_visitor *visitor)
+                                            bool with_threads,
+                                            bool with_THDs,
+                                            PFS_connection_visitor *visitor)
 {
   DBUG_ASSERT(visitor != NULL);
+  DBUG_ASSERT(! with_threads || ! with_THDs);
 
   visitor->visit_account(account);
 
   if (with_threads)
   {
-    PFS_thread *pfs= thread_array;
-    PFS_thread *pfs_last= pfs + thread_max;
-    for ( ; pfs < pfs_last; pfs++)
+    PFS_thread_iterator it= global_thread_container.iterate();
+    PFS_thread *pfs= it.scan_next();
+
+    while (pfs != NULL)
     {
-      if ((pfs->m_account == account) && pfs->m_lock.is_populated())
+      if (pfs->m_account == account)
       {
         visitor->visit_thread(pfs);
       }
+      pfs= it.scan_next();
     }
   }
+
+  if (with_THDs)
+  {
+    All_account_THD_visitor_adapter adapter(visitor, account);
+    Global_THD_manager::get_instance()->do_for_all_thd(& adapter);
+  }
+}
+
+void PFS_connection_iterator::visit_THD(THD *thd,
+                                        PFS_connection_visitor *visitor)
+{
+  DBUG_ASSERT(visitor != NULL);
+  visitor->visit_THD(thd);
 }
 
 void PFS_instance_iterator::visit_all(PFS_instance_visitor *visitor)
@@ -246,14 +396,13 @@ void PFS_instance_iterator::visit_all_mutex_classes(PFS_instance_visitor *visito
 
 void PFS_instance_iterator::visit_all_mutex_instances(PFS_instance_visitor *visitor)
 {
-  PFS_mutex *pfs= mutex_array;
-  PFS_mutex *pfs_last= pfs + mutex_max;
-  for ( ; pfs < pfs_last; pfs++)
+  PFS_mutex_iterator it= global_mutex_container.iterate();
+  PFS_mutex *pfs= it.scan_next();
+
+  while (pfs != NULL)
   {
-    if (pfs->m_lock.is_populated())
-    {
-      visitor->visit_mutex(pfs);
-    }
+    visitor->visit_mutex(pfs);
+    pfs= it.scan_next();
   }
 }
 
@@ -278,14 +427,13 @@ void PFS_instance_iterator::visit_all_rwlock_classes(PFS_instance_visitor *visit
 
 void PFS_instance_iterator::visit_all_rwlock_instances(PFS_instance_visitor *visitor)
 {
-  PFS_rwlock *pfs= rwlock_array;
-  PFS_rwlock *pfs_last= pfs + rwlock_max;
-  for ( ; pfs < pfs_last; pfs++)
+  PFS_rwlock_iterator it= global_rwlock_container.iterate();
+  PFS_rwlock *pfs= it.scan_next();
+
+  while (pfs != NULL)
   {
-    if (pfs->m_lock.is_populated())
-    {
-      visitor->visit_rwlock(pfs);
-    }
+    visitor->visit_rwlock(pfs);
+    pfs= it.scan_next();
   }
 }
 
@@ -310,14 +458,13 @@ void PFS_instance_iterator::visit_all_cond_classes(PFS_instance_visitor *visitor
 
 void PFS_instance_iterator::visit_all_cond_instances(PFS_instance_visitor *visitor)
 {
-  PFS_cond *pfs= cond_array;
-  PFS_cond *pfs_last= pfs + cond_max;
-  for ( ; pfs < pfs_last; pfs++)
+  PFS_cond_iterator it= global_cond_container.iterate();
+  PFS_cond *pfs= it.scan_next();
+
+  while (pfs != NULL)
   {
-    if (pfs->m_lock.is_populated())
-    {
-      visitor->visit_cond(pfs);
-    }
+    visitor->visit_cond(pfs);
+    pfs= it.scan_next();
   }
 }
 
@@ -342,14 +489,13 @@ void PFS_instance_iterator::visit_all_file_classes(PFS_instance_visitor *visitor
 
 void PFS_instance_iterator::visit_all_file_instances(PFS_instance_visitor *visitor)
 {
-  PFS_file *pfs= file_array;
-  PFS_file *pfs_last= pfs + file_max;
-  for ( ; pfs < pfs_last; pfs++)
+  PFS_file_iterator it= global_file_container.iterate();
+  PFS_file *pfs= it.scan_next();
+
+  while (pfs != NULL)
   {
-    if (pfs->m_lock.is_populated())
-    {
-      visitor->visit_file(pfs);
-    }
+    visitor->visit_file(pfs);
+    pfs= it.scan_next();
   }
 }
 
@@ -375,14 +521,16 @@ void PFS_instance_iterator::visit_mutex_instances(PFS_mutex_class *klass,
   }
   else
   {
-    PFS_mutex *pfs= mutex_array;
-    PFS_mutex *pfs_last= pfs + mutex_max;
-    for ( ; pfs < pfs_last; pfs++)
+    PFS_mutex_iterator it= global_mutex_container.iterate();
+    PFS_mutex *pfs= it.scan_next();
+
+    while (pfs != NULL)
     {
-      if ((pfs->m_class == klass) && pfs->m_lock.is_populated())
+      if (pfs->m_class == klass)
       {
         visitor->visit_mutex(pfs);
       }
+      pfs= it.scan_next();
     }
   }
 }
@@ -407,14 +555,16 @@ void PFS_instance_iterator::visit_rwlock_instances(PFS_rwlock_class *klass,
   }
   else
   {
-    PFS_rwlock *pfs= rwlock_array;
-    PFS_rwlock *pfs_last= pfs + rwlock_max;
-    for ( ; pfs < pfs_last; pfs++)
+    PFS_rwlock_iterator it= global_rwlock_container.iterate();
+    PFS_rwlock *pfs= it.scan_next();
+
+    while (pfs != NULL)
     {
-      if ((pfs->m_class == klass) && pfs->m_lock.is_populated())
+      if (pfs->m_class == klass)
       {
         visitor->visit_rwlock(pfs);
       }
+      pfs= it.scan_next();
     }
   }
 }
@@ -439,14 +589,16 @@ void PFS_instance_iterator::visit_cond_instances(PFS_cond_class *klass,
   }
   else
   {
-    PFS_cond *pfs= cond_array;
-    PFS_cond *pfs_last= pfs + cond_max;
-    for ( ; pfs < pfs_last; pfs++)
+    PFS_cond_iterator it= global_cond_container.iterate();
+    PFS_cond *pfs= it.scan_next();
+
+    while (pfs != NULL)
     {
-      if ((pfs->m_class == klass) && pfs->m_lock.is_populated())
+      if (pfs->m_class == klass)
       {
         visitor->visit_cond(pfs);
       }
+      pfs= it.scan_next();
     }
   }
 }
@@ -471,14 +623,16 @@ void PFS_instance_iterator::visit_file_instances(PFS_file_class *klass,
   }
   else
   {
-    PFS_file *pfs= file_array;
-    PFS_file *pfs_last= pfs + file_max;
-    for ( ; pfs < pfs_last; pfs++)
+    PFS_file_iterator it= global_file_container.iterate();
+    PFS_file *pfs= it.scan_next();
+
+    while (pfs != NULL)
     {
-      if ((pfs->m_class == klass) && pfs->m_lock.is_populated())
+      if (pfs->m_class == klass)
       {
         visitor->visit_file(pfs);
       }
+      pfs= it.scan_next();
     }
   }
 }
@@ -505,14 +659,16 @@ void PFS_instance_iterator::visit_socket_instances(PFS_socket_class *klass,
   }
   else
   {
-    PFS_socket *pfs= socket_array;
-    PFS_socket *pfs_last= pfs + socket_max;
-    for ( ; pfs < pfs_last; pfs++)
+    PFS_socket_iterator it= global_socket_container.iterate();
+    PFS_socket *pfs= it.scan_next();
+
+    while (pfs != NULL)
     {
-      if ((pfs->m_class == klass) && pfs->m_lock.is_populated())
+      if (pfs->m_class == klass)
       {
         visitor->visit_socket(pfs);
       }
+      pfs= it.scan_next();
     }
   }
 }
@@ -542,16 +698,17 @@ void PFS_instance_iterator::visit_socket_instances(PFS_socket_class *klass,
   else
   {
     /* Get current socket stats from each socket instance owned by this thread */
-    PFS_socket *pfs= socket_array;
-    PFS_socket *pfs_last= pfs + socket_max;
+    PFS_socket_iterator it= global_socket_container.iterate();
+    PFS_socket *pfs= it.scan_next();
 
-    for ( ; pfs < pfs_last; pfs++)
+    while (pfs != NULL)
     {
       if (unlikely((pfs->m_class == klass) &&
                    (pfs->m_thread_owner == thread)))
       {
         visitor->visit_socket(pfs);
       }
+      pfs= it.scan_next();
     }
   }
 }
@@ -586,6 +743,50 @@ void PFS_object_iterator::visit_all(PFS_object_visitor *visitor)
   visit_all_tables(visitor);
 }
 
+class Proc_all_table_shares
+  : public PFS_buffer_processor<PFS_table_share>
+{
+public:
+  Proc_all_table_shares(PFS_object_visitor *visitor)
+    : m_visitor(visitor)
+  {}
+
+  virtual void operator()(PFS_table_share *pfs)
+  {
+    if (pfs->m_enabled)
+    {
+      m_visitor->visit_table_share(pfs);
+    }
+  }
+
+private:
+  PFS_object_visitor* m_visitor;
+};
+
+class Proc_all_table_handles
+  : public PFS_buffer_processor<PFS_table>
+{
+public:
+  Proc_all_table_handles(PFS_object_visitor *visitor)
+    : m_visitor(visitor)
+  {}
+
+  virtual void operator()(PFS_table *pfs)
+  {
+    PFS_table_share *safe_share= sanitize_table_share(pfs->m_share);
+    if (safe_share != NULL)
+    {
+      if (safe_share->m_enabled)
+      {
+        m_visitor->visit_table(pfs);
+      }
+    }
+  }
+
+private:
+  PFS_object_visitor* m_visitor;
+};
+
 void PFS_object_iterator::visit_all_tables(PFS_object_visitor *visitor)
 {
   DBUG_ASSERT(visitor != NULL);
@@ -593,46 +794,76 @@ void PFS_object_iterator::visit_all_tables(PFS_object_visitor *visitor)
   visitor->visit_global();
 
   /* For all the table shares ... */
-  PFS_table_share *share= table_share_array;
-  PFS_table_share *share_last= table_share_array + table_share_max;
-  for ( ; share < share_last; share++)
+  Proc_all_table_shares proc_shares(visitor);
+  global_table_share_container.apply(proc_shares);
+
+  /* For all the table handles ... */
+  Proc_all_table_handles proc_handles(visitor);
+  global_table_container.apply(proc_handles);
+}
+
+class Proc_one_table_share_handles
+  : public PFS_buffer_processor<PFS_table>
+{
+public:
+  Proc_one_table_share_handles(PFS_object_visitor *visitor, PFS_table_share *share)
+    : m_visitor(visitor), m_share(share)
+  {}
+
+  virtual void operator()(PFS_table *pfs)
   {
-    if (share->m_lock.is_populated())
+    if (pfs->m_share == m_share)
     {
-      visitor->visit_table_share(share);
+      m_visitor->visit_table(pfs);
     }
   }
 
-  /* For all the table handles ... */
-  PFS_table *table= table_array;
-  PFS_table *table_last= table_array + table_max;
-  for ( ; table < table_last; table++)
-  {
-    if (table->m_lock.is_populated())
-    {
-      visitor->visit_table(table);
-    }
-  }
-}
+private:
+  PFS_object_visitor* m_visitor;
+  PFS_table_share* m_share;
+};
 
 void PFS_object_iterator::visit_tables(PFS_table_share *share,
                                        PFS_object_visitor *visitor)
 {
   DBUG_ASSERT(visitor != NULL);
 
+  if (!share->m_enabled)
+    return;
+
   visitor->visit_table_share(share);
 
+#ifdef LATER
+  if (share->get_refcount() == 0)
+    return;
+#endif
+
   /* For all the table handles ... */
-  PFS_table *table= table_array;
-  PFS_table *table_last= table_array + table_max;
-  for ( ; table < table_last; table++)
+  Proc_one_table_share_handles proc(visitor, share);
+  global_table_container.apply(proc);
+}
+
+class Proc_one_table_share_indexes
+  : public PFS_buffer_processor<PFS_table>
+{
+public:
+  Proc_one_table_share_indexes(PFS_object_visitor *visitor, PFS_table_share *share, uint index)
+    : m_visitor(visitor), m_share(share), m_index(index)
+  {}
+
+  virtual void operator()(PFS_table *pfs)
   {
-    if ((table->m_share == share) && table->m_lock.is_populated())
+    if (pfs->m_share == m_share)
     {
-      visitor->visit_table(table);
+      m_visitor->visit_table_index(pfs, m_index);
     }
   }
-}
+
+private:
+  PFS_object_visitor* m_visitor;
+  PFS_table_share* m_share;
+  uint m_index;
+};
 
 void PFS_object_iterator::visit_table_indexes(PFS_table_share *share,
                                               uint index,
@@ -640,18 +871,19 @@ void PFS_object_iterator::visit_table_indexes(PFS_table_share *share,
 {
   DBUG_ASSERT(visitor != NULL);
 
+  if (!share->m_enabled)
+    return;
+
   visitor->visit_table_share_index(share, index);
 
+#ifdef LATER
+  if (share->get_refcount() == 0)
+    return;
+#endif
+
   /* For all the table handles ... */
-  PFS_table *table= table_array;
-  PFS_table *table_last= table_array + table_max;
-  for ( ; table < table_last; table++)
-  {
-    if ((table->m_share == share) && table->m_lock.is_populated())
-    {
-      visitor->visit_table_index(table, index);
-    }
-  }
+  Proc_one_table_share_indexes proc(visitor, share, index);
+  global_table_container.apply(proc);
 }
 
 /** Connection wait visitor */
@@ -668,32 +900,62 @@ PFS_connection_wait_visitor::~PFS_connection_wait_visitor()
 void PFS_connection_wait_visitor::visit_global()
 {
   /*
-    This visitor is used only for idle instruments.
+    This visitor is used only for global instruments
+    that do not have instances.
     For waits, do not sum by connection but by instances,
     it is more efficient.
   */
-  DBUG_ASSERT(m_index == global_idle_class.m_event_name_index);
-  m_stat.aggregate(& global_idle_stat);
+  DBUG_ASSERT(   (m_index == global_idle_class.m_event_name_index)
+              || (m_index == global_metadata_class.m_event_name_index));
+
+  if (m_index == global_idle_class.m_event_name_index)
+  {
+    m_stat.aggregate(& global_idle_stat);
+  }
+  else
+  {
+    m_stat.aggregate(& global_metadata_stat);
+  }
 }
 
 void PFS_connection_wait_visitor::visit_host(PFS_host *pfs)
 {
-  m_stat.aggregate(& pfs->m_instr_class_waits_stats[m_index]);
+  const PFS_single_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_waits_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
 }
 
 void PFS_connection_wait_visitor::visit_user(PFS_user *pfs)
 {
-  m_stat.aggregate(& pfs->m_instr_class_waits_stats[m_index]);
+  const PFS_single_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_waits_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
 }
 
 void PFS_connection_wait_visitor::visit_account(PFS_account *pfs)
 {
-  m_stat.aggregate(& pfs->m_instr_class_waits_stats[m_index]);
+  const PFS_single_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_waits_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
 }
 
 void PFS_connection_wait_visitor::visit_thread(PFS_thread *pfs)
 {
-  m_stat.aggregate(& pfs->m_instr_class_waits_stats[m_index]);
+  const PFS_single_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_waits_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
 }
 
 PFS_connection_all_wait_visitor
@@ -711,11 +973,14 @@ void PFS_connection_all_wait_visitor::visit_global()
 
 void PFS_connection_all_wait_visitor::visit_connection_slice(PFS_connection_slice *pfs)
 {
-  PFS_single_stat *stat= pfs->m_instr_class_waits_stats;
-  PFS_single_stat *stat_last= stat + wait_class_max;
-  for ( ; stat < stat_last; stat++)
+  const PFS_single_stat *stat= pfs->read_instr_class_waits_stats();
+  if (stat != NULL)
   {
-    m_stat.aggregate(stat);
+    const PFS_single_stat *stat_last= stat + wait_class_max;
+    for ( ; stat < stat_last; stat++)
+    {
+      m_stat.aggregate(stat);
+    }
   }
 }
 
@@ -754,22 +1019,42 @@ void PFS_connection_stage_visitor::visit_global()
 
 void PFS_connection_stage_visitor::visit_host(PFS_host *pfs)
 {
-  m_stat.aggregate(& pfs->m_instr_class_stages_stats[m_index]);
+  const PFS_stage_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_stages_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
 }
 
 void PFS_connection_stage_visitor::visit_user(PFS_user *pfs)
 {
-  m_stat.aggregate(& pfs->m_instr_class_stages_stats[m_index]);
+  const PFS_stage_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_stages_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
 }
 
 void PFS_connection_stage_visitor::visit_account(PFS_account *pfs)
 {
-  m_stat.aggregate(& pfs->m_instr_class_stages_stats[m_index]);
+  const PFS_stage_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_stages_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
 }
 
 void PFS_connection_stage_visitor::visit_thread(PFS_thread *pfs)
 {
-  m_stat.aggregate(& pfs->m_instr_class_stages_stats[m_index]);
+  const PFS_stage_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_stages_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
 }
 
 PFS_connection_statement_visitor
@@ -788,22 +1073,42 @@ void PFS_connection_statement_visitor::visit_global()
 
 void PFS_connection_statement_visitor::visit_host(PFS_host *pfs)
 {
-  m_stat.aggregate(& pfs->m_instr_class_statements_stats[m_index]);
+  const PFS_statement_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_statements_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
 }
 
 void PFS_connection_statement_visitor::visit_user(PFS_user *pfs)
 {
-  m_stat.aggregate(& pfs->m_instr_class_statements_stats[m_index]);
+  const PFS_statement_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_statements_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
 }
 
 void PFS_connection_statement_visitor::visit_account(PFS_account *pfs)
 {
-  m_stat.aggregate(& pfs->m_instr_class_statements_stats[m_index]);
+  const PFS_statement_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_statements_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
 }
 
 void PFS_connection_statement_visitor::visit_thread(PFS_thread *pfs)
 {
-  m_stat.aggregate(& pfs->m_instr_class_statements_stats[m_index]);
+  const PFS_statement_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_statements_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
 }
 
 /** Instance wait visitor */
@@ -826,11 +1131,14 @@ void PFS_connection_all_statement_visitor::visit_global()
 
 void PFS_connection_all_statement_visitor::visit_connection_slice(PFS_connection_slice *pfs)
 {
-  PFS_statement_stat *stat= pfs->m_instr_class_statements_stats;
-  PFS_statement_stat *stat_last= stat + statement_class_max;
-  for ( ; stat < stat_last; stat++)
+  const PFS_statement_stat *stat= pfs->read_instr_class_statements_stats();
+  if (stat != NULL)
   {
-    m_stat.aggregate(stat);
+    const PFS_statement_stat *stat_last= stat + statement_class_max;
+    for ( ; stat < stat_last; stat++)
+    {
+      m_stat.aggregate(stat);
+    }
   }
 }
 
@@ -854,6 +1162,102 @@ void PFS_connection_all_statement_visitor::visit_thread(PFS_thread *pfs)
   visit_connection_slice(pfs);
 }
 
+PFS_connection_transaction_visitor
+::PFS_connection_transaction_visitor(PFS_transaction_class *klass)
+{
+  m_index= klass->m_event_name_index;
+}
+
+PFS_connection_transaction_visitor::~PFS_connection_transaction_visitor()
+{}
+
+void PFS_connection_transaction_visitor::visit_global()
+{
+  m_stat.aggregate(&global_transaction_stat);
+}
+
+void PFS_connection_transaction_visitor::visit_host(PFS_host *pfs)
+{
+  const PFS_transaction_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_transactions_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
+}
+
+void PFS_connection_transaction_visitor::visit_user(PFS_user *pfs)
+{
+  const PFS_transaction_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_transactions_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
+}
+
+void PFS_connection_transaction_visitor::visit_account(PFS_account *pfs)
+{
+  const PFS_transaction_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_transactions_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
+}
+
+void PFS_connection_transaction_visitor::visit_thread(PFS_thread *pfs)
+{
+  const PFS_transaction_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_transactions_stats();
+  if (event_name_array != NULL)
+  {
+    m_stat.aggregate(& event_name_array[m_index]);
+  }
+}
+
+/** Disabled pending code review */
+#if 0
+/** Instance wait visitor */
+PFS_connection_all_transaction_visitor
+::PFS_connection_all_transaction_visitor()
+{}
+
+PFS_connection_all_transaction_visitor::~PFS_connection_all_transaction_visitor()
+{}
+
+void PFS_connection_all_transaction_visitor::visit_global()
+{
+  m_stat.aggregate(&global_transaction_stat);
+}
+
+void PFS_connection_all_transaction_visitor::visit_connection_slice(PFS_connection_slice *pfs)
+{
+  PFS_transaction_stat *stat= pfs->m_instr_class_transactions_stats;
+  m_stat.aggregate(stat);
+}
+
+void PFS_connection_all_transaction_visitor::visit_host(PFS_host *pfs)
+{
+  visit_connection_slice(pfs);
+}
+
+void PFS_connection_all_transaction_visitor::visit_user(PFS_user *pfs)
+{
+  visit_connection_slice(pfs);
+}
+
+void PFS_connection_all_transaction_visitor::visit_account(PFS_account *pfs)
+{
+  visit_connection_slice(pfs);
+}
+
+void PFS_connection_all_transaction_visitor::visit_thread(PFS_thread *pfs)
+{
+  visit_connection_slice(pfs);
+}
+#endif
+
 PFS_connection_stat_visitor::PFS_connection_stat_visitor()
 {}
 
@@ -889,10 +1293,119 @@ void PFS_connection_stat_visitor::visit_thread(PFS_thread *pfs)
     m_stat.aggregate_active(1);
 }
 
-PFS_instance_wait_visitor::PFS_instance_wait_visitor()
+PFS_connection_memory_visitor
+::PFS_connection_memory_visitor(PFS_memory_class *klass)
+{
+  m_index= klass->m_event_name_index;
+  m_stat.reset();
+}
+
+PFS_connection_memory_visitor::~PFS_connection_memory_visitor()
+{}
+
+void PFS_connection_memory_visitor::visit_global()
+{
+  PFS_memory_stat *stat;
+  stat= & global_instr_class_memory_array[m_index];
+  stat->full_aggregate_to(& m_stat);
+}
+
+void PFS_connection_memory_visitor::visit_host(PFS_host *pfs)
+{
+  const PFS_memory_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_memory_stats();
+  if (event_name_array != NULL)
+  {
+    const PFS_memory_stat *stat;
+    stat= & event_name_array[m_index];
+    stat->full_aggregate_to(& m_stat);
+  }
+}
+
+void PFS_connection_memory_visitor::visit_user(PFS_user *pfs)
+{
+  const PFS_memory_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_memory_stats();
+  if (event_name_array != NULL)
+  {
+    const PFS_memory_stat *stat;
+    stat= & event_name_array[m_index];
+    stat->full_aggregate_to(& m_stat);
+  }
+}
+
+void PFS_connection_memory_visitor::visit_account(PFS_account *pfs)
+{
+  const PFS_memory_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_memory_stats();
+  if (event_name_array != NULL)
+  {
+    const PFS_memory_stat *stat;
+    stat= & event_name_array[m_index];
+    stat->full_aggregate_to(& m_stat);
+  }
+}
+
+void PFS_connection_memory_visitor::visit_thread(PFS_thread *pfs)
+{
+  const PFS_memory_stat *event_name_array;
+  event_name_array= pfs->read_instr_class_memory_stats();
+  if (event_name_array != NULL)
+  {
+    const PFS_memory_stat *stat;
+    stat= & event_name_array[m_index];
+    stat->full_aggregate_to(& m_stat);
+  }
+}
+
+
+PFS_connection_status_visitor::
+PFS_connection_status_visitor(STATUS_VAR *status_vars) : m_status_vars(status_vars)
+{
+  memset(m_status_vars, 0, sizeof(STATUS_VAR));
+}
+
+PFS_connection_status_visitor::~PFS_connection_status_visitor()
+{}
+
+/** Aggregate from global status. */
+void PFS_connection_status_visitor::visit_global()
+{
+   /* NOTE: Requires lock on LOCK_status. */
+   mysql_mutex_assert_owner(&LOCK_status);
+   add_to_status(m_status_vars, &global_status_var, false);
+}
+
+void PFS_connection_status_visitor::visit_host(PFS_host *pfs)
+{
+  pfs->m_status_stats.aggregate_to(m_status_vars);
+}
+
+void PFS_connection_status_visitor::visit_user(PFS_user *pfs)
+{
+  pfs->m_status_stats.aggregate_to(m_status_vars);
+}
+
+void PFS_connection_status_visitor::visit_account(PFS_account *pfs)
+{
+  pfs->m_status_stats.aggregate_to(m_status_vars);
+}
+
+void PFS_connection_status_visitor::visit_thread(PFS_thread *pfs)
 {
 }
 
+void PFS_connection_status_visitor::visit_THD(THD *thd)
+{
+  if (!thd->status_var_aggregated) {
+    add_to_status(m_status_vars, &thd->status_var, false);
+  }
+}
+
+
+PFS_instance_wait_visitor::PFS_instance_wait_visitor()
+{}
+
 PFS_instance_wait_visitor::~PFS_instance_wait_visitor()
 {}
 
@@ -936,7 +1449,7 @@ void PFS_instance_wait_visitor::visit_cond(PFS_cond *pfs)
   m_stat.aggregate(& pfs->m_cond_stat.m_wait_stat);
 }
 
-void PFS_instance_wait_visitor::visit_file(PFS_file *pfs) 
+void PFS_instance_wait_visitor::visit_file(PFS_file *pfs)
 {
   /* Combine per-operation file wait stats before aggregating */
   PFS_single_stat stat;
@@ -944,7 +1457,7 @@ void PFS_instance_wait_visitor::visit_file(PFS_file *pfs)
   m_stat.aggregate(&stat);
 }
 
-void PFS_instance_wait_visitor::visit_socket(PFS_socket *pfs) 
+void PFS_instance_wait_visitor::visit_socket(PFS_socket *pfs)
 {
   /* Combine per-operation socket wait stats before aggregating */
   PFS_single_stat stat;
@@ -969,7 +1482,7 @@ void PFS_object_wait_visitor::visit_global()
 void PFS_object_wait_visitor::visit_table_share(PFS_table_share *pfs)
 {
   uint safe_key_count= sanitize_index_count(pfs->m_key_count);
-  pfs->m_table_stat.sum(& m_stat, safe_key_count);
+  pfs->sum(& m_stat, safe_key_count);
 }
 
 void PFS_object_wait_visitor::visit_table(PFS_table *pfs)
@@ -998,13 +1511,20 @@ void PFS_table_io_wait_visitor::visit_table_share(PFS_table_share *pfs)
   PFS_table_io_stat io_stat;
   uint safe_key_count= sanitize_index_count(pfs->m_key_count);
   uint index;
+  PFS_table_share_index *index_stat;
 
   /* Aggregate index stats */
   for (index= 0; index < safe_key_count; index++)
-    io_stat.aggregate(& pfs->m_table_stat.m_index_stat[index]);
+  {
+    index_stat= pfs->find_index_stat(index);
+    if (index_stat != NULL)
+      io_stat.aggregate(& index_stat->m_stat);
+  }
 
   /* Aggregate global stats */
-  io_stat.aggregate(& pfs->m_table_stat.m_index_stat[MAX_INDEXES]);
+  index_stat= pfs->find_index_stat(MAX_INDEXES);
+  if (index_stat != NULL)
+    io_stat.aggregate(& index_stat->m_stat);
 
   io_stat.sum(& m_stat);
 }
@@ -1042,13 +1562,20 @@ void PFS_table_io_stat_visitor::visit_table_share(PFS_table_share *pfs)
 {
   uint safe_key_count= sanitize_index_count(pfs->m_key_count);
   uint index;
+  PFS_table_share_index *index_stat;
 
   /* Aggregate index stats */
   for (index= 0; index < safe_key_count; index++)
-    m_stat.aggregate(& pfs->m_table_stat.m_index_stat[index]);
+  {
+    index_stat= pfs->find_index_stat(index);
+    if (index_stat != NULL)
+      m_stat.aggregate(& index_stat->m_stat);
+  }
 
   /* Aggregate global stats */
-  m_stat.aggregate(& pfs->m_table_stat.m_index_stat[MAX_INDEXES]);
+  index_stat= pfs->find_index_stat(MAX_INDEXES);
+  if (index_stat != NULL)
+    m_stat.aggregate(& index_stat->m_stat);
 }
 
 void PFS_table_io_stat_visitor::visit_table(PFS_table *pfs)
@@ -1079,7 +1606,11 @@ PFS_index_io_stat_visitor::~PFS_index_io_stat_visitor()
 
 void PFS_index_io_stat_visitor::visit_table_share_index(PFS_table_share *pfs, uint index)
 {
-  m_stat.aggregate(& pfs->m_table_stat.m_index_stat[index]);
+  PFS_table_share_index *index_stat;
+
+  index_stat= pfs->find_index_stat(index);
+  if (index_stat != NULL)
+    m_stat.aggregate(& index_stat->m_stat);
 }
 
 void PFS_index_io_stat_visitor::visit_table_index(PFS_table *pfs, uint index)
@@ -1102,7 +1633,7 @@ void PFS_table_lock_wait_visitor::visit_global()
 
 void PFS_table_lock_wait_visitor::visit_table_share(PFS_table_share *pfs)
 {
-  pfs->m_table_stat.sum_lock(& m_stat);
+  pfs->sum_lock(& m_stat);
 }
 
 void PFS_table_lock_wait_visitor::visit_table(PFS_table *pfs)
@@ -1120,7 +1651,11 @@ PFS_table_lock_stat_visitor::~PFS_table_lock_stat_visitor()
 
 void PFS_table_lock_stat_visitor::visit_table_share(PFS_table_share *pfs)
 {
-  m_stat.aggregate(& pfs->m_table_stat.m_lock_stat);
+  PFS_table_share_lock *lock_stat;
+
+  lock_stat= pfs->find_lock_stat();
+  if (lock_stat != NULL)
+    m_stat.aggregate(& lock_stat->m_stat);
 }
 
 void PFS_table_lock_stat_visitor::visit_table(PFS_table *pfs)
@@ -1134,32 +1669,31 @@ PFS_instance_socket_io_stat_visitor::PFS_instance_socket_io_stat_visitor()
 PFS_instance_socket_io_stat_visitor::~PFS_instance_socket_io_stat_visitor()
 {}
 
-void PFS_instance_socket_io_stat_visitor::visit_socket_class(PFS_socket_class *pfs) 
+void PFS_instance_socket_io_stat_visitor::visit_socket_class(PFS_socket_class *pfs)
 {
   /* Aggregate wait times, event counts and byte counts */
   m_socket_io_stat.aggregate(&pfs->m_socket_stat.m_io_stat);
 }
 
-void PFS_instance_socket_io_stat_visitor::visit_socket(PFS_socket *pfs) 
+void PFS_instance_socket_io_stat_visitor::visit_socket(PFS_socket *pfs)
 {
   /* Aggregate wait times, event counts and byte counts */
   m_socket_io_stat.aggregate(&pfs->m_socket_stat.m_io_stat);
 }
 
-
 PFS_instance_file_io_stat_visitor::PFS_instance_file_io_stat_visitor()
 {}
 
 PFS_instance_file_io_stat_visitor::~PFS_instance_file_io_stat_visitor()
 {}
 
-void PFS_instance_file_io_stat_visitor::visit_file_class(PFS_file_class *pfs) 
+void PFS_instance_file_io_stat_visitor::visit_file_class(PFS_file_class *pfs)
 {
   /* Aggregate wait times, event counts and byte counts */
   m_file_io_stat.aggregate(&pfs->m_file_stat.m_io_stat);
 }
 
-void PFS_instance_file_io_stat_visitor::visit_file(PFS_file *pfs) 
+void PFS_instance_file_io_stat_visitor::visit_file(PFS_file *pfs)
 {
   /* Aggregate wait times, event counts and byte counts */
   m_file_io_stat.aggregate(&pfs->m_file_stat.m_io_stat);
