@@ -24,9 +24,9 @@
 #include <vector>
 
 /* MySQL header files */
-#include "./log.h"
-#include "./my_stacktrace.h"
-#include "./sql_array.h"
+#include "log.h"
+#include "my_stacktrace.h"
+#include "sql_array.h"
 
 /* MyRocks header files */
 #include "./rdb_datadic.h"
@@ -43,12 +43,19 @@ my_bool rocksdb_compaction_sequential_deletes_count_sd = false;
 
 Rdb_tbl_prop_coll::Rdb_tbl_prop_coll(Rdb_ddl_manager *const ddl_manager,
                                      const Rdb_compact_params &params,
-                                     const uint32_t &cf_id,
-                                     const uint8_t &table_stats_sampling_pct)
-    : m_cf_id(cf_id), m_ddl_manager(ddl_manager), m_last_stats(nullptr),
-      m_rows(0l), m_window_pos(0l), m_deleted_rows(0l), m_max_deleted_rows(0l),
-      m_file_size(0), m_params(params),
-      m_cardinality_collector(table_stats_sampling_pct) {
+                                     const uint32_t cf_id,
+                                     const uint8_t table_stats_sampling_pct)
+    : m_cf_id(cf_id),
+      m_ddl_manager(ddl_manager),
+      m_last_stats(nullptr),
+      m_rows(0l),
+      m_window_pos(0l),
+      m_deleted_rows(0l),
+      m_max_deleted_rows(0l),
+      m_file_size(0),
+      m_params(params),
+      m_cardinality_collector(table_stats_sampling_pct),
+      m_recorded(false) {
   DBUG_ASSERT(ddl_manager != nullptr);
 
   m_deleted_rows_window.resize(m_params.m_window, false);
@@ -142,35 +149,36 @@ Rdb_index_stats *Rdb_tbl_prop_coll::AccessStats(const rocksdb::Slice &key) {
 void Rdb_tbl_prop_coll::CollectStatsForRow(const rocksdb::Slice &key,
                                            const rocksdb::Slice &value,
                                            const rocksdb::EntryType &type,
-                                           const uint64_t &file_size) {
+                                           const uint64_t file_size) {
   auto stats = AccessStats(key);
 
   stats->m_data_size += key.size() + value.size();
 
   // Incrementing per-index entry-type statistics
   switch (type) {
-  case rocksdb::kEntryPut:
-    stats->m_rows++;
-    break;
-  case rocksdb::kEntryDelete:
-    stats->m_entry_deletes++;
-    break;
-  case rocksdb::kEntrySingleDelete:
-    stats->m_entry_single_deletes++;
-    break;
-  case rocksdb::kEntryMerge:
-    stats->m_entry_merges++;
-    break;
-  case rocksdb::kEntryOther:
-    stats->m_entry_others++;
-    break;
-  default:
-    // NO_LINT_DEBUG
-    sql_print_error("RocksDB: Unexpected entry type found: %u. "
-                    "This should not happen so aborting the system.",
-                    type);
-    abort_with_stack_traces();
-    break;
+    case rocksdb::kEntryPut:
+      stats->m_rows++;
+      break;
+    case rocksdb::kEntryDelete:
+      stats->m_entry_deletes++;
+      break;
+    case rocksdb::kEntrySingleDelete:
+      stats->m_entry_single_deletes++;
+      break;
+    case rocksdb::kEntryMerge:
+      stats->m_entry_merges++;
+      break;
+    case rocksdb::kEntryOther:
+      stats->m_entry_others++;
+      break;
+    default:
+      // NO_LINT_DEBUG
+      sql_print_error(
+          "RocksDB: Unexpected entry type found: %u. "
+          "This should not happen so aborting the system.",
+          type);
+      abort();
+      break;
   }
 
   stats->m_actual_disk_size += file_size - m_file_size;
@@ -186,8 +194,8 @@ const char *Rdb_tbl_prop_coll::INDEXSTATS_KEY = "__indexstats__";
 /*
   This function is called by RocksDB to compute properties to store in sst file
 */
-rocksdb::Status
-Rdb_tbl_prop_coll::Finish(rocksdb::UserCollectedProperties *const properties) {
+rocksdb::Status Rdb_tbl_prop_coll::Finish(
+    rocksdb::UserCollectedProperties *const properties) {
   uint64_t num_sst_entry_put = 0;
   uint64_t num_sst_entry_delete = 0;
   uint64_t num_sst_entry_singledelete = 0;
@@ -204,28 +212,32 @@ Rdb_tbl_prop_coll::Finish(rocksdb::UserCollectedProperties *const properties) {
     num_sst_entry_other += it->m_entry_others;
   }
 
-  if (num_sst_entry_put > 0) {
-    rocksdb_num_sst_entry_put += num_sst_entry_put;
-  }
+  if (!m_recorded) {
+    if (num_sst_entry_put > 0) {
+      rocksdb_num_sst_entry_put += num_sst_entry_put;
+    }
 
-  if (num_sst_entry_delete > 0) {
-    rocksdb_num_sst_entry_delete += num_sst_entry_delete;
-  }
+    if (num_sst_entry_delete > 0) {
+      rocksdb_num_sst_entry_delete += num_sst_entry_delete;
+    }
 
-  if (num_sst_entry_singledelete > 0) {
-    rocksdb_num_sst_entry_singledelete += num_sst_entry_singledelete;
-  }
+    if (num_sst_entry_singledelete > 0) {
+      rocksdb_num_sst_entry_singledelete += num_sst_entry_singledelete;
+    }
 
-  if (num_sst_entry_merge > 0) {
-    rocksdb_num_sst_entry_merge += num_sst_entry_merge;
-  }
+    if (num_sst_entry_merge > 0) {
+      rocksdb_num_sst_entry_merge += num_sst_entry_merge;
+    }
 
-  if (num_sst_entry_other > 0) {
-    rocksdb_num_sst_entry_other += num_sst_entry_other;
-  }
+    if (num_sst_entry_other > 0) {
+      rocksdb_num_sst_entry_other += num_sst_entry_other;
+    }
 
-  for (Rdb_index_stats &stat : m_stats) {
-    m_cardinality_collector.AdjustStats(&stat);
+    for (Rdb_index_stats &stat : m_stats) {
+      m_cardinality_collector.SetCardinality(&stat);
+      m_cardinality_collector.AdjustStats(&stat);
+    }
+    m_recorded = true;
   }
   properties->insert({INDEXSTATS_KEY, Rdb_index_stats::materialize(m_stats)});
   return rocksdb::Status::OK();
@@ -240,8 +252,8 @@ bool Rdb_tbl_prop_coll::NeedCompact() const {
 /*
   Returns the same as above, but in human-readable way for logging
 */
-rocksdb::UserCollectedProperties
-Rdb_tbl_prop_coll::GetReadableProperties() const {
+rocksdb::UserCollectedProperties Rdb_tbl_prop_coll::GetReadableProperties()
+    const {
   std::string s;
 #ifdef DBUG_OFF
   s.append("[...");
@@ -249,7 +261,7 @@ Rdb_tbl_prop_coll::GetReadableProperties() const {
   s.append("  records...]");
 #else
   bool first = true;
-  for (auto it : m_stats) {
+  for (const auto &it : m_stats) {
     if (first) {
       first = false;
     } else {
@@ -312,8 +324,8 @@ void Rdb_tbl_prop_coll::read_stats_from_tbl_props(
 /*
   Serializes an array of Rdb_index_stats into a network string.
 */
-std::string
-Rdb_index_stats::materialize(const std::vector<Rdb_index_stats> &stats) {
+std::string Rdb_index_stats::materialize(
+    const std::vector<Rdb_index_stats> &stats) {
   String ret;
   rdb_netstr_append_uint16(&ret, INDEX_STATS_VERSION_ENTRY_TYPES);
   for (const auto &i : stats) {
@@ -359,10 +371,11 @@ int Rdb_index_stats::unmaterialize(const std::string &s,
   if (version < INDEX_STATS_VERSION_INITIAL ||
       version > INDEX_STATS_VERSION_ENTRY_TYPES) {
     // NO_LINT_DEBUG
-    sql_print_error("Index stats version %d was outside of supported range. "
-                    "This should not happen so aborting the system.",
-                    version);
-    abort_with_stack_traces();
+    sql_print_error(
+        "Index stats version %d was outside of supported range. "
+        "This should not happen so aborting the system.",
+        version);
+    abort();
   }
 
   size_t needed = sizeof(stats.m_gl_index_id.cf_id) +
@@ -390,8 +403,7 @@ int Rdb_index_stats::unmaterialize(const std::string &s,
       stats.m_entry_merges = rdb_netbuf_read_uint64(&p);
       stats.m_entry_others = rdb_netbuf_read_uint64(&p);
     }
-    if (p +
-            stats.m_distinct_keys_per_prefix.size() *
+    if (p + stats.m_distinct_keys_per_prefix.size() *
                 sizeof(stats.m_distinct_keys_per_prefix[0]) >
         p2) {
       return HA_EXIT_FAILURE;
@@ -404,12 +416,18 @@ int Rdb_index_stats::unmaterialize(const std::string &s,
   return HA_EXIT_SUCCESS;
 }
 
+void Rdb_index_stats::reset_cardinality() {
+  for (size_t i = 0; i < m_distinct_keys_per_prefix.size(); i++) {
+    m_distinct_keys_per_prefix[i] = 0;
+  }
+}
+
 /*
   Merges one Rdb_index_stats into another. Can be used to come up with the stats
   for the index based on stats for each sst
 */
-void Rdb_index_stats::merge(const Rdb_index_stats &s, const bool &increment,
-                            const int64_t &estimated_data_len) {
+void Rdb_index_stats::merge(const Rdb_index_stats &s, const bool increment,
+                            const int64_t estimated_data_len) {
   std::size_t i;
 
   DBUG_ASSERT(estimated_data_len >= 0);
@@ -435,15 +453,8 @@ void Rdb_index_stats::merge(const Rdb_index_stats &s, const bool &increment,
     m_entry_single_deletes += s.m_entry_single_deletes;
     m_entry_merges += s.m_entry_merges;
     m_entry_others += s.m_entry_others;
-    if (s.m_distinct_keys_per_prefix.size() > 0) {
-      for (i = 0; i < s.m_distinct_keys_per_prefix.size(); i++) {
-        m_distinct_keys_per_prefix[i] += s.m_distinct_keys_per_prefix[i];
-      }
-    } else {
-      for (i = 0; i < m_distinct_keys_per_prefix.size(); i++) {
-        m_distinct_keys_per_prefix[i] +=
-            s.m_rows >> (m_distinct_keys_per_prefix.size() - i - 1);
-      }
+    for (i = 0; i < s.m_distinct_keys_per_prefix.size(); i++) {
+      m_distinct_keys_per_prefix[i] += s.m_distinct_keys_per_prefix[i];
     }
   } else {
     m_rows -= s.m_rows;
@@ -454,20 +465,19 @@ void Rdb_index_stats::merge(const Rdb_index_stats &s, const bool &increment,
     m_entry_single_deletes -= s.m_entry_single_deletes;
     m_entry_merges -= s.m_entry_merges;
     m_entry_others -= s.m_entry_others;
-    if (s.m_distinct_keys_per_prefix.size() > 0) {
-      for (i = 0; i < s.m_distinct_keys_per_prefix.size(); i++) {
-        m_distinct_keys_per_prefix[i] -= s.m_distinct_keys_per_prefix[i];
-      }
-    } else {
-      for (i = 0; i < m_distinct_keys_per_prefix.size(); i++) {
-        m_distinct_keys_per_prefix[i] -=
-            s.m_rows >> (m_distinct_keys_per_prefix.size() - i - 1);
-      }
+    for (i = 0; i < s.m_distinct_keys_per_prefix.size(); i++) {
+      m_distinct_keys_per_prefix[i] -= s.m_distinct_keys_per_prefix[i];
     }
   }
 }
 
-Rdb_tbl_card_coll::Rdb_tbl_card_coll(uint8_t table_stats_sampling_pct)
+void Rdb_index_stats::adjust_cardinality(uint64_t adjustment_factor) {
+  for (int64_t &num_keys : m_distinct_keys_per_prefix) {
+    num_keys = num_keys * adjustment_factor;
+  }
+}
+
+Rdb_tbl_card_coll::Rdb_tbl_card_coll(const uint8_t table_stats_sampling_pct)
     : m_table_stats_sampling_pct(table_stats_sampling_pct),
       m_seed(time(nullptr)) {}
 
@@ -507,16 +517,12 @@ void Rdb_tbl_card_coll::ProcessKey(const rocksdb::Slice &key,
     if (new_key) {
       DBUG_ASSERT(column <= stats->m_distinct_keys_per_prefix.size());
 
-      for (auto i = column; i < stats->m_distinct_keys_per_prefix.size(); i++) {
-        stats->m_distinct_keys_per_prefix[i]++;
-      }
-
-      // assign new last_key for the next call
-      // however, we only need to change the last key
-      // if one of the first n-1 columns is different
-      // If the n-1 prefix is the same, no sense in storing
-      // the new key
       if (column < stats->m_distinct_keys_per_prefix.size()) {
+        // At this point, stats->m_distinct_keys_per_prefix does
+        // not hold its final values.
+        // After all keys are processed, SetCardinality() needs
+        // to be called to calculate the final values.
+        stats->m_distinct_keys_per_prefix[column]++;
         m_last_key.assign(key.data(), key.size());
       }
     }
@@ -538,4 +544,11 @@ void Rdb_tbl_card_coll::AdjustStats(Rdb_index_stats *stats) {
   }
 }
 
-} // namespace myrocks
+void Rdb_tbl_card_coll::SetCardinality(Rdb_index_stats *stats) {
+  for (size_t i = 1; i < stats->m_distinct_keys_per_prefix.size(); i++) {
+    stats->m_distinct_keys_per_prefix[i] +=
+        stats->m_distinct_keys_per_prefix[i - 1];
+  }
+}
+
+}  // namespace myrocks
