@@ -7,7 +7,7 @@ This file is part of TokuDB
 
 Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 
-    TokuDBis is free software: you can redistribute it and/or modify
+    TokuDB is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License, version 2,
     as published by the Free Software Foundation.
 
@@ -26,6 +26,8 @@ Copyright (c) 2006, 2015, Percona and/or its affiliates. All rights reserved.
 #include "tokudb_sysvars.h"
 #include "toku_time.h"
 
+#include "my_check_opt.h"
+
 namespace tokudb {
 namespace analyze {
 
@@ -43,13 +45,11 @@ public:
     virtual ~recount_rows_t();
 
     virtual const char* key();
-
-    virtual void status(
-        char* database,
-        char* table,
-        char* type,
-        char* params,
-        char* status);
+    virtual const char* database();
+    virtual const char* table();
+    virtual const char* type();
+    virtual const char* parameters();
+    virtual const char* status();
 
 protected:
     virtual void on_run();
@@ -64,6 +64,8 @@ private:
     ulonglong       _throttle;
 
     // for recount rows status reporting
+    char            _parameters[256];
+    char            _status[1024];
     int             _result;
     ulonglong       _recount_start; // in microseconds
     ulonglong       _total_elapsed_time; // in microseconds
@@ -78,7 +80,6 @@ private:
         uint64_t deleted,
         void* extra);
     int analyze_recount_rows_progress(uint64_t count, uint64_t deleted);
-    void get_analyze_status(char*);
 };
 
 void* recount_rows_t::operator new(size_t sz) {
@@ -114,10 +115,19 @@ recount_rows_t::recount_rows_t(
     }
 
     _throttle = tokudb::sysvars::analyze_throttle(thd);
+
+    snprintf(_parameters,
+             sizeof(_parameters),
+             "TOKUDB_ANALYZE_THROTTLE=%llu;",
+             _throttle);
+    _status[0] = '\0';
 }
 recount_rows_t::~recount_rows_t() {
 }
 void recount_rows_t::on_run() {
+    const char* orig_proc_info = NULL;
+    if (_thd)
+        orig_proc_info = tokudb_thd_get_proc_info(_thd);
     _recount_start = tokudb::time::microsec();
     _total_elapsed_time = 0;
 
@@ -171,6 +181,8 @@ void recount_rows_t::on_run() {
         _result,
         _share->row_count());
 error:
+    if(_thd)
+        tokudb_thd_set_proc_info(_thd, orig_proc_info);
     return;
 }
 void recount_rows_t::on_destroy() {
@@ -179,18 +191,21 @@ void recount_rows_t::on_destroy() {
 const char* recount_rows_t::key() {
     return _share->full_table_name();
 }
-void recount_rows_t::status(
-    char* database,
-    char* table,
-    char* type,
-    char* params,
-    char* status) {
-
-    strcpy(database, _share->database_name());
-    strcpy(table, _share->table_name());
-    strcpy(type, "TOKUDB_ANALYZE_MODE_RECOUNT_ROWS");
-    sprintf(params, "TOKUDB_ANALYZE_THROTTLE=%llu;", _throttle);
-    get_analyze_status(status);
+const char* recount_rows_t::database() {
+    return _share->database_name();
+}
+const char* recount_rows_t::table() {
+    return _share->table_name();
+}
+const char* recount_rows_t::type() {
+    static const char* type = "TOKUDB_ANALYZE_MODE_RECOUNT_ROWS";
+    return type;
+}
+const char* recount_rows_t::parameters() {
+    return _parameters;
+}
+const char* recount_rows_t::status() {
+    return _status;
 }
 int recount_rows_t::analyze_recount_rows_progress(
     uint64_t count,
@@ -217,12 +232,32 @@ int recount_rows_t::analyze_recount_rows_progress(
             return ER_ABORTING_CONNECTION;
         }
 
+        // rebuild status
+        // There is a slight race condition here,
+        // _status is used here for tokudb_thd_set_proc_info and it is also used
+        // for the status column in i_s.background_job_status.
+        // If someone happens to be querying/building the i_s table
+        // at the exact same time that the status is being rebuilt here,
+        // the i_s table could get some garbage status.
+        // This solution is a little heavy handed but it works, it prevents us
+        // from changing the status while someone might be immediately observing
+        // us and it prevents someone from observing us while we change the
+        // status
+        tokudb::background::_job_manager->lock();
+        snprintf(_status,
+                 sizeof(_status),
+                 "recount_rows %s.%s counted %llu rows and %llu deleted "
+                 "in %llu seconds.",
+                 _share->database_name(),
+                 _share->table_name(),
+                 _rows,
+                 _deleted_rows,
+                 _total_elapsed_time / tokudb::time::MICROSECONDS);
+        tokudb::background::_job_manager->unlock();
+
         // report
-        if (_thd) {
-            char status[256];
-            get_analyze_status(status);
-            thd_proc_info(_thd, status);
-        }
+        if (_thd)
+            tokudb_thd_set_proc_info(_thd, _status);
 
         // throttle
         // given the throttle value, lets calculate the maximum number of rows
@@ -238,18 +273,6 @@ int recount_rows_t::analyze_recount_rows_progress(
     }
     return 0;
 }
-void recount_rows_t::get_analyze_status(char* msg) {
-    sprintf(
-        msg,
-        "recount_rows %s.%s counted %llu rows and %llu deleted in %llu "
-        "seconds.",
-        _share->database_name(),
-        _share->table_name(),
-        _rows,
-        _deleted_rows,
-        _total_elapsed_time / tokudb::time::MICROSECONDS);
-}
-
 
 class standard_t : public tokudb::background::job_manager_t::job_t {
 public:
@@ -261,13 +284,11 @@ public:
     virtual ~standard_t();
 
     virtual const char* key(void);
-
-    virtual void status(
-        char* database,
-        char* table,
-        char* type,
-        char* params,
-        char* status);
+    virtual const char* database();
+    virtual const char* table();
+    virtual const char* type();
+    virtual const char* parameters();
+    virtual const char* status();
 
 protected:
     virtual void on_run();
@@ -284,6 +305,8 @@ private:
     double          _delete_fraction;
 
     // for analyze status reporting, may also use other state
+    char            _parameters[256];
+    char            _status[1024];
     int             _result;
     ulonglong       _analyze_start; // in microseconds
     ulonglong       _total_elapsed_time; // in microseconds
@@ -305,7 +328,6 @@ private:
         uint64_t deleted_rows);
     bool analyze_standard_cursor_callback(uint64_t deleted_rows);
 
-    void get_analyze_status(char*);
     int analyze_key_progress();
     int analyze_key(uint64_t* rec_per_key_part);
 };
@@ -351,6 +373,16 @@ standard_t::standard_t(
     _time_limit =
         tokudb::sysvars::analyze_time(thd) * tokudb::time::MICROSECONDS;
     _delete_fraction = tokudb::sysvars::analyze_delete_fraction(thd);
+
+    snprintf(_parameters,
+             sizeof(_parameters),
+             "TOKUDB_ANALYZE_DELETE_FRACTION=%f; "
+             "TOKUDB_ANALYZE_TIME=%llu; TOKUDB_ANALYZE_THROTTLE=%llu;",
+             _delete_fraction,
+             _time_limit / tokudb::time::MICROSECONDS,
+             _throttle);
+
+    _status[0] = '\0';
 }
 standard_t::~standard_t() {
 }
@@ -358,6 +390,10 @@ void standard_t::on_run() {
     DB_BTREE_STAT64 stat64;
     uint64_t rec_per_key_part[_share->_max_key_parts];
     uint64_t total_key_parts = 0;
+    const char* orig_proc_info = NULL;
+    if (_thd)
+        orig_proc_info = tokudb_thd_get_proc_info(_thd);
+
     _analyze_start = tokudb::time::microsec();
     _half_time = _time_limit > 0 ? _time_limit/2 : 0;
 
@@ -374,6 +410,7 @@ void standard_t::on_run() {
         _local_txn = false;
     }
 
+    assert_always(_share->key_file[0] != NULL);
     _result = _share->key_file[0]->stat64(_share->key_file[0], _txn, &stat64);
     if (_result != 0) {
         _result = HA_ADMIN_FAILED;
@@ -394,7 +431,7 @@ void standard_t::on_run() {
             _result = HA_ADMIN_FAILED;
         }
         if (_thd && (_result == HA_ADMIN_FAILED ||
-            (double)_deleted_rows >
+            static_cast<double>(_deleted_rows) >
                 _delete_fraction * (_rows + _deleted_rows))) {
 
             char name[256]; int namelen;
@@ -406,10 +443,10 @@ void standard_t::on_run() {
                     _share->database_name(),
                     _share->table_name(),
                     _share->_key_descriptors[_current_key]._name);
-            _thd->protocol->prepare_for_resend();
-            _thd->protocol->store(name, namelen,  system_charset_info);
-            _thd->protocol->store("analyze", 7, system_charset_info);
-            _thd->protocol->store("info", 4, system_charset_info);
+            _thd->get_protocol()->start_row();
+            _thd->get_protocol()->store(name, namelen,  system_charset_info);
+            _thd->get_protocol()->store("analyze", 7, system_charset_info);
+            _thd->get_protocol()->store("info", 4, system_charset_info);
             char rowmsg[256];
             int rowmsglen;
             rowmsglen =
@@ -419,8 +456,9 @@ void standard_t::on_run() {
                     "rows processed %llu rows deleted %llu",
                     _rows,
                     _deleted_rows);
-            _thd->protocol->store(rowmsg, rowmsglen, system_charset_info);
-            _thd->protocol->write();
+            _thd->get_protocol()->store(rowmsg, rowmsglen,
+                                        system_charset_info);
+            _thd->get_protocol()->end_row();
 
             sql_print_information(
                 "tokudb analyze on %.*s %.*s",
@@ -459,8 +497,9 @@ cleanup:
     }
 
 error:
+    if (_thd)
+        tokudb_thd_set_proc_info(_thd, orig_proc_info);
     return;
-
 }
 void standard_t::on_destroy() {
     _share->lock();
@@ -471,24 +510,21 @@ void standard_t::on_destroy() {
 const char* standard_t::key() {
     return _share->full_table_name();
 }
-void standard_t::status(
-    char* database,
-    char* table,
-    char* type,
-    char* params,
-    char* status) {
-
-    strcpy(database, _share->database_name());
-    strcpy(table, _share->table_name());
-    strcpy(type, "TOKUDB_ANALYZE_MODE_STANDARD");
-    sprintf(
-        params,
-        "TOKUDB_ANALYZE_DELETE_FRACTION=%f; "
-        "TOKUDB_ANALYZE_TIME=%llu; TOKUDB_ANALYZE_THROTTLE=%llu;",
-        _delete_fraction,
-        _time_limit / tokudb::time::MICROSECONDS,
-        _throttle);
-    get_analyze_status(status);
+const char* standard_t::database() {
+    return _share->database_name();
+}
+const char* standard_t::table() {
+    return _share->table_name();
+}
+const char* standard_t::type() {
+    static const char* type = "TOKUDB_ANALYZE_MODE_STANDARD";
+    return type;
+}
+const char* standard_t::parameters() {
+    return _parameters;
+}
+const char* standard_t::status() {
+    return _status;
 }
 bool standard_t::analyze_standard_cursor_callback(
     void* extra,
@@ -501,41 +537,6 @@ bool standard_t::analyze_standard_cursor_callback(uint64_t deleted_rows) {
     _ticks += deleted_rows;
     return analyze_key_progress() != 0;
 }
-void standard_t::get_analyze_status(char* msg) {
-    static const char* scan_direction_str[] = {
-        "not scanning",
-        "scanning forward",
-        "scanning backward",
-        "scan unknown"
-    };
-
-    const char* scan_direction = NULL;
-    switch (_scan_direction) {
-        case 0: scan_direction = scan_direction_str[0]; break;
-        case DB_NEXT: scan_direction = scan_direction_str[1]; break;
-        case DB_PREV: scan_direction = scan_direction_str[2]; break;
-        default: scan_direction = scan_direction_str[3]; break;
-    }
-
-    float progress_rows = 0.0;
-    if (_share->row_count() > 0)
-        progress_rows = (float) _rows / (float) _share->row_count();
-    float progress_time = 0.0;
-    if (_time_limit > 0)
-        progress_time = (float) _key_elapsed_time / (float) _time_limit;
-    sprintf(
-        msg,
-        "analyze table standard %s.%s.%s %llu of %u %.lf%% rows %.lf%% time, "
-        "%s",
-        _share->database_name(),
-        _share->table_name(),
-        _share->_key_descriptors[_current_key]._name,
-        _current_key,
-        _share->_keys,
-        progress_rows * 100.0,
-        progress_time * 100.0,
-        scan_direction);
-}
 int standard_t::analyze_key_progress(void) {
     if (_ticks > 1000) {
         _ticks = 0;
@@ -545,19 +546,72 @@ int standard_t::analyze_key_progress(void) {
         if ((_thd && thd_killed(_thd)) || cancelled()) {
             // client killed
             return ER_ABORTING_CONNECTION;
-        } else if(_time_limit > 0 &&
-                  (uint64_t)_key_elapsed_time > _time_limit) {
+        } else if (_time_limit > 0 &&
+                   static_cast<uint64_t>(_key_elapsed_time) > _time_limit) {
             // time limit reached
             return ETIME;
         }
 
-        // report
-        if (_thd) {
-            char status[256];
-            get_analyze_status(status);
-            thd_proc_info(_thd, status);
+        // rebuild status
+        // There is a slight race condition here,
+        // _status is used here for tokudb_thd_set_proc_info and it is also used
+        // for the status column in i_s.background_job_status.
+        // If someone happens to be querying/building the i_s table
+        // at the exact same time that the status is being rebuilt here,
+        // the i_s table could get some garbage status.
+        // This solution is a little heavy handed but it works, it prevents us
+        // from changing the status while someone might be immediately observing
+        // us and it prevents someone from observing us while we change the
+        // status.
+        static const char* scan_direction_str[] = {"not scanning",
+                                                   "scanning forward",
+                                                   "scanning backward",
+                                                   "scan unknown"};
+
+        const char* scan_direction = NULL;
+        switch (_scan_direction) {
+            case 0:
+                scan_direction = scan_direction_str[0];
+                break;
+            case DB_NEXT:
+                scan_direction = scan_direction_str[1];
+                break;
+            case DB_PREV:
+                scan_direction = scan_direction_str[2];
+                break;
+            default:
+                scan_direction = scan_direction_str[3];
+                break;
         }
 
+        float progress_rows = 0.0;
+        if (_share->row_count() > 0)
+            progress_rows = static_cast<float>(_rows) /
+                            static_cast<float>(_share->row_count());
+        float progress_time = 0.0;
+        if (_time_limit > 0)
+            progress_time = static_cast<float>(_key_elapsed_time) /
+                            static_cast<float>(_time_limit);
+        tokudb::background::_job_manager->lock();
+        snprintf(
+            _status,
+            sizeof(_status),
+            "analyze table standard %s.%s.%s %llu of %u %.lf%% rows %.lf%% "
+            "time, %s",
+            _share->database_name(),
+            _share->table_name(),
+            _share->_key_descriptors[_current_key]._name,
+            _current_key,
+            _share->_keys,
+            progress_rows * 100.0,
+            progress_time * 100.0,
+            scan_direction);
+        tokudb::background::_job_manager->unlock();
+
+        // report
+        if (_thd)
+            tokudb_thd_set_proc_info(_thd, _status);
+
         // throttle
         // given the throttle value, lets calculate the maximum number of rows
         // we should have seen so far in a .1 sec resolution
@@ -575,6 +629,7 @@ int standard_t::analyze_key_progress(void) {
 int standard_t::analyze_key(uint64_t* rec_per_key_part) {
     int error = 0;
     DB* db = _share->key_file[_current_key];
+    assert_always(db != NULL);
     uint64_t num_key_parts = _share->_key_descriptors[_current_key]._parts;
     uint64_t unique_rows[num_key_parts];
     bool is_unique = _share->_key_descriptors[_current_key]._is_unique;
@@ -610,8 +665,8 @@ int standard_t::analyze_key(uint64_t* rec_per_key_part) {
                 analyze_standard_cursor_callback,
                 this);
 
-            memset(&key, 0, sizeof(DBT)); key.flags = DB_DBT_REALLOC;
-            memset(&prev_key, 0, sizeof(DBT)); prev_key.flags = DB_DBT_REALLOC;
+            memset(&key, 0, sizeof(DBT));
+            memset(&prev_key, 0, sizeof(DBT));
             copy_key = true;
         }
 
@@ -679,7 +734,6 @@ int standard_t::analyze_key(uint64_t* rec_per_key_part) {
             _key_elapsed_time >= _half_time &&
             _rows < _half_rows)) {
 
-            tokudb::memory::free(key.data); key.data = NULL;
             tokudb::memory::free(prev_key.data); prev_key.data = NULL;
             close_error = cursor->c_close(cursor);
             assert_always(close_error == 0);
@@ -688,12 +742,16 @@ int standard_t::analyze_key(uint64_t* rec_per_key_part) {
         }
     }
     // cleanup
-    if (key.data) tokudb::memory::free(key.data);
     if (prev_key.data) tokudb::memory::free(prev_key.data);
     if (cursor) close_error = cursor->c_close(cursor);
     assert_always(close_error == 0);
 
 done:
+    // in case we timed out (bunch of deleted records) without hitting a
+    // single row
+    if (_rows == 0)
+        _rows = 1;
+
     // return cardinality
     for (uint64_t i = 0; i < num_key_parts; i++) {
         rec_per_key_part[i] = _rows / unique_rows[i];
@@ -705,7 +763,7 @@ done:
 } // namespace tokudb
 
 
-int ha_tokudb::analyze(THD *thd, HA_CHECK_OPT *check_opt) {
+int ha_tokudb::analyze(THD *thd, TOKUDB_UNUSED(HA_CHECK_OPT *check_opt)) {
     TOKUDB_HANDLER_DBUG_ENTER("%s", share->table_name());
     int result = HA_ADMIN_OK;
     tokudb::sysvars::analyze_mode_t mode = tokudb::sysvars::analyze_mode(thd);
@@ -733,7 +791,6 @@ int TOKUDB_SHARE::analyze_recount_rows(THD* thd,DB_TXN* txn) {
 
     assert_always(thd != NULL);
 
-    const char *orig_proc_info = tokudb_thd_get_proc_info(thd);
     int result = HA_ADMIN_OK;
 
     tokudb::analyze::recount_rows_t* job
@@ -753,8 +810,6 @@ int TOKUDB_SHARE::analyze_recount_rows(THD* thd,DB_TXN* txn) {
         result = HA_ADMIN_FAILED;
     }
 
-    thd_proc_info(thd, orig_proc_info);
-
     TOKUDB_HANDLER_DBUG_RETURN(result);
 }
 
@@ -770,15 +825,14 @@ int TOKUDB_SHARE::analyze_standard(THD* thd, DB_TXN* txn) {
     int result = HA_ADMIN_OK;
 
     // stub out analyze if optimize is remapped to alter recreate + analyze
-    // when not auto analyze
-    if (txn &&
-        thd_sql_command(thd) != SQLCOM_ANALYZE &&
-        thd_sql_command(thd) != SQLCOM_ALTER_TABLE) {
+    // when not auto analyze or if this is an alter
+    if ((txn &&
+         thd_sql_command(thd) != SQLCOM_ANALYZE &&
+         thd_sql_command(thd) != SQLCOM_ALTER_TABLE) ||
+        thd_sql_command(thd) == SQLCOM_ALTER_TABLE) {
         TOKUDB_HANDLER_DBUG_RETURN(result);
     }
 
-    const char *orig_proc_info = tokudb_thd_get_proc_info(thd);
-
     tokudb::analyze::standard_t* job
         = new tokudb::analyze::standard_t(txn == NULL ? false : true, thd,
                                           this, txn);
@@ -807,8 +861,6 @@ int TOKUDB_SHARE::analyze_standard(THD* thd, DB_TXN* txn) {
 
     lock();
 
-    thd_proc_info(thd, orig_proc_info);
-
     TOKUDB_HANDLER_DBUG_RETURN(result);
 }
 
@@ -897,6 +949,7 @@ int ha_tokudb::do_optimize(THD* thd) {
         }
 
         DB* db = share->key_file[i];
+        assert_always(db != NULL);
         error = db->optimize(db);
         if (error) {
             goto cleanup;
@@ -935,7 +988,8 @@ cleanup:
     TOKUDB_HANDLER_DBUG_RETURN(error);
 }
 
-int ha_tokudb::optimize(THD* thd, HA_CHECK_OPT* check_opt) {
+int ha_tokudb::optimize(TOKUDB_UNUSED(THD* thd),
+                        TOKUDB_UNUSED(HA_CHECK_OPT* check_opt)) {
     TOKUDB_HANDLER_DBUG_ENTER("%s", share->table_name());
     int error;
 #if TOKU_OPTIMIZE_WITH_RECREATE
@@ -950,7 +1004,8 @@ struct check_context {
     THD* thd;
 };
 
-static int ha_tokudb_check_progress(void* extra, float progress) {
+static int ha_tokudb_check_progress(void* extra,
+                                    TOKUDB_UNUSED(float progress)) {
     struct check_context* context = (struct check_context*)extra;
     int result = 0;
     if (thd_killed(context->thd))
@@ -959,25 +1014,26 @@ static int ha_tokudb_check_progress(void* extra, float progress) {
 }
 
 static void ha_tokudb_check_info(THD* thd, TABLE* table, const char* msg) {
-    if (thd->vio_ok()) {
-        char tablename[
-            table->s->db.length + 1 +
-            table->s->table_name.length + 1];
-        snprintf(
-            tablename,
-            sizeof(tablename),
-            "%.*s.%.*s",
-            (int)table->s->db.length,
-            table->s->db.str,
-            (int)table->s->table_name.length,
-            table->s->table_name.str);
-        thd->protocol->prepare_for_resend();
-        thd->protocol->store(tablename, strlen(tablename), system_charset_info);
-        thd->protocol->store("check", 5, system_charset_info);
-        thd->protocol->store("info", 4, system_charset_info);
-        thd->protocol->store(msg, strlen(msg), system_charset_info);
-        thd->protocol->write();
-    }
+    DBUG_ASSERT(thd->active_vio);
+
+    char tablename[
+        table->s->db.length + 1 +
+        table->s->table_name.length + 1];
+    snprintf(
+        tablename,
+        sizeof(tablename),
+        "%.*s.%.*s",
+        (int)table->s->db.length,
+        table->s->db.str,
+        (int)table->s->table_name.length,
+        table->s->table_name.str);
+    thd->get_protocol()->start_row();
+    thd->get_protocol()->store(tablename, strlen(tablename),
+                               system_charset_info);
+    thd->get_protocol()->store("check", 5, system_charset_info);
+    thd->get_protocol()->store("info", 4, system_charset_info);
+    thd->get_protocol()->store(msg, strlen(msg), system_charset_info);
+    thd->get_protocol()->end_row();
 }
 
 int ha_tokudb::check(THD* thd, HA_CHECK_OPT* check_opt) {
@@ -1016,7 +1072,8 @@ int ha_tokudb::check(THD* thd, HA_CHECK_OPT* check_opt) {
                 write_status_msg);
         }
         for (uint i = 0; i < num_DBs; i++) {
-            DB *db = share->key_file[i];
+            DB* db = share->key_file[i];
+            assert_always(db != NULL);
             const char* kname =
                 i == primary_key ? "primary" : table_share->key_info[i].name;
             snprintf(
