@@ -1,13 +1,20 @@
 /* Copyright (c) 2016, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -235,7 +242,7 @@ namespace keyring__api_unittest
     EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key,
                               &key_len), 0);
     EXPECT_STREQ("AES", key_type);
-    std::string key_data_with_version= "0:" + sample_key_data;
+    std::string key_data_with_version= "1:" + sample_key_data;
     EXPECT_EQ(key_len, key_data_with_version.length() + 1);
     ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key), key_data_with_version.c_str(), key_len) == 0);
     my_free(key_type);
@@ -259,18 +266,7 @@ namespace keyring__api_unittest
 
     char *key_type= NULL;
     size_t key_len= 0;
-    void *key_ver0= NULL;
-    EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key_ver0,
-                              &key_len), 0);
-    EXPECT_STREQ("AES", key_type);
-    EXPECT_EQ(key_len, static_cast<size_t>(18));
-    ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key_ver0), "0:", 2) == 0);
-    my_free(key_type);
-    key_type= NULL;
-
     void *key_ver1= NULL;
-    EXPECT_EQ(mysql_key_generate("percona_binlog", "AES", NULL, 16), 0);
-
     EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key_ver1,
                               &key_len), 0);
     EXPECT_STREQ("AES", key_type);
@@ -279,11 +275,22 @@ namespace keyring__api_unittest
     my_free(key_type);
     key_type= NULL;
 
+    void *key_ver2= NULL;
+    EXPECT_EQ(mysql_key_generate("percona_binlog", "AES", NULL, 16), 0);
+
+    EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key_ver2,
+                              &key_len), 0);
+    EXPECT_STREQ("AES", key_type);
+    EXPECT_EQ(key_len, static_cast<size_t>(18));
+    ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key_ver2), "2:", 2) == 0);
+    my_free(key_type);
+    key_type= NULL;
+
     // make sure that rotated key is different than the original one
-    ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key_ver0) + 2, reinterpret_cast<char*>(key_ver1) + 2, 16) != 0);
+    ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key_ver1) + 2, reinterpret_cast<char*>(key_ver2) + 2, 16) != 0);
 
-    my_free(key_ver0);
     my_free(key_ver1);
+    my_free(key_ver2);
   }
 
   TEST_F(Keyring_api_test, GeneratePBRotatePBFetchFirstVersionFetchLatestPB)
@@ -294,69 +301,70 @@ namespace keyring__api_unittest
 
     char *key_type= NULL;
     size_t key_len= 0;
-    void *key_ver0= NULL;
+    void *key_ver1= NULL;
 
-    EXPECT_EQ(mysql_key_fetch("percona_binlog:0", &key_type, NULL, &key_ver0,
+    EXPECT_EQ(mysql_key_fetch("percona_binlog:1", &key_type, NULL, &key_ver1,
                               &key_len), 0);
     EXPECT_STREQ("AES", key_type);
     EXPECT_EQ(key_len, static_cast<size_t>(16));
     my_free(key_type);
     key_type= NULL;
 
-    void *key_ver1= NULL;
+    void *key_ver2= NULL;
 
-    EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key_ver1,
+    EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key_ver2,
                               &key_len), 0);
     EXPECT_STREQ("AES", key_type);
     EXPECT_EQ(key_len, static_cast<size_t>(18));
-    ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key_ver1), "1:", 2) == 0);
+    ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key_ver2), "2:", 2) == 0);
     my_free(key_type);
     key_type= NULL;
 
     // make sure that rotated key is different than the original one
-    ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key_ver0) + 2, reinterpret_cast<char*>(key_ver1) + 2, 16) != 0);
+    // + 2 to skip key version of retrieved latest percona_binlog key
+    ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key_ver1), reinterpret_cast<char*>(key_ver2) + 2, 16) != 0);
 
-    my_free(key_ver0);
     my_free(key_ver1);
+    my_free(key_ver2);
   }
 
   TEST_F(Keyring_api_test, StorePBRotatePBFetchFirstVersionFetchLatestPB)
   {
-    std::string percona_binlog_key_ver0_data("key_ver0");
-
-    EXPECT_EQ(mysql_key_store("percona_binlog", "AES", NULL, percona_binlog_key_ver0_data.c_str(),
-                              percona_binlog_key_ver0_data.length() + 1), 0);
-
     std::string percona_binlog_key_ver1_data("key_ver1");
 
     EXPECT_EQ(mysql_key_store("percona_binlog", "AES", NULL, percona_binlog_key_ver1_data.c_str(),
                               percona_binlog_key_ver1_data.length() + 1), 0);
 
+    std::string percona_binlog_key_ver2_data("key_ver2");
+
+    EXPECT_EQ(mysql_key_store("percona_binlog", "AES", NULL, percona_binlog_key_ver2_data.c_str(),
+                              percona_binlog_key_ver2_data.length() + 1), 0);
+
     char *key_type= NULL;
     size_t key_len= 0;
-    void *key_ver0= NULL;
-
-    EXPECT_EQ(mysql_key_fetch("percona_binlog:0", &key_type, NULL, &key_ver0,
-                              &key_len), 0);
-    EXPECT_STREQ("AES", key_type);
-    EXPECT_EQ(key_len, percona_binlog_key_ver0_data.length() + 1);
-    ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key_ver0), percona_binlog_key_ver0_data.c_str(), key_len) == 0);
-    my_free(key_type);
-    key_type= NULL;
-
     void *key_ver1= NULL;
 
-    EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key_ver1,
+    EXPECT_EQ(mysql_key_fetch("percona_binlog:1", &key_type, NULL, &key_ver1,
                               &key_len), 0);
     EXPECT_STREQ("AES", key_type);
-    EXPECT_EQ(key_len, percona_binlog_key_ver1_data.length() + 3);
-    std::string expected_percona_binlog_key_ver1_data_with_verion= "1:" + percona_binlog_key_ver1_data;
-    ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key_ver1), expected_percona_binlog_key_ver1_data_with_verion.c_str(), key_len) == 0);
+    EXPECT_EQ(key_len, percona_binlog_key_ver1_data.length() + 1);
+    ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key_ver1), percona_binlog_key_ver1_data.c_str(), key_len) == 0);
+    my_free(key_type);
+    key_type= NULL;
+
+    void *key_ver2= NULL;
+
+    EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key_ver2,
+                              &key_len), 0);
+    EXPECT_STREQ("AES", key_type);
+    EXPECT_EQ(key_len, percona_binlog_key_ver2_data.length() + 3);
+    std::string expected_percona_binlog_key_ver2_data_with_verion= "2:" + percona_binlog_key_ver2_data;
+    ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key_ver2), expected_percona_binlog_key_ver2_data_with_verion.c_str(), key_len) == 0);
     my_free(key_type);
     key_type= NULL;
 
-    my_free(key_ver0);
     my_free(key_ver1);
+    my_free(key_ver2);
   }
 
   TEST_F(Keyring_api_test, RotatePBStoreSKFetchPBRotatePBFetchPBRotatePBRotateSKFetchPBFetchSK)
@@ -377,7 +385,7 @@ namespace keyring__api_unittest
     EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key,
                               &key_len), 0);
     EXPECT_STREQ("AES", key_type);
-    std::string key_data_with_version= "0:" + percona_binlog_key_data_1;
+    std::string key_data_with_version= "1:" + percona_binlog_key_data_1;
     EXPECT_EQ(key_len, key_data_with_version.length() + 1);
     ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key), key_data_with_version.c_str(), key_len) == 0);
     my_free(key_type);
@@ -393,7 +401,7 @@ namespace keyring__api_unittest
     EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key,
                               &key_len), 0);
     EXPECT_STREQ("AES", key_type);
-    key_data_with_version= "1:" + percona_binlog_key_data_2;
+    key_data_with_version= "2:" + percona_binlog_key_data_2;
     EXPECT_EQ(key_len, key_data_with_version.length() + 1);
     ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key), key_data_with_version.c_str(), key_len) == 0);
     my_free(key_type);
@@ -414,7 +422,7 @@ namespace keyring__api_unittest
     EXPECT_EQ(mysql_key_fetch("percona_binlog", &key_type, NULL, &key,
                               &key_len), 0);
     EXPECT_STREQ("AES", key_type);
-    key_data_with_version= "2:" + percona_binlog_key_data_3;
+    key_data_with_version= "3:" + percona_binlog_key_data_3;
     EXPECT_EQ(key_len, key_data_with_version.length() + 1);
     ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key), key_data_with_version.c_str(), key_len) == 0);
     my_free(key_type);
@@ -425,7 +433,7 @@ namespace keyring__api_unittest
     EXPECT_EQ(mysql_key_fetch("percona_sk", &key_type, NULL, &key,
                               &key_len), 0);
     EXPECT_STREQ("AES", key_type);
-    key_data_with_version= "1:" + percona_sk_data_2;
+    key_data_with_version= "2:" + percona_sk_data_2;
     EXPECT_EQ(key_len, key_data_with_version.length() + 1);
     ASSERT_TRUE(memcmp(reinterpret_cast<char*>(key), key_data_with_version.c_str(), key_len) == 0);
     my_free(key_type);
