@@ -1,13 +1,25 @@
-/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
+
+   Without limiting anything contained in the foregoing, this file,
+   which is part of C Driver for MySQL (Connector/C), is also subject to the
+   Universal FOSS Exception, version 1.0, a copy of which can be found at
+   http://oss.oracle.com/licenses/universal-foss-exception.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -22,6 +34,36 @@
 
 #include "vio_priv.h"
 
+#ifdef HAVE_OPENSSL
+PSI_memory_key key_memory_vio_ssl_fd;
+#endif
+
+PSI_memory_key key_memory_vio;
+PSI_memory_key key_memory_vio_read_buffer;
+PSI_memory_key key_memory_vio_proxy_networks;
+
+#ifdef HAVE_PSI_INTERFACE
+static PSI_memory_info all_vio_memory[]=
+{
+#ifdef HAVE_OPENSSL
+  {&key_memory_vio_ssl_fd, "ssl_fd", 0},
+#endif
+
+  {&key_memory_vio, "vio", 0},
+  {&key_memory_vio_read_buffer, "read_buffer", 0},
+  {&key_memory_vio_proxy_networks, "proxy_networks", 0},
+};
+
+void init_vio_psi_keys()
+{
+  const char* category= "vio";
+  int count;
+
+  count= array_elements(all_vio_memory);
+  mysql_memory_register(category, all_vio_memory, count);
+}
+#endif
+
 #ifdef _WIN32
 
 /**
@@ -37,16 +79,16 @@
   @retval 1       The requested I/O event has occurred.
 */
 
-static int no_io_wait(Vio *vio __attribute__((unused)),
-                      enum enum_vio_io_event event __attribute__((unused)),
-                      int timeout __attribute__((unused)))
+static int no_io_wait(Vio *vio MY_ATTRIBUTE((unused)),
+                      enum enum_vio_io_event event MY_ATTRIBUTE((unused)),
+                      int timeout MY_ATTRIBUTE((unused)))
 {
   return 1;
 }
 
 #endif
 
-static my_bool has_no_data(Vio *vio __attribute__((unused)))
+static my_bool has_no_data(Vio *vio MY_ATTRIBUTE((unused)))
 {
   return FALSE;
 }
@@ -68,9 +110,6 @@ static void vio_init(Vio *vio, enum enum_vio_type type,
   DBUG_ENTER("vio_init");
   DBUG_PRINT("enter", ("type: %d  sd: %d  flags: %d", type, sd, flags));
 
-#ifndef HAVE_VIO_READ_BUFF
-  flags&= ~VIO_BUFFERED_READ;
-#endif
   memset(vio, 0, sizeof(*vio));
   vio->type= type;
   vio->mysql_socket= MYSQL_INVALID_SOCKET;
@@ -78,7 +117,8 @@ static void vio_init(Vio *vio, enum enum_vio_type type,
   vio->localhost= flags & VIO_LOCALHOST;
   vio->read_timeout= vio->write_timeout= -1;
   if ((flags & VIO_BUFFERED_READ) &&
-      !(vio->read_buffer= (char*)my_malloc(VIO_READ_BUFFER_SIZE, MYF(MY_WME))))
+      !(vio->read_buffer= (char*)my_malloc(key_memory_vio_read_buffer,
+                                           VIO_READ_BUFFER_SIZE, MYF(MY_WME))))
     flags&= ~VIO_BUFFERED_READ;
 #ifdef _WIN32
   if (type == VIO_TYPE_NAMEDPIPE)
@@ -99,8 +139,7 @@ static void vio_init(Vio *vio, enum enum_vio_type type,
     vio->has_data       =has_no_data;
     DBUG_VOID_RETURN;
   }
-#endif
-#ifdef HAVE_SMEM
+#ifndef EMBEDDED_LIBRARY
   if (type == VIO_TYPE_SHARED_MEMORY)
   {
     vio->viodelete	=vio_delete_shared_memory;
@@ -119,7 +158,8 @@ static void vio_init(Vio *vio, enum enum_vio_type type,
     vio->has_data       =vio_shared_memory_has_data;
     DBUG_VOID_RETURN;
   }
-#endif
+#endif /* !EMBEDDED_LIBRARY */
+#endif /* _WIN32 */
 #ifdef HAVE_OPENSSL
   if (type == VIO_TYPE_SSL)
   {
@@ -188,7 +228,7 @@ static void vio_init(Vio *vio, enum enum_vio_type type,
 */
 
 my_bool vio_reset(Vio* vio, enum enum_vio_type type,
-                  my_socket sd, void *ssl __attribute__((unused)), uint flags)
+                  my_socket sd, void *ssl MY_ATTRIBUTE((unused)), uint flags)
 {
   int ret= FALSE;
   Vio new_vio;
@@ -256,7 +296,8 @@ Vio *mysql_socket_vio_new(MYSQL_SOCKET mysql_socket, enum enum_vio_type type, ui
   my_socket sd= mysql_socket_getfd(mysql_socket);
   DBUG_ENTER("mysql_socket_vio_new");
   DBUG_PRINT("enter", ("sd: %d", sd));
-  if ((vio = (Vio*) my_malloc(sizeof(*vio),MYF(MY_WME))))
+  if ((vio = (Vio*) my_malloc(key_memory_vio,
+                              sizeof(*vio),MYF(MY_WME))))
   {
     vio_init(vio, type, sd, flags);
     vio->mysql_socket= mysql_socket;
@@ -285,7 +326,8 @@ Vio *vio_new_win32pipe(HANDLE hPipe)
 {
   Vio *vio;
   DBUG_ENTER("vio_new_handle");
-  if ((vio = (Vio*) my_malloc(sizeof(Vio),MYF(MY_WME))))
+  if ((vio = (Vio*) my_malloc(key_memory_vio,
+                              sizeof(Vio),MYF(MY_WME))))
   {
     vio_init(vio, VIO_TYPE_NAMEDPIPE, 0, VIO_LOCALHOST);
     /* Create an object for event notification. */
@@ -296,12 +338,12 @@ Vio *vio_new_win32pipe(HANDLE hPipe)
       DBUG_RETURN(NULL);
     }
     vio->hPipe= hPipe;
-    strmov(vio->desc, "named pipe");
+    my_stpcpy(vio->desc, "named pipe");
   }
   DBUG_RETURN(vio);
 }
 
-#ifdef HAVE_SMEM
+#ifndef EMBEDDED_LIBRARY
 Vio *vio_new_win32shared_memory(HANDLE handle_file_map, HANDLE handle_map,
                                 HANDLE event_server_wrote, HANDLE event_server_read,
                                 HANDLE event_client_wrote, HANDLE event_client_read,
@@ -309,7 +351,8 @@ Vio *vio_new_win32shared_memory(HANDLE handle_file_map, HANDLE handle_map,
 {
   Vio *vio;
   DBUG_ENTER("vio_new_win32shared_memory");
-  if ((vio = (Vio*) my_malloc(sizeof(Vio),MYF(MY_WME))))
+  if ((vio = (Vio*) my_malloc(key_memory_vio,
+                              sizeof(Vio),MYF(MY_WME))))
   {
     vio_init(vio, VIO_TYPE_SHARED_MEMORY, 0, VIO_LOCALHOST);
     vio->handle_file_map= handle_file_map;
@@ -321,7 +364,7 @@ Vio *vio_new_win32shared_memory(HANDLE handle_file_map, HANDLE handle_map,
     vio->event_conn_closed= event_conn_closed;
     vio->shared_memory_remain= 0;
     vio->shared_memory_pos= handle_map;
-    strmov(vio->desc, "shared memory");
+    my_stpcpy(vio->desc, "shared memory");
   }
   DBUG_RETURN(vio);
 }
@@ -391,14 +434,50 @@ void vio_delete(Vio* vio)
 */
 void vio_end(void)
 {
-#if defined(HAVE_YASSL)
-  yaSSL_CleanUp();
-#elif defined(HAVE_OPENSSL)
-  // This one is needed on the client side
-  ERR_remove_state(0);
-  ERR_free_strings();
-  EVP_cleanup();
-  CRYPTO_cleanup_all_ex_data();
-  sk_SSL_COMP_free(SSL_COMP_get_compression_methods());
+#if defined(HAVE_OPENSSL)
+  vio_ssl_end();
 #endif
+  vio_proxy_cleanup();
 }
+
+struct vio_string
+{
+  const char * m_str;
+  int m_len;
+};
+typedef struct vio_string vio_string;
+
+/**
+  Names for each VIO TYPE.
+  Indexed by enum_vio_type.
+  If you add more, please update audit_log.cc
+*/
+static const vio_string vio_type_names[] =
+{
+  { "", 0},
+  { C_STRING_WITH_LEN("TCP/IP") },
+  { C_STRING_WITH_LEN("Socket") },
+  { C_STRING_WITH_LEN("Named Pipe") },
+  { C_STRING_WITH_LEN("SSL/TLS") },
+  { C_STRING_WITH_LEN("Shared Memory") },
+  { C_STRING_WITH_LEN("Internal") },
+  { C_STRING_WITH_LEN("Plugin") }
+};
+
+void get_vio_type_name(enum enum_vio_type vio_type, const char ** str, int * len)
+{
+  int index;
+
+  if ((vio_type >= FIRST_VIO_TYPE) && (vio_type <= LAST_VIO_TYPE))
+  {
+    index= vio_type;
+  }
+  else
+  {
+    index= 0;
+  }
+  *str= vio_type_names[index].m_str;
+  *len= vio_type_names[index].m_len;
+  return;
+}
+
