@@ -1,15 +1,26 @@
 /*
-   Copyright (c) 2001, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; version 2 of
-   the License.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
+
+   Without limiting anything contained in the foregoing, this file,
+   which is part of C Driver for MySQL (Connector/C), is also subject to the
+   Universal FOSS Exception, version 1.0, a copy of which can be found at
+   http://oss.oracle.com/licenses/universal-foss-exception.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-   GNU General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -28,6 +39,18 @@
 #ifdef FIONREAD_IN_SYS_FILIO
 # include <sys/filio.h>
 #endif
+#ifndef _WIN32
+# include <netinet/tcp.h>
+#endif
+#ifdef HAVE_POLL_H
+# include <poll.h>
+#endif
+#ifdef HAVE_SYS_IOCTL_H
+# include <sys/ioctl.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
+# include <arpa/inet.h>
+#endif
 
 /* Network io wait callbacks  for threadpool */
 static void (*before_io_wait)(void)= 0;
@@ -65,7 +88,7 @@ void vio_set_wait_callback(void (*before_wait)(void),
 static struct st_vio_network *vio_pp_networks= NULL;
 static size_t vio_pp_networks_nb= 0;
 
-int vio_errno(Vio *vio __attribute__((unused)))
+int vio_errno(Vio *vio MY_ATTRIBUTE((unused)))
 {
   /* These transport types are not Winsock based. */
 #ifdef _WIN32
@@ -327,8 +350,8 @@ static int vio_set_blocking(Vio *vio, my_bool status)
 
 
 int vio_socket_timeout(Vio *vio,
-                       uint which __attribute__((unused)),
-                       my_bool old_mode __attribute__((unused)))
+                       uint which MY_ATTRIBUTE((unused)),
+                       my_bool old_mode MY_ATTRIBUTE((unused)))
 {
   int ret= 0;
   DBUG_ENTER("vio_socket_timeout");
@@ -394,7 +417,7 @@ int vio_socket_timeout(Vio *vio,
 }
 
 
-int vio_fastsend(Vio * vio __attribute__((unused)))
+int vio_fastsend(Vio * vio MY_ATTRIBUTE((unused)))
 {
   int r=0;
   DBUG_ENTER("vio_fastsend");
@@ -408,7 +431,7 @@ int vio_fastsend(Vio * vio __attribute__((unused)))
 #endif                                    /* IPTOS_THROUGHPUT */
   if (!r)
   {
-#ifdef __WIN__
+#ifdef _WIN32
     BOOL nodelay= 1;
 #else
     int nodelay = 1;
@@ -488,8 +511,10 @@ int vio_shutdown(Vio * vio, int how)
   r= vio_cancel(vio, how);
 
   if (vio->inactive == FALSE)
+  {
     if (mysql_socket_close(vio->mysql_socket))
       r= -1;
+  }
 
   if (r)
   {
@@ -574,9 +599,9 @@ my_socket vio_fd(Vio* vio)
   @param dst_length [out] actual length of the normalized IP address.
 */
 static void vio_get_normalized_ip(const struct sockaddr *src,
-                                  int src_length,
+                                  size_t src_length,
                                   struct sockaddr *dst,
-                                  int *dst_length)
+                                  size_t *dst_length)
 {
   switch (src->sa_family) {
   case AF_INET:
@@ -650,13 +675,13 @@ static void vio_get_normalized_ip(const struct sockaddr *src,
 */
 
 my_bool vio_get_normalized_ip_string(const struct sockaddr *addr,
-                                     int addr_length,
+                                     size_t addr_length,
                                      char *ip_string,
                                      size_t ip_string_size)
 {
   struct sockaddr_storage norm_addr_storage;
   struct sockaddr *norm_addr= (struct sockaddr *) &norm_addr_storage;
-  int norm_addr_length;
+  size_t norm_addr_length;
   int err_code;
 
   vio_get_normalized_ip(addr, addr_length, norm_addr, &norm_addr_length);
@@ -681,12 +706,17 @@ void vio_proxy_protocol_add(const struct st_vio_network *net)
   is not particularly efficient, but this is done once per server startup with
   relatively few allowed networks. */
   vio_pp_networks_nb++;
-  vio_pp_networks= my_realloc(vio_pp_networks,
+  vio_pp_networks= my_realloc(key_memory_vio_proxy_networks, vio_pp_networks,
                               vio_pp_networks_nb * sizeof(*net),
                               MYF(MY_ALLOW_ZERO_PTR | MY_FAE | MY_WME));
   memcpy(&vio_pp_networks[vio_pp_networks_nb - 1], net, sizeof(*net));
 }
 
+void vio_proxy_cleanup()
+{
+  my_free(vio_pp_networks);
+}
+
 /* Check whether a connection from this source address must provide the proxy
 protocol header */
 static my_bool vio_client_must_be_proxied(const struct sockaddr *addr)
@@ -723,7 +753,7 @@ static my_bool vio_client_must_be_proxied(const struct sockaddr *addr)
 
 /* Process the proxy protocol header. Return true on an error. */
 static my_bool vio_process_proxy_header(int socket_fd, struct sockaddr *addr,
-                                        size_socket *addr_length)
+                                        socket_len_t *addr_length)
 {
   /* The ip source network matches an expected proxy protocol network. */
   static const char v2sig[12]=
@@ -743,17 +773,17 @@ static my_bool vio_process_proxy_header(int socket_fd, struct sockaddr *addr,
           uint32_t dst_addr;
           uint16_t src_port;
           uint16_t dst_port;
-        } __attribute__((packed)) ip4;
+        } MY_ATTRIBUTE((packed)) ip4;
 #ifdef HAVE_IPV6
         struct {  /* for TCP/UDP over IPv6, len = 36 */
           uint8_t  src_addr[16];
           uint8_t  dst_addr[16];
           uint16_t src_port;
           uint16_t dst_port;
-        } __attribute__((packed)) ip6;
+        } MY_ATTRIBUTE((packed)) ip6;
 #endif
       } addr;
-    } __attribute__((packed)) v2;
+    } MY_ATTRIBUTE((packed)) v2;
   } hdr;
 
   int size;
@@ -960,7 +990,7 @@ my_bool vio_peer_addr(Vio *vio, char *ip_buffer, uint16 *port,
 
     /* Initialize ip_buffer and port. */
 
-    strmov(ip_buffer, "127.0.0.1");
+    my_stpcpy(ip_buffer, "127.0.0.1");
     *port= 0;
   }
   else
@@ -970,7 +1000,7 @@ my_bool vio_peer_addr(Vio *vio, char *ip_buffer, uint16 *port,
 
     struct sockaddr_storage addr_storage;
     struct sockaddr *addr= (struct sockaddr *) &addr_storage;
-    size_socket addr_length= sizeof (addr_storage);
+    socket_len_t addr_length= sizeof (addr_storage);
 
     /* Get sockaddr by socked fd. */
 
@@ -1101,12 +1131,19 @@ static my_bool socket_peek_read(Vio *vio, uint *bytes)
 int vio_io_wait(Vio *vio, enum enum_vio_io_event event, int timeout)
 {
   int ret;
-  short revents __attribute__((unused)) = 0;
+#ifndef DBUG_OFF
+  short revents= 0;
+#endif
   struct pollfd pfd;
   my_socket sd= mysql_socket_getfd(vio->mysql_socket);
   MYSQL_SOCKET_WAIT_VARIABLES(locker, state) /* no ';' */
   DBUG_ENTER("vio_io_wait");
 
+#ifdef WITH_WSREP
+  if (sd == INVALID_SOCKET)
+    DBUG_RETURN(-1);
+#endif /* WITH_WSREP */
+
   memset(&pfd, 0, sizeof(pfd));
 
   pfd.fd= sd;
@@ -1119,12 +1156,16 @@ int vio_io_wait(Vio *vio, enum enum_vio_io_event event, int timeout)
   {
   case VIO_IO_EVENT_READ:
     pfd.events= MY_POLL_SET_IN;
+#ifndef DBUG_OFF
     revents= MY_POLL_SET_IN | MY_POLL_SET_ERR | POLLRDHUP;
+#endif
     break;
   case VIO_IO_EVENT_WRITE:
   case VIO_IO_EVENT_CONNECT:
     pfd.events= MY_POLL_SET_OUT;
+#ifndef DBUG_OFF
     revents= MY_POLL_SET_OUT | MY_POLL_SET_ERR;
+#endif
     break;
   }
 
@@ -1172,6 +1213,11 @@ int vio_io_wait(Vio *vio, enum enum_vio_io_event event, int timeout)
   if (fd == INVALID_SOCKET)
     DBUG_RETURN(-1);
 
+#ifdef __APPLE__
+  if (fd >= FD_SETSIZE)
+    DBUG_RETURN(-1);
+#endif
+
   /* Convert the timeout, in milliseconds, to seconds and microseconds. */
   if (timeout >= 0)
   {
@@ -1203,7 +1249,7 @@ int vio_io_wait(Vio *vio, enum enum_vio_io_event event, int timeout)
                     PSI_SOCKET_SELECT, timeout);
 
   /* The first argument is ignored on Windows. */
-  ret= select(fd + 1, &readfds, &writefds, &exceptfds, 
+  ret= select((int)(fd + 1), &readfds, &writefds, &exceptfds, 
               (timeout >= 0) ? &tm : NULL);
 
   END_SOCKET_WAIT(locker, timeout);
@@ -1408,11 +1454,6 @@ ssize_t vio_pending(Vio *vio)
       return -1;
   }
 
-  /*
-    SSL not checked due to a yaSSL bug in SSL_pending that
-    causes it to attempt to read from the socket.
-  */
-
   return (ssize_t) bytes;
 }
 
